@@ -1,4 +1,4 @@
-# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -280,8 +280,8 @@ class CodeGenerator(object):
 
         for l in self.lines:
 
-            l = re.sub(ur'_\((\".*?\")\)', replace, l)
-            l = re.sub(ur'_\((\'.*?\')\)', replace, l)
+            l = re.sub(r'_\((\".*?\")\)', replace, l)
+            l = re.sub(r'_\((\'.*?\')\)', replace, l)
 
             lines.append(l)
 
@@ -323,6 +323,9 @@ class CodeGenerator(object):
 
                     m = re.match(r'## ?([ *]*)(.*)', s)
 
+                    if m is None:
+                        raise Exception("Comment translation doesn't start with '## ': {}".format(s))
+
                     prefix = m.group(1)
                     empty = ' ' * len(prefix)
                     rest = m.group(2)
@@ -353,7 +356,7 @@ class CodeGenerator(object):
     def copy_files(self):
 
         for src, dst in language_copies[self.p.language]:
-            src = os.path.join(renpy.config.gamedir, src)
+            src = os.path.join(renpy.config.renpy_base, src)
             dst = os.path.join(self.p.prefix, dst)
 
             if os.path.exists(dst):
@@ -372,7 +375,7 @@ class CodeGenerator(object):
         if os.path.exists(dst):
             return
 
-        language = renpy.store._preferences.language  # @UndefinedVariable
+        language = renpy.store._preferences.language # @UndefinedVariable
 
         if language is None:
             language = "None"
