@@ -8,9 +8,10 @@
 #include <SDL.h>
 #include <SDL_thread.h>
 
-#if defined(__arm__) && !(__MACOS__ || __IPHONEOS__)
-#define USE_MEMALIGN
 #include <stdlib.h>
+
+#ifndef _WIN32
+#define USE_POSIX_MEMALIGN
 #endif
 
 /* Should a mono channel be split into two equal stero channels (true) or
@@ -34,14 +35,20 @@ const int BPS = 4; // Bytes per sample.
 
 const int FRAMES = 3;
 
+// The alignment of each row of pixels.
+const int ROW_ALIGNMENT = 16;
+
 // The number of pixels on each side. This has to be greater that 0 (since
 // Ren'Py needs some padding), FRAME_PADDING * BPS has to be a multiple of
 // 16 (alignment issues on ARM NEON), and has to match the crop in the
 // read_video function of renpysound.pyx.
-const int FRAME_PADDING = 4;
+const int FRAME_PADDING = ROW_ALIGNMENT / 4;
 
 const int SPEED = 1;
 
+// How many seconds early can frames be delivered?
+static const double frame_early_delivery = .005;
+
 static SDL_Surface *rgb_surface = NULL;
 static SDL_Surface *rgba_surface = NULL;
 
@@ -84,6 +91,9 @@ static int64_t rwops_seek(void *opaque, int64_t offset, int whence) {
 static AVIOContext *rwops_open(SDL_RWops *rw) {
 
     unsigned char *buffer = av_malloc(RWOPS_BUFFER);
+	if (buffer == NULL) {
+		return NULL;
+	}
     AVIOContext *rv = avio_alloc_context(
         buffer,
         RWOPS_BUFFER,
@@ -92,6 +102,10 @@ static AVIOContext *rwops_open(SDL_RWops *rw) {
         rwops_read,
         rwops_write,
         rwops_seek);
+    if (rv == NULL) {
+    	av_free(buffer);
+    	return NULL;
+    }
 
     return rv;
 }
@@ -134,12 +148,16 @@ typedef struct SurfaceQueueEntry {
 
 typedef struct MediaState {
 
+    /* The next entry in a list of MediaStates */
+    struct MediaState *next;
+
+    /* The thread associated with decoding this media. */
+    SDL_Thread *thread;
 
 	/* The condition and lock. */
 	SDL_cond* cond;
 	SDL_mutex* lock;
 
-
 	SDL_RWops *rwops;
 	char *filename;
 
@@ -148,12 +166,6 @@ typedef struct MediaState {
 	 */
 	int want_video;
 
-	/*
-	 * This becomes true when the decode thread starts, when
-	 * it is the decode thread's job to deallocate this object.
-	 */
-	int started;
-
 	/* This becomes true once the decode thread has finished initializing
 	 * and the readers and writers can do their thing.
 	 */
@@ -250,37 +262,68 @@ typedef struct MediaState {
 	/* Are frame drops allowed? */
 	int frame_drops;
 
+	/* The time the pause happened, or 0 if we're not paused. */
+	double pause_time;
+
+	/* The offset between now and the time of the current frame, at least for video. */
+	double time_offset;
+
+
 } MediaState;
 
 static AVFrame *dequeue_frame(FrameQueue *fq);
 static void free_packet_queue(PacketQueue *pq);
 static SurfaceQueueEntry *dequeue_surface(SurfaceQueueEntry **queue);
 
-static void deallocate(MediaState *ms) {
-	/* Destroy video stuff. */
 
-	while (1) {
+/* A queue of MediaState objects that are awaiting deallocation.*/
+static MediaState *deallocate_queue = NULL;
+
+/* A mutex that discards deallocate_queue. */
+SDL_mutex *deallocate_mutex = NULL;
+
+/* Deallocates a single MediaState. */
+static void deallocate(MediaState *ms) {
+
+    while (1) {
 		SurfaceQueueEntry *sqe = dequeue_surface(&ms->surface_queue);
 
 		if (! sqe) {
 			break;
 		}
 
-		SDL_free(sqe->pixels);
+		if (sqe->pixels) {
+#ifndef USE_POSIX_MEMALIGN
+			SDL_free(sqe->pixels);
+#else
+			free(sqe->pixels);
+#endif
+		}
 		av_free(sqe);
 	}
 
-	sws_freeContext(ms->sws);
+	if (ms->sws) {
+		sws_freeContext(ms->sws);
+	}
 
-	av_frame_free(&ms->video_decode_frame);
+	if (ms->video_decode_frame) {
+		av_frame_free(&ms->video_decode_frame);
+	}
 
 	av_packet_unref(&ms->video_pkt);
 
 	/* Destroy audio stuff. */
-	swr_free(&ms->swr);
+	if (ms->swr) {
+		swr_free(&ms->swr);
+	}
 
-	av_frame_free(&ms->audio_decode_frame);
-	av_frame_free(&ms->audio_out_frame);
+	if (ms->audio_decode_frame) {
+		av_frame_free(&ms->audio_decode_frame);
+	}
+
+	if (ms->audio_out_frame) {
+		av_frame_free(&ms->audio_out_frame);
+	}
 
 	while (1) {
 		AVFrame *f = dequeue_frame(&ms->audio_queue);
@@ -296,30 +339,69 @@ static void deallocate(MediaState *ms) {
 	free_packet_queue(&ms->audio_packet_queue);
 	free_packet_queue(&ms->video_packet_queue);
 
-	avcodec_free_context(&ms->video_context);
-	avcodec_free_context(&ms->audio_context);
+	if (ms->video_context) {
+		avcodec_free_context(&ms->video_context);
+	}
+	if (ms->audio_context) {
+		avcodec_free_context(&ms->audio_context);
+	}
 
 	if (ms->ctx) {
-		for (int i = 0; i < ms->ctx->nb_streams; i++) {
-			avcodec_close(ms->ctx->streams[i]->codec);
+
+		if (ms->ctx->pb) {
+			if (ms->ctx->pb->buffer) {
+				av_freep(&ms->ctx->pb->buffer);
+			}
+			av_freep(&ms->ctx->pb);
 		}
-	}
 
-	if (ms->ctx->pb) {
-		av_freep(&ms->ctx->pb->buffer);
-		av_freep(&ms->ctx->pb);
+		avformat_close_input(&ms->ctx);
+		avformat_free_context(ms->ctx);
 	}
 
-	avformat_close_input(&ms->ctx);
-
 	/* Destroy alloc stuff. */
-	SDL_DestroyCond(ms->cond);
-	SDL_DestroyMutex(ms->lock);
+	if (ms->cond) {
+		SDL_DestroyCond(ms->cond);
+	}
+	if (ms->lock) {
+		SDL_DestroyMutex(ms->lock);
+	}
 
-	rwops_close(ms->rwops);
+	if (ms->rwops) {
+		rwops_close(ms->rwops);
+	}
 
-	av_free(ms->filename);
-	av_free(ms);
+	if (ms->filename) {
+		av_free(ms->filename);
+	}
+
+	/* Add this MediaState to a queue to have its thread ended, and the MediaState
+	 * deactivated.
+	 */
+	SDL_LockMutex(deallocate_mutex);
+    ms->next = deallocate_queue;
+    deallocate_queue = ms;
+    SDL_UnlockMutex(deallocate_mutex);
+
+}
+
+/* Perform the portion of deallocation that's been deferred to the main thread. */
+static void deallocate_deferred() {
+
+    SDL_LockMutex(deallocate_mutex);
+
+    while (deallocate_queue) {
+        MediaState *ms = deallocate_queue;
+        deallocate_queue = ms->next;
+
+        if (ms->thread) {
+            SDL_WaitThread(ms->thread, NULL);
+        }
+
+        av_free(ms);
+    }
+
+    SDL_UnlockMutex(deallocate_mutex);
 }
 
 static void enqueue_frame(FrameQueue *fq, AVFrame *frame) {
@@ -351,6 +433,10 @@ static AVFrame *dequeue_frame(FrameQueue *fq) {
 
 static void enqueue_packet(PacketQueue *pq, AVPacket *pkt) {
 	AVPacketList *pl = av_malloc(sizeof(AVPacketList));
+	if (pl == NULL)
+	{
+		return;
+	}
 
 	av_init_packet(&pl->pkt);
 	av_packet_ref(&pl->pkt, pkt);
@@ -480,38 +566,102 @@ static int read_packet(MediaState *ms, PacketQueue *pq, AVPacket *pkt) {
 
 static AVCodecContext *find_context(AVFormatContext *ctx, int index) {
 
+    AVDictionary *opts = NULL;
+
 	if (index == -1) {
 		return NULL;
 	}
 
-	AVCodec *codec;
+	AVCodec *codec = NULL;
 	AVCodecContext *codec_ctx = NULL;
-	AVCodecContext *codec_ctx_orig = ctx->streams[index]->codec;
 
-	codec = avcodec_find_decoder(codec_ctx_orig->codec_id);
+	codec_ctx = avcodec_alloc_context3(NULL);
 
-	if (codec == NULL) {
+	if (codec_ctx == NULL) {
 		return NULL;
 	}
 
-	codec_ctx = avcodec_alloc_context3(codec);
-
-	if (avcodec_copy_context(codec_ctx, codec_ctx_orig)) {
+	if (avcodec_parameters_to_context(codec_ctx, ctx->streams[index]->codecpar) < 0) {
 		goto fail;
 	}
 
-	if (avcodec_open2(codec_ctx, codec, NULL)) {
+	codec_ctx->pkt_timebase = ctx->streams[index]->time_base;
+
+    codec = avcodec_find_decoder(codec_ctx->codec_id);
+
+    if (codec == NULL) {
+        goto fail;
+    }
+
+    codec_ctx->codec_id = codec->id;
+
+    av_dict_set(&opts, "threads", "auto", 0);
+    av_dict_set(&opts, "refcounted_frames", "0", 0);
+
+	if (avcodec_open2(codec_ctx, codec, &opts)) {
 		goto fail;
 	}
 
 	return codec_ctx;
 
 fail:
+
+    av_dict_free(&opts);
+
 	avcodec_free_context(&codec_ctx);
 	return NULL;
 }
 
 
+/**
+ * Given a packet, decodes a frame if possible. This is intended to be a drop-in replacement
+ * for the now deprecated avcodec_decode_audio4/video2 APIs.
+ *
+ * \param[in]   context     The context the decoding is done in.
+ * \param[out]  frame       A frame that is updated with the decoded data.
+ * \param[out]  got_frame   Set to 1 if a frame was decoded, 0 if not.
+ * \param[in]   pkt         The packet data to present.
+ *
+ * Returns pkt->size if the packet was consumed, 0 if not, or < 0 on error (including
+ * end of file.)
+ */
+static int decode_common(AVCodecContext *context, AVFrame *frame, int *got_frame, AVPacket *pkt) {
+
+    int ret;
+    int rv = 0;
+
+    if (pkt) {
+        ret = avcodec_send_packet(context, pkt);
+
+        if (ret >= 0) {
+            rv = pkt->size;
+        } else if (ret == AVERROR(EAGAIN) || ret == AVERROR_EOF) {
+            rv = 0;
+        } else {
+           return ret;
+        }
+    }
+
+    ret = avcodec_receive_frame(context, frame);
+
+    if (ret >= 0) {
+        *got_frame = 1;
+    } else if (ret == AVERROR(EAGAIN)) {
+        *got_frame = 0;
+    } else if (ret == AVERROR_EOF) {
+        *got_frame = 0;
+        if (!pkt || pkt->size == 0) {
+            return ret;
+        }
+    } else {
+        *got_frame = 0;
+        return ret;
+    }
+
+    return rv;
+}
+
+
 /**
  * Decodes audio. Returns 0 if no audio was decoded, or 1 if some audio was
  * decoded.
@@ -530,6 +680,11 @@ static void decode_audio(MediaState *ms) {
 		ms->audio_decode_frame = av_frame_alloc();
 	}
 
+	if (ms->audio_decode_frame == NULL) {
+		ms->audio_finished = 1;
+		return;
+	}
+
 	av_init_packet(&pkt);
 
 	double timebase = av_q2d(ms->ctx->streams[ms->audio_stream]->time_base);
@@ -546,7 +701,7 @@ static void decode_audio(MediaState *ms) {
 
 		do {
 			int got_frame;
-			int read_size = avcodec_decode_audio4(ms->audio_context, ms->audio_decode_frame, &got_frame, &pkt_temp);
+			int read_size = decode_common(ms->audio_context, ms->audio_decode_frame, &got_frame, &pkt_temp);
 
 			if (read_size < 0) {
 
@@ -572,6 +727,12 @@ static void decode_audio(MediaState *ms) {
 			}
 
             converted_frame = av_frame_alloc();
+
+			if (converted_frame == NULL) {
+				ms->audio_finished = 1;
+				return;
+			}
+
             converted_frame->sample_rate = audio_sample_rate;
             converted_frame->channel_layout = AV_CH_LAYOUT_STEREO;
             converted_frame->format = AV_SAMPLE_FMT_S16;
@@ -600,7 +761,7 @@ static void decode_audio(MediaState *ms) {
 				continue;
 			}
 
-			double start = av_frame_get_best_effort_timestamp(ms->audio_decode_frame) * timebase;
+			double start = ms->audio_decode_frame->best_effort_timestamp * timebase;
 			double end = start + 1.0 * converted_frame->nb_samples / audio_sample_rate;
 
 			SDL_LockMutex(ms->lock);
@@ -670,7 +831,7 @@ static SurfaceQueueEntry *decode_video_frame(MediaState *ms) {
 		}
 
 		int got_frame = 0;
-		int read_size = avcodec_decode_video2(ms->video_context, ms->video_decode_frame, &got_frame, &ms->video_pkt_tmp);
+		int read_size = decode_common(ms->video_context, ms->video_decode_frame, &got_frame, &ms->video_pkt_tmp);
 
 		if (read_size < 0) {
 			ms->video_finished = 1;
@@ -691,8 +852,7 @@ static SurfaceQueueEntry *decode_video_frame(MediaState *ms) {
 
 	}
 
-	double pts = av_frame_get_best_effort_timestamp(ms->video_decode_frame) \
-			* av_q2d(ms->ctx->streams[ms->video_stream]->time_base);
+	double pts = ms->video_decode_frame->best_effort_timestamp * av_q2d(ms->ctx->streams[ms->video_stream]->time_base);
 
 	if (pts < ms->skip) {
 		return NULL;
@@ -738,19 +898,24 @@ static SurfaceQueueEntry *decode_video_frame(MediaState *ms) {
 	}
 
 	SurfaceQueueEntry *rv = av_malloc(sizeof(SurfaceQueueEntry));
+	if (rv == NULL) {
+		ms->video_finished = 1;
+		return NULL;
+	}
 	rv->w = ms->video_decode_frame->width + FRAME_PADDING * 2;
-	rv->pitch = rv->w * sample->format->BytesPerPixel;
 	rv->h = ms->video_decode_frame->height + FRAME_PADDING * 2;
 
-	// We have to use SDL_calloc here, since SDL frees these pixels. This
-	// Should be
+	rv->pitch = rv->w * sample->format->BytesPerPixel;
 
+	if (rv->pitch % ROW_ALIGNMENT) {
+	    rv->pitch += ROW_ALIGNMENT - (rv->pitch % ROW_ALIGNMENT);
+	}
 
-#ifdef USE_MEMALIGN
-    posix_memalign(&rv->pixels, 16, rv->pitch * rv->h);
-    memset(rv->pixels, 0, rv->pitch * rv->h);
+#ifndef USE_POSIX_MEMALIGN
+    rv->pixels = SDL_calloc(rv->pitch * rv->h, 1);
 #else
-    rv->pixels = SDL_calloc(1, rv->pitch * rv->h);
+    posix_memalign(&rv->pixels, ROW_ALIGNMENT, rv->pitch * rv->h);
+    memset(rv->pixels, 0, rv->pitch * rv->h);
 #endif
 
 	rv->format = sample->format;
@@ -788,6 +953,11 @@ static void decode_video(MediaState *ms) {
 		ms->video_decode_frame = av_frame_alloc();
 	}
 
+	if (!ms->video_decode_frame) {
+		ms->video_finished = 1;
+		return;
+	}
+
 	SDL_LockMutex(ms->lock);
 
 	if (!ms->video_finished && (ms->surface_queue_size < FRAMES)) {
@@ -835,6 +1005,12 @@ int media_video_ready(struct MediaState *ms) {
 		goto done;
 	}
 
+	if (ms->pause_time > 0) {
+	    goto done;
+	}
+
+	double offset_time = current_time - ms->time_offset;
+
 	/*
 	 * If we have an obsolete frame, drop it.
 	 */
@@ -850,7 +1026,13 @@ int media_video_ready(struct MediaState *ms) {
 			SurfaceQueueEntry *sqe = dequeue_surface(&ms->surface_queue);
 			ms->surface_queue_size -= 1;
 
-			SDL_free(sqe->pixels);
+			if (sqe->pixels) {
+#ifndef USE_POSIX_MEMALIGN
+				SDL_free(sqe->pixels);
+#else
+				free(sqe->pixels);
+#endif
+			}
 			av_free(sqe);
 
 			consumed = 1;
@@ -864,7 +1046,7 @@ int media_video_ready(struct MediaState *ms) {
 
 	if (ms->surface_queue) {
 		if (ms->video_pts_offset) {
-			if (ms->surface_queue->pts + ms->video_pts_offset <= current_time) {
+			if (ms->surface_queue->pts + ms->video_pts_offset <= offset_time + frame_early_delivery) {
 				rv = 1;
 			}
 		} else {
@@ -895,6 +1077,8 @@ SDL_Surface *media_read_video(MediaState *ms) {
 		return NULL;
 	}
 
+	double offset_time = current_time - ms->time_offset;
+
 	SDL_LockMutex(ms->lock);
 
 #ifndef __EMSCRIPTEN__
@@ -903,15 +1087,19 @@ SDL_Surface *media_read_video(MediaState *ms) {
 	}
 #endif
 
+	if (ms->pause_time > 0) {
+	    goto done;
+	}
+
 	if (!ms->surface_queue_size) {
 		goto done;
 	}
 
 	if (ms->video_pts_offset == 0.0) {
-		ms->video_pts_offset = current_time - ms->surface_queue->pts;
+		ms->video_pts_offset = offset_time - ms->surface_queue->pts;
 	}
 
-	if (ms->surface_queue->pts + ms->video_pts_offset <= current_time) {
+	if (ms->surface_queue->pts + ms->video_pts_offset <= offset_time + frame_early_delivery) {
 		sqe = dequeue_surface(&ms->surface_queue);
 		ms->surface_queue_size -= 1;
 
@@ -922,7 +1110,7 @@ done:
     /* Only signal if we've consumed something. */
 	if (sqe) {
 		ms->needs_decode = 1;
-		ms->video_read_time = current_time;
+		ms->video_read_time = offset_time;
 		SDL_CondBroadcast(ms->cond);
 	}
 
@@ -956,13 +1144,21 @@ static int decode_thread(void *arg) {
 	int err;
 
 	AVFormatContext *ctx = avformat_alloc_context();
+	if (ctx == NULL) {
+		goto finish;
+	}
 	ms->ctx = ctx;
 
 	AVIOContext *io_context = rwops_open(ms->rwops);
+	if (io_context == NULL) {
+		goto finish;
+	}
 	ctx->pb = io_context;
 
 	err = avformat_open_input(&ctx, ms->filename, NULL, NULL);
 	if (err) {
+		avformat_free_context(ctx);
+		ms->ctx = NULL;
 		goto finish;
 	}
 
@@ -975,14 +1171,14 @@ static int decode_thread(void *arg) {
 	ms->video_stream = -1;
 	ms->audio_stream = -1;
 
-	for (int i = 0; i < ctx->nb_streams; i++) {
-		if (ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO) {
+	for (unsigned int i = 0; i < ctx->nb_streams; i++) {
+		if (ctx->streams[i]->codecpar->codec_type == AVMEDIA_TYPE_VIDEO) {
 			if (ms->want_video && ms->video_stream == -1) {
 				ms->video_stream = i;
 			}
 		}
 
-		if (ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_AUDIO) {
+		if (ctx->streams[i]->codecpar->codec_type == AVMEDIA_TYPE_AUDIO) {
 			if (ms->audio_stream == -1) {
 				ms->audio_stream = i;
 			}
@@ -993,6 +1189,9 @@ static int decode_thread(void *arg) {
 	ms->audio_context = find_context(ctx, ms->audio_stream);
 
 	ms->swr = swr_alloc();
+	if (ms->swr == NULL) {
+		goto finish;
+	}
 
 	av_init_packet(&ms->video_pkt);
 
@@ -1107,33 +1306,41 @@ static int decode_sync_start(void *arg) {
 	int err;
 
 	AVFormatContext *ctx = avformat_alloc_context();
+	if (ctx == NULL) {
+		media_read_sync_finish(ms);
+	}
 	ms->ctx = ctx;
 
 	AVIOContext *io_context = rwops_open(ms->rwops);
+	if (io_context == NULL) {
+		media_read_sync_finish(ms);
+	}
 	ctx->pb = io_context;
 
 	err = avformat_open_input(&ctx, ms->filename, NULL, NULL);
 	if (err) {
-	  media_read_sync_finish(ms);
+		avformat_free_context(ctx);
+		ms->ctx = NULL;
+		media_read_sync_finish(ms);
 	}
 
 	err = avformat_find_stream_info(ctx, NULL);
 	if (err) {
-	  media_read_sync_finish(ms);
+		media_read_sync_finish(ms);
 	}
 
 
 	ms->video_stream = -1;
 	ms->audio_stream = -1;
 
-	for (int i = 0; i < ctx->nb_streams; i++) {
-		if (ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO) {
+	for (unsigned int i = 0; i < ctx->nb_streams; i++) {
+		if (ctx->streams[i]->codecpar->codec_type == AVMEDIA_TYPE_VIDEO) {
 			if (ms->want_video && ms->video_stream == -1) {
 				ms->video_stream = i;
 			}
 		}
 
-		if (ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_AUDIO) {
+		if (ctx->streams[i]->codecpar->codec_type == AVMEDIA_TYPE_AUDIO) {
 			if (ms->audio_stream == -1) {
 				ms->audio_stream = i;
 			}
@@ -1144,6 +1351,9 @@ static int decode_sync_start(void *arg) {
 	ms->audio_context = find_context(ctx, ms->audio_stream);
 
 	ms->swr = swr_alloc();
+	if (ms->swr == NULL) {
+		media_read_sync_finish(ms);
+	}
 
 	av_init_packet(&ms->video_pkt);
 
@@ -1229,7 +1439,7 @@ int media_read_audio(struct MediaState *ms, Uint8 *stream, int len) {
 	int rv = 0;
 
 	if (ms->audio_duration >= 0) {
-		unsigned int remaining = (ms->audio_duration - ms->audio_read_samples) * BPS;
+		int remaining = (ms->audio_duration - ms->audio_read_samples) * BPS;
 		if (len > remaining) {
 			len = remaining;
 		}
@@ -1327,23 +1537,39 @@ void media_start(MediaState *ms) {
 
 	snprintf(buf, 1024, "decode: %s", ms->filename);
 	SDL_Thread *t = SDL_CreateThread(decode_thread, buf, (void *) ms);
-
-	if (t) {
-		ms->started = 1;
-		SDL_DetachThread(t);
-	}
+	ms->thread = t;
 #endif
 }
 
 
 MediaState *media_open(SDL_RWops *rwops, const char *filename) {
-	MediaState *ms = av_calloc(1, sizeof(MediaState));
+
+    deallocate_deferred();
+
+    MediaState *ms = av_calloc(1, sizeof(MediaState));
+	if (ms == NULL) {
+		return NULL;
+	}
 
 	ms->filename = av_strdup(filename);
+	if (ms->filename == NULL) {
+		deallocate(ms);
+		return NULL;
+	}
 	ms->rwops = rwops;
 
+#ifndef __EMSCRIPTEN__
 	ms->cond = SDL_CreateCond();
+	if (ms->cond == NULL) {
+		deallocate(ms);
+		return NULL;
+	}
 	ms->lock = SDL_CreateMutex();
+	if (ms->lock == NULL) {
+		deallocate(ms);
+		return NULL;
+	}
+#endif
 
 	ms->audio_duration = -1;
 	ms->frame_drops = 1;
@@ -1373,7 +1599,6 @@ void media_start_end(MediaState *ms, double start, double end) {
 	}
 }
 
-
 /**
  * Marks the channel as having video.
  */
@@ -1382,9 +1607,18 @@ void media_want_video(MediaState *ms, int video) {
 	ms->frame_drops = (video != 2);
 }
 
+void media_pause(MediaState *ms, int pause) {
+    if (pause && (ms->pause_time == 0)) {
+        ms->pause_time = current_time;
+    } else if ((!pause) && (ms->pause_time > 0)) {
+        ms->time_offset += current_time - ms->pause_time;
+        ms->pause_time = 0;
+    }
+}
+
 void media_close(MediaState *ms) {
 
-	if (!ms->started) {
+	if (!ms->thread) {
 		deallocate(ms);
 		return;
 	}
@@ -1413,11 +1647,11 @@ void media_sample_surfaces(SDL_Surface *rgb, SDL_Surface *rgba) {
 
 void media_init(int rate, int status, int equal_mono) {
 
+    deallocate_mutex = SDL_CreateMutex();
+
 	audio_sample_rate = rate / SPEED;
 	audio_equal_mono = equal_mono;
 
-    av_register_all();
-
     if (status) {
         av_log_set_level(AV_LOG_INFO);
     } else {
@@ -1425,5 +1659,3 @@ void media_init(int rate, int status, int equal_mono) {
     }
 
 }
-
-
