@@ -73,8 +73,8 @@ print(("cdef char *break_rules = \"" + "".join(rules) + "\""))
 
 cc = [ 'XX' ] * 65536
 
-for l in file("LineBreak.txt"):
-    m = re.match("(\w+)\.\.(\w+);(\w\w)", l)
+for l in open("LineBreak.txt"):
+    m = re.match(r"(\w+)\.\.(\w+);(\w\w)", l)
     if m:
         start = int(m.group(1), 16)
         end = int(m.group(2), 16)
@@ -90,7 +90,7 @@ for l in file("LineBreak.txt"):
 
         continue
 
-    m = re.match("(\w+);(\w\w)", l)
+    m = re.match(r"(\w+);(\w\w)", l)
     if m:
         start = int(m.group(1), 16)
 
