@@ -1,4 +1,4 @@
-# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,15 +21,15 @@
 
 from __future__ import print_function, unicode_literals, division, absolute_import
 
-str = unicode  # @ReservedAssignment
+from future import standard_library
+standard_library.install_aliases()
+
+from builtins import str
 
 import collections
 import os
 
-try:
-    from io import StringIO  # @UnusedImport
-except:
-    from StringIO import StringIO  # @Reimport
+from io import StringIO # @UnusedImport
 
 # Paths
 BASE = os.path.dirname(os.path.abspath(__file__))
@@ -101,14 +101,13 @@ Prefix(4, 'selected_idle_', 3, [ ])
 Prefix(3, 'selected_insensitive_', 3, [ ])
 Prefix(-3, 'selected_', 2, [ "selected_hover_", "selected_idle_", "selected_insensitive_" ])
 Prefix(2, 'hover_', 1, [ "selected_hover_" ])
-Prefix(1, 'idle_', 1, [ "selected_idle_" ] )
+Prefix(1, 'idle_', 1, [ "selected_idle_" ])
 Prefix(0, 'insensitive_', 1, [ "selected_insensitive_" ])
-Prefix(-4, '', 0, [ "selected_hover_", "selected_idle_", "selected_insensitive_", "idle_", "hover_", "insensitive_" ] )
+Prefix(-4, '', 0, [ "selected_hover_", "selected_idle_", "selected_insensitive_", "idle_", "hover_", "insensitive_" ])
 
 Prefix(-2, 'activate_', 0, [ ])
 Prefix(-1, 'selected_activate_', 0, [ ])
 
-
 # The images that are searched by a prefix.
 PREFIX_SEARCH = {
     "idle_" : [ "idle_", "" ],
@@ -120,7 +119,6 @@ PREFIX_SEARCH = {
     "" : [ "" ],
     }
 
-
 ################################################################################
 # Style Properties
 ################################################################################
@@ -182,7 +180,9 @@ style_properties = sorted_dict(
     left_padding=None,
     line_spacing=None,
     mouse=None,
+    modal=None,
     min_width=None,
+    mipmap=None,
     newline_indent=None,
     order_reverse=None,
     outlines='expand_outlines',
@@ -213,7 +213,7 @@ style_properties = sorted_dict(
     xfill=None,
     xfit=None,
     xmaximum=None,
-    xminimum=None,
+    xminimum='none_is_0',
     xoffset=None,
     xpos=None,
     xspacing=None,
@@ -221,7 +221,7 @@ style_properties = sorted_dict(
     yfill=None,
     yfit=None,
     ymaximum=None,
-    yminimum=None,
+    yminimum='none_is_0',
     yoffset=None,
     ypos=None,
     yspacing=None,
@@ -239,7 +239,6 @@ displayable_properties = {
     "thumb_shadow",
     }
 
-
 # A map from a style property to its index in the order of style_properties.
 style_property_index = collections.OrderedDict()
 for i, name in enumerate(style_properties):
@@ -250,7 +249,6 @@ style_property_count = len(style_properties)
 # print("{} properties * {} prefixes = {} cache entries".format(
 #     style_property_count, PREFIX_COUNT, style_property_count * PREFIX_COUNT))
 
-
 # Special priority properties - these take a +1 compared to others. Generally,
 # these would be listed in the tuples in synthetic_properies, below.
 property_priority = sorted_dict(
@@ -276,7 +274,6 @@ property_priority = sorted_dict(
     yfill=1,
 )
 
-
 # A list of synthetic style properties, where each property is expanded into
 # multiple style properties. Each property are mapped into a list of tuples,
 # with each consisting of:
@@ -350,8 +347,8 @@ synthetic_properties = sorted_dict(
         ('aft_bar', None),
         ],
 
-    box_spacing=[ ( 'spacing', None ) ],
-    box_first_spacing=[ ( 'first_spacing', None) ],
+    box_spacing=[ ('spacing', None) ],
+    box_first_spacing=[ ('first_spacing', None) ],
 
     pos=[
         ('xpos', 'index_0'),
@@ -459,13 +456,13 @@ class CodeGen(object):
         text = self.f.getvalue()
 
         if os.path.exists(self.filename):
-            with open(self.filename, "rb") as f:
+            with open(self.filename, "r") as f:
                 old = f.read()
 
             if old == text:
                 return
 
-        with open(self.filename, "wb") as f:
+        with open(self.filename, "w") as f:
             f.write(text)
 
     def write(self, s, *args, **kwargs):
@@ -641,13 +638,13 @@ def generate_sets():
 
     g.write("# This file is generated by generate_styles.py.")
     g.write("")
-    g.write('exec """\\')
+    g.write('exec("""\\')
     g.write("all_properties = {}", ap)
     g.write("prefix_priority = {}", prefix_priority)
     g.write("prefix_alts = {}", prefix_alts)
     g.write("prefix_search = {}", PREFIX_SEARCH)
     g.write("property_priority = {}", property_priority)
-    g.write('"""')
+    g.write('""")')
     g.close()
 
 
