@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -28,21 +28,22 @@ import sys
 import os
 import subprocess
 
+import future
+
 # Change to the directory containing this file.
-BASE=os.path.abspath(os.path.dirname(sys.argv[0]))
+BASE = os.path.abspath(os.path.dirname(sys.argv[0]))
 os.chdir(BASE)
 
 # Create the gen directory if it doesn't exist.
 try:
     os.makedirs("gen")
-except:
+except Exception:
     pass
 
 # Generate styles.
 import generate_styles
 generate_styles.generate()
 
-
 # If RENPY_CC or RENPY_LD are in the environment, and CC or LD are not, use them.
 def setup_env(name):
     renpy_name = "RENPY_" + name
@@ -55,7 +56,7 @@ setup_env("LD")
 setup_env("CXX")
 
 import setuplib
-from setuplib import android, ios, emscripten, raspi, include, library, cython, cmodule, copyfile, find_unnecessary_gen
+from setuplib import android, ios, emscripten, raspi, include, library, cython, cmodule, copyfile, find_unnecessary_gen, generate_all_cython
 
 # These control the level of optimization versus debugging.
 setuplib.extra_compile_args = [ "-Wno-unused-function" ]
@@ -65,8 +66,11 @@ setuplib.extra_link_args = [ ]
 if platform.win32_ver()[0]:
     windows = True
     setuplib.extra_compile_args.append("-fno-strict-aliasing")
+    tfd_libs = [ "comdlg32", "ole32" ]
+
 else:
     windows = False
+    tfd_libs = [ ]
 
 if raspi:
     setuplib.extra_compile_args.append("-DRASPBERRY_PI")
@@ -75,11 +79,11 @@ include("zlib.h")
 include("png.h")
 include("SDL.h", directory="SDL2")
 include("ft2build.h")
-include("freetype/freetype.h", directory="freetype2", optional=True) or include("freetype.h", directory="freetype2")
-include("libavutil/avstring.h", directory="ffmpeg", optional=True) or include("libavutil/avstring.h")
-include("libavformat/avformat.h", directory="ffmpeg", optional=True) or include("libavformat/avformat.h")
-include("libavcodec/avcodec.h", directory="ffmpeg", optional=True) or include("libavcodec/avcodec.h")
-include("libswscale/swscale.h", directory="ffmpeg", optional=True) or include("libswscale/swscale.h")
+include("freetype/freetype.h", directory="freetype2", optional=True) or include("freetype.h", directory="freetype2") # type: ignore
+include("libavutil/avstring.h", directory="ffmpeg", optional=True) or include("libavutil/avstring.h") # type: ignore
+include("libavformat/avformat.h", directory="ffmpeg", optional=True) or include("libavformat/avformat.h") # type: ignore
+include("libavcodec/avcodec.h", directory="ffmpeg", optional=True) or include("libavcodec/avcodec.h") # type: ignore
+include("libswscale/swscale.h", directory="ffmpeg", optional=True) or include("libswscale/swscale.h") # type: ignore
 include("GL/glew.h")
 include("pygame_sdl2/pygame_sdl2.h", directory="python{}.{}".format(sys.version_info.major, sys.version_info.minor))
 
@@ -96,8 +100,6 @@ library("z")
 has_libglew = library("GLEW", optional=True)
 has_libglew32 = library("glew32", optional=True)
 
-has_angle = windows and library("EGL", optional=True) and library("GLESv2", optional=True)
-
 if android:
     sdl = [ 'SDL2', 'GLESv2', 'log' ]
     png = 'png16'
@@ -105,17 +107,14 @@ else:
     sdl = [ 'SDL2' ]
     png = 'png'
 
-steam_sdk = os.environ.get("RENPY_STEAM_SDK", None)
-steam_platform = os.environ.get("RENPY_STEAM_PLATFORM", "")
-
-if steam_sdk:
-    setuplib.library_dirs.append("{}/redistributable_bin/{}".format(steam_sdk, steam_platform))
-    setuplib.include_dirs.append("{}/public".format(steam_sdk))
+cubism = os.environ.get("CUBISM", None)
+if cubism:
+    setuplib.include_dirs.append("{}/Core/include".format(cubism))
 
 # Modules directory.
 cython(
     "_renpy",
-    [ "IMG_savepng.c", "core.c", "subpixel.c"],
+    [ "IMG_savepng.c", "core.c" ],
     sdl + [ png, 'z', 'm' ])
 
 FRIBIDI_SOURCES = """
@@ -144,8 +143,8 @@ cython(
         ("HAVE_CONFIG_H", "1"),
         ])
 
-
-cython("_renpysteam", language="c++", compile_if=steam_sdk, libs=["steam_api"])
+if not (android or ios or emscripten):
+    cython("_renpytfd", [ "tinyfiledialogs/tinyfiledialogs.c" ], libs=tfd_libs)
 
 # Sound.
 
@@ -173,6 +172,9 @@ cython("renpy.parsersupport")
 cython("renpy.pydict")
 cython("renpy.style")
 
+# renpy.compat
+cython("renpy.compat.dictviews")
+
 # renpy.styledata
 cython("renpy.styledata.styleclass")
 cython("renpy.styledata.stylesets")
@@ -185,69 +187,26 @@ cython("renpy.display.matrix")
 cython("renpy.display.render", libs=[ 'z', 'm' ])
 cython("renpy.display.accelerator", libs=sdl + [ 'z', 'm' ])
 
-# renpy.gl
-if (android or ios):
-    glew_libs = [ 'GLESv2', 'z', 'm' ]
-    gl2_only = True
-    egl = "egl_none.c"
-elif emscripten:
-    glew_libs = []
-    gl2_only = True
-    egl = "egl_none.c"
-elif raspi:
-    glew_libs = [ 'SDL2', 'GLESv2', 'EGL', 'z', 'm' ]
-    gl2_only = True
-    egl = "egl_x11.c"
-elif has_libglew:
-    glew_libs = [ 'GLEW' ]
-    gl2_only = False
-    egl = "egl_none.c"
-else:
-    glew_libs = [ 'glew32', 'opengl32' ]
-    gl2_only = False
-    egl = "egl_none.c"
+cython("renpy.uguu.gl", libs=sdl)
+cython("renpy.uguu.uguu", libs=sdl)
 
-cython("renpy.gl.gl", libs=glew_libs)
-cython("renpy.gl.gl1", libs=glew_libs, compile_if=not gl2_only)
-cython("renpy.gl.gldraw", libs=glew_libs, source=[ egl ])
-cython("renpy.gl.gltexture", libs=glew_libs)
-cython("renpy.gl.glenviron_shader", libs=glew_libs)
-cython("renpy.gl.glenviron_fixed", libs=glew_libs, compile_if=not gl2_only)
-cython("renpy.gl.glenviron_limited", libs=glew_libs, compile_if=not gl2_only)
-cython("renpy.gl.glrtt_copy", libs=glew_libs)
-cython("renpy.gl.glrtt_fbo", libs=glew_libs)
+cython("renpy.gl.gldraw", libs=sdl)
+cython("renpy.gl.gltexture", libs=sdl)
+cython("renpy.gl.glenviron_shader", libs=sdl)
+cython("renpy.gl.glrtt_copy", libs=sdl)
+cython("renpy.gl.glrtt_fbo", libs=sdl)
 
-cython("renpy.gl2.uguugl", libs=sdl)
-cython("renpy.gl2.uguu", libs=sdl)
-cython("renpy.gl2.gl2geometry")
+cython("renpy.gl2.gl2mesh")
+cython("renpy.gl2.gl2mesh2")
+cython("renpy.gl2.gl2mesh3")
+cython("renpy.gl2.gl2polygon")
+cython("renpy.gl2.gl2model")
 cython("renpy.gl2.gl2draw", libs=sdl)
 cython("renpy.gl2.gl2texture", libs=sdl)
 cython("renpy.gl2.gl2shader", libs=sdl)
 
-if not (android or ios or emscripten):
-    # renpy.angle
-    def anglecopy(fn):
-        copyfile("renpy/gl/" + fn, "renpy/angle/" + fn, "DEF ANGLE = False", "DEF ANGLE = True")
-
-    anglecopy("gldraw.pxd")
-    anglecopy("gldraw.pyx")
-    anglecopy("glenviron_shader.pyx")
-    anglecopy("glrtt_fbo.pyx")
-    anglecopy("glrtt_copy.pyx")
-    anglecopy("gltexture.pxd")
-    anglecopy("gltexture.pyx")
-
-    angle_libs = [ "SDL2", "EGL", "GLESv2" ]
-
-    def anglecython(name, source=[]):
-        cython(name, libs=angle_libs, compile_if=has_angle, define_macros=[ ( "ANGLE", None ) ], source=source)
-
-    anglecython("renpy.angle.gl")
-    anglecython("renpy.angle.gldraw", source=[ "egl_angle.c" ])
-    anglecython("renpy.angle.gltexture")
-    anglecython("renpy.angle.glenviron_shader")
-    anglecython("renpy.angle.glrtt_fbo")
-    anglecython("renpy.angle.glrtt_copy")
+if cubism:
+    cython("renpy.gl2.live2dmodel", libs=sdl)
 
 # renpy.text
 cython("renpy.text.textsupport")
@@ -258,6 +217,7 @@ cython(
     [ "ftsupport.c", "ttgsubtable.c" ],
     libs=sdl + [ 'freetype', 'z', 'm' ])
 
+generate_all_cython()
 find_unnecessary_gen()
 
 # Figure out the version, and call setup.
@@ -265,4 +225,4 @@ sys.path.insert(0, '..')
 
 import renpy
 
-setuplib.setup("Ren'Py", renpy.version[7:])  # @UndefinedVariable
+setuplib.setup("Ren'Py", renpy.version[7:]) # @UndefinedVariable
