@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -19,12 +19,14 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-from __future__ import print_function
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
+
+
 
 import collections
 import renpy
 import os
-import codecs
 
 # A map from filename to position, target label pairs.
 missing = collections.defaultdict(list)
@@ -34,9 +36,6 @@ def report_missing(target, filename, position):
     """
     Reports that the call statement ending at `position` in `filename`
     is missing a from clause.
-
-    `target`
-        The string
     """
 
     missing[filename].append((position, target))
@@ -51,6 +50,8 @@ def generate_label(target):
     Generate a reasonable and unique new label for a call to `target`.
     """
 
+    target = target.replace(".", "_")
+
     n = 0
 
     while True:
@@ -79,8 +80,8 @@ def process_file(fn):
     edits = missing[fn]
     edits.sort()
 
-    with codecs.open(fn, "r", "utf-8") as f:
-        data = f.read()
+    with open(fn, "rb") as f:
+        data = f.read().decode("utf-8")
 
     # How much of the input has been consumed.
     consumed = 0
@@ -96,12 +97,12 @@ def process_file(fn):
 
     output += data[consumed:]
 
-    with codecs.open(fn + ".new", "w", "utf-8") as f:
-        f.write(output)
+    with open(fn + ".new", "wb") as f:
+        f.write(output.encode("utf-8"))
 
     try:
         os.unlink(fn + ".bak")
-    except:
+    except Exception:
         pass
 
     os.rename(fn, fn + ".bak")
@@ -118,4 +119,5 @@ def add_from():
 
     return False
 
+
 renpy.arguments.register_command("add_from", add_from)
