@@ -1,4 +1,4 @@
-# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,16 +21,19 @@
 
 # The Character object (and friends).
 
-from __future__ import print_function
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
 
-import renpy.display
+from typing import Any
+
+import renpy
 
 import re
 import os
 import collections
 
 # This matches the dialogue-relevant text tags.
-TAG_RE = re.compile(r'(\{\{)|(\{(p|w|nw|fast)(?:\=([^}]*))?\})', re.S)
+TAG_RE = re.compile(r'(\{\{)|(\{(p|w|nw|fast|done)(?:\=([^}]*))?\})', re.S)
 
 less_pauses = ("RENPY_LESS_PAUSES" in os.environ)
 
@@ -58,17 +61,23 @@ class DialogueTextTags(object):
         # True if we've encountered the no-wait tag.
         self.no_wait = False
 
+        # Does this statement have a done tag?
+        self.has_done = False
+
+        # Does this statement have a fast tag?
+        self.fast = False
+
         i = iter(TAG_RE.split(s))
 
         while True:
 
             try:
-                self.text += i.next()
+                self.text += next(i)
 
-                quoted = i.next()
-                full_tag = i.next()
-                tag = i.next()
-                value = i.next()
+                quoted = next(i)
+                full_tag = next(i)
+                tag = next(i)
+                value = next(i)
 
                 if value is not None:
                     value = float(value)
@@ -91,6 +100,12 @@ class DialogueTextTags(object):
                     self.pause_end = [ ]
                     self.pause_delay = [ ]
                     self.no_wait = False
+                    self.fast = True
+
+                elif tag == "done":
+                    self.has_done = True
+                    self.text += full_tag
+                    break
 
                 self.text += full_tag
 
@@ -99,6 +114,25 @@ class DialogueTextTags(object):
 
         self.pause_end.append(len(self.text))
 
+        while True:
+
+            try:
+                self.text += next(i)
+
+                quoted = next(i)
+                full_tag = next(i)
+                tag = next(i)
+                value = next(i)
+
+                if quoted is not None:
+                    self.text += quoted
+                    continue
+
+                self.text += full_tag
+
+            except StopIteration:
+                break
+
         if self.no_wait:
             self.pause_delay.append(0)
         else:
@@ -342,8 +376,9 @@ def show_display_say(who, what, who_args={}, what_args={}, window_args={},
 class SlowDone(object):
     delay = None
     ctc_kwargs = { }
+    last_pause = True
 
-    def __init__(self, ctc, ctc_position, callback, interact, type, cb_args, delay, ctc_kwargs):  # @ReservedAssignment
+    def __init__(self, ctc, ctc_position, callback, interact, type, cb_args, delay, ctc_kwargs, last_pause): # @ReservedAssignment
         self.ctc = ctc
         self.ctc_position = ctc_position
         self.callback = callback
@@ -352,6 +387,7 @@ class SlowDone(object):
         self.cb_args = cb_args
         self.delay = delay
         self.ctc_kwargs = ctc_kwargs
+        self.last_pause = last_pause
 
     def __call__(self):
 
@@ -372,16 +408,15 @@ class SlowDone(object):
                 renpy.exports.restart_interaction()
 
         if self.delay is not None:
-            renpy.ui.pausebehavior(self.delay, True, voice=True)
+            renpy.ui.pausebehavior(self.delay, True, voice=self.last_pause)
             renpy.exports.restart_interaction()
 
         for c in self.callback:
             c("slow_done", interact=self.interact, type=self.type, **self.cb_args)
 
-
-# This function takes care of repeatably showing the screen as part of
-# an interaction.
-
+# This is a queue for text that's going to be passed to the say behavior to
+# set the AFM info.
+afm_text_queue = [ ]
 
 def display_say(
         who,
@@ -397,20 +432,29 @@ def display_say(
         cb_args,
         with_none,
         callback,
-        type,  # @ReservedAssignment
+        type, # @ReservedAssignment
         checkpoint=True,
         ctc_timedpause=None,
         ctc_force=False,
         advance=True,
-        multiple=None):
+        multiple=None,
+        dtt=None):
+
+    global afm_text_queue
 
     # Final is true if this statement should perform an interaction.
 
     if multiple is None:
         final = interact
+
+        afm_text_queue = [ ]
+
     else:
         step, total = multiple
 
+        if step == 1:
+            afm_text_queue = [ ]
+
         if step == total:
             final = interact
         else:
@@ -419,7 +463,7 @@ def display_say(
     if not final:
         advance = False
 
-    if final and (not renpy.game.preferences.skip_unseen) and (not renpy.game.context().seen_current(True)) and renpy.config.skipping == "fast":
+    if final and (not renpy.game.preferences.skip_unseen) and (not renpy.game.context().seen_current(True)) and renpy.config.skipping == "fast": # type: ignore
         renpy.config.skipping = None
 
     # If we're in fast skipping mode, don't bother with say
@@ -433,6 +477,11 @@ def display_say(
         renpy.exports.with_statement(None)
         return
 
+    # If we're not interacting, call the noniteractive_callbacks.
+    if interact is False:
+        for i in renpy.config.nointeract_callbacks:
+            i()
+
     # Figure out the callback(s) we want to use.
     if callback is None:
         if renpy.config.character_callback:
@@ -463,21 +512,22 @@ def display_say(
     # If we're committed to skipping this statement, disable slow.
     elif (renpy.config.skipping and
           advance and
-          (renpy.game.preferences.skip_unseen or
+          (renpy.game.preferences.skip_unseen or # type: ignore
            renpy.game.context().seen_current(True))):
         slow = False
         all_at_once = True
 
     # Figure out which pause we're on. (Or set the pause to None in
     # order to put us in all-at-once mode.)
-    if not interact or renpy.game.preferences.self_voicing:
+    if not interact or renpy.game.preferences.self_voicing: # type: ignore
         all_at_once = True
 
-    dtt = DialogueTextTags(what)
+    if dtt is None:
+        dtt = DialogueTextTags(what)
 
     if all_at_once:
         pause_start = [ dtt.pause_start[0] ]
-        pause_end = [ len(dtt.text) ]
+        pause_end = [ dtt.pause_end[-1] ]
         pause_delay = [ dtt.pause_delay[-1] ]
     else:
         pause_start = dtt.pause_start
@@ -486,12 +536,20 @@ def display_say(
 
     exception = None
 
+    if dtt.fast:
+        for i in renpy.config.say_sustain_callbacks:
+            i()
+
     try:
 
         for i, (start, end, delay) in enumerate(zip(pause_start, pause_end, pause_delay)):
 
+            # True if the is the last pause in a line of dialogue.
             last_pause = (i == len(pause_start) - 1)
 
+            if dtt.no_wait:
+                last_pause = False
+
             # If we're going to do an interaction, then saybehavior needs
             # to be here.
             if advance:
@@ -530,15 +588,21 @@ def display_say(
                 what_ctc = what_ctc._duplicate(None)
                 what_ctc._unique()
 
+            if ctc is not what_ctc:
+                if (ctc is not None) and ctc._duplicatable:
+                    ctc = ctc._duplicate(None)
+                    ctc._unique()
+
             if delay == 0:
                 what_ctc = None
+                ctc = None
 
             # Run the show callback.
             for c in callback:
                 c("show", interact=interact, type=type, **cb_args)
 
             # Create the callback that is called when the slow text is done.
-            slow_done = SlowDone(what_ctc, ctc_position, callback, interact, type, cb_args, delay, ctc_kwargs)
+            slow_done = SlowDone(what_ctc, ctc_position, callback, interact, type, cb_args, delay, ctc_kwargs, last_pause)
 
             # Show the text.
             if multiple:
@@ -546,12 +610,17 @@ def display_say(
             else:
                 what_text = show_function(who, what_string)
 
+            if isinstance(what_text, tuple):
+                what_text = renpy.display.screen.get_widget(what_text[0], what_text[1], what_text[2])
+
+            if not multiple:
+                afm_text_queue = [ what_text ]
+            else:
+                afm_text_queue.append(what_text)
+
             if interact or what_string or (what_ctc is not None) or (behavior and afm):
 
-                if isinstance(what_text, tuple):
-                    what_text = renpy.display.screen.get_widget(what_text[0], what_text[1], what_text[2])
-
-                if not isinstance(what_text, renpy.text.text.Text):  # @UndefinedVariable
+                if not isinstance(what_text, renpy.text.text.Text): # @UndefinedVariable
                     raise Exception("The say screen (or show_function) must return a Text object.")
 
                 if what_ctc:
@@ -559,7 +628,13 @@ def display_say(
                     if ctc_position == "nestled":
                         what_text.set_ctc(what_ctc)
                     elif ctc_position == "nestled-close":
-                        what_text.set_ctc([ u"\ufeff", what_ctc])
+                        what_text.set_ctc([ u"\ufeff", what_ctc, ])
+
+                if (not last_pause) and ctc:
+                    if ctc_position == "nestled":
+                        what_text.set_last_ctc(ctc)
+                    elif ctc_position == "nestled-close":
+                        what_text.set_last_ctc([ u"\ufeff", ctc, ])
 
                 if what_text.text[0] == what_string:
 
@@ -575,7 +650,7 @@ def display_say(
                     raise Exception("The displayable with id 'what' was not given the exact contents of the what variable given to the say screen.")
 
                 if behavior and afm:
-                    behavior.set_text(what_text)
+                    behavior.set_text(*afm_text_queue)
 
             else:
 
@@ -610,11 +685,10 @@ def display_say(
     if final:
 
         if not dtt.no_wait:
-            if checkpoint:
-                if exception is None:
-                    renpy.exports.checkpoint(True)
-                else:
-                    renpy.exports.checkpoint(exception)
+            if exception is None:
+                renpy.exports.checkpoint(True, hard=checkpoint)
+            else:
+                renpy.exports.checkpoint(exception)
 
         else:
             renpy.game.after_rollback = after_rollback
@@ -633,7 +707,7 @@ def display_say(
         c("end", interact=interact, type=type, **cb_args)
 
     if exception is not None:
-        raise
+        raise exception
 
 
 class HistoryEntry(renpy.object.Object):
@@ -645,6 +719,17 @@ class HistoryEntry(renpy.object.Object):
     # See ADVCharacter.add_history for the fields.
 
     multiple = None
+    who = None
+    what = None
+
+    def __eq__(self, other):
+        if isinstance(other, type(self)):
+            return self.__dict__ == other.__dict__
+        else:
+            return False
+
+    def __hash__(self):
+        return hash(tuple(sorted(self.__dict__.items())))
 
     def __repr__(self):
         return "<History {!r} {!r}>".format(self.who, self.what)
@@ -653,7 +738,6 @@ class HistoryEntry(renpy.object.Object):
 # This is used to flag values that haven't been set by the user.
 NotSet = renpy.object.Sentinel("NotSet")
 
-
 # The number of multiple characters we've seen during the current
 # interaction.
 multiple_count = 0
@@ -681,6 +765,8 @@ class ADVCharacter(object):
     voice_tag = None
     properties = { }
 
+    _statement_name = None
+
     # When adding a new argument here, remember to add it to copy below.
     def __init__(
             self,
@@ -748,6 +834,8 @@ class ADVCharacter(object):
             advance=d('advance'),
             )
 
+        self._statement_name = properties.pop("statement_name", None)
+
         self.properties = collections.defaultdict(dict)
 
         if kind:
@@ -757,8 +845,8 @@ class ADVCharacter(object):
             self.show_args = kind.show_args.copy()
             self.cb_args = kind.cb_args.copy()
 
-            for k, v in kind.properties.items():
-                self.properties[k] = dict(v)
+            for k, val in kind.properties.items():
+                self.properties[k] = dict(val)
 
         else:
             self.who_args = { "substitute" : False }
@@ -779,8 +867,8 @@ class ADVCharacter(object):
 
         split = renpy.easy.split_properties(properties, *split_args)
 
-        for prefix, d in zip(prefixes, split):
-            self.properties[prefix].update(d)
+        for prefix, dictionary in zip(prefixes, split):
+            self.properties[prefix].update(dictionary)
 
         self.properties["who"].update(split[-1])
 
@@ -854,13 +942,13 @@ class ADVCharacter(object):
             properties=self.properties,
             **self.show_args)
 
-    def resolve_say_attributes(self, predict, attrs, wanted=[], remove=[]):
+    def resolve_say_attributes(self, predict, attrs):
         """
         Deals with image attributes associated with the current say
         statement. Returns True if an image is shown, None otherwise.
         """
 
-        if not (attrs or wanted or remove):
+        if not attrs:
             return
 
         if not self.image_tag:
@@ -874,15 +962,15 @@ class ADVCharacter(object):
         else:
             attrs = tuple(attrs)
 
-        tagged_attrs = ( self.image_tag,) + attrs
+        tagged_attrs = (self.image_tag,) + attrs
         images = renpy.game.context().images
 
-        layer = renpy.config.tag_layer.get(self.image_tag, "master")
+        layer = renpy.exports.default_layer(None, self.image_tag)
 
         # If image is showing already, resolve it, then show or predict it.
         if images.showing(layer, (self.image_tag,)):
 
-            new_image = images.apply_attributes(layer, self.image_tag, tagged_attrs, wanted, remove)
+            new_image = images.apply_attributes(layer, self.image_tag, tagged_attrs)
 
             if new_image is None:
                 new_image = tagged_attrs
@@ -890,10 +978,10 @@ class ADVCharacter(object):
             if images.showing(layer, new_image, exact=True):
                 return
 
-            show_image = (self.image_tag,) + attrs + tuple(wanted) + tuple( "-" + i for i in remove)
+            show_image = (self.image_tag,) + attrs
 
             if predict:
-                images.predict_show(layer, show_image)
+                renpy.exports.predict_show(new_image)
             else:
                 renpy.exports.show(show_image)
                 return True
@@ -904,7 +992,7 @@ class ADVCharacter(object):
 
                 tagged_attrs = (renpy.config.side_image_prefix_tag,) + tagged_attrs
 
-                new_image = images.apply_attributes(layer, self.image_tag, tagged_attrs, wanted, remove)
+                new_image = images.apply_attributes(layer, self.image_tag, tagged_attrs)
 
                 if new_image is None:
                     new_image = tagged_attrs
@@ -975,12 +1063,10 @@ class ADVCharacter(object):
 
         trans, layer = renpy.config.say_attribute_transition_callback(
             self.image_tag, mode, *delta)
-
         if trans is not None:
             if layer is None:
                 renpy.exports.with_statement(trans)
             else:
-                renpy.exports.with_statement(None)
                 renpy.exports.transition(trans, layer=layer)
 
     def restore_say_attributes(self, predicting, state, interact):
@@ -1010,25 +1096,30 @@ class ADVCharacter(object):
                 renpy.exports.show(image_with_attrs)
                 return True
             else:
-                images.predict_show(None, image_with_attrs)
+                renpy.exports.predict_show(image_with_attrs)
 
         else:
             images.predict_show(None, image_with_attrs, show=False)
 
-    def __unicode__(self):
+    def __str__(self):
 
         who = self.name
 
         if self.dynamic:
-            who = renpy.python.py_eval(who)
+            if callable(who):
+                who = who()
+            else:
+                who = renpy.python.py_eval(who)
 
-        return renpy.substitutions.substitute(who)[0]
+        rv = renpy.substitutions.substitute(who)[0]
 
-    def __str__(self):
-        return unicode(self).encode("utf-8")
+        if PY2:
+            rv = rv.encode("utf-8")
+
+        return rv
 
     def __format__(self, spec):
-        return format(unicode(self), spec)
+        return format(str(self), spec)
 
     def __repr__(self):
         return "<Character: {!r}>".format(self.name)
@@ -1040,6 +1131,46 @@ class ADVCharacter(object):
 
         self("", interact=False, _call_done=False)
 
+    def has_character_arguments(self, **kwargs):
+        """
+        Returns True if `kwargs` contains any keyword arguments that will
+        cause the creation of a new Character object and the proxying of a
+        call to that Character object, and False otherwise.
+        """
+
+        safe_kwargs_keys = { "interact", "_mode", "_call_done", "multiple", "_with_none" }
+
+        for i in kwargs:
+            if i not in safe_kwargs_keys:
+                return False
+
+        return True
+
+    def prefix_suffix(self, thing, prefix, body, suffix):
+
+        def sub(s, scope=None, force=False, translate=True):
+            return renpy.substitutions.substitute(s, scope=scope, force=force, translate=translate)[0]
+
+        thingvar_quoted = "[[" + thing + "]"
+        thingvar = "[" + thing + "]"
+
+        if not renpy.config.new_substitutions:
+            return prefix + body + suffix
+
+        # Used before Ren'Py 7.4.
+        elif renpy.config.who_what_sub_compat == 0:
+            pattern = sub(prefix + thingvar_quoted + suffix)
+            return pattern.replace(thingvar, sub(body))
+
+        # Used from Ren'Py 7.4 to Ren'Py 7.4.4
+        elif renpy.config.who_what_sub_compat == 1:
+            pattern = sub(sub(prefix) + thingvar_quoted + sub(suffix))
+            return pattern.replace(thingvar, sub(body))
+
+        # 7.4.5 on.
+        else:
+            return (sub(prefix) + sub(body) + sub(suffix))
+
     def __call__(self, what, interact=True, _call_done=True, multiple=None, **kwargs):
 
         _mode = kwargs.pop("_mode", None)
@@ -1094,9 +1225,11 @@ class ADVCharacter(object):
             if interact:
                 mode = _mode or self.mode
                 renpy.exports.mode(mode)
+            else:
+                renpy.game.context().deferred_translate_identifier = renpy.game.context().translate_identifier
 
             # Figure out the arguments to display.
-            display_args = self.display_args.copy()
+            display_args = self.display_args.copy() # type: Any
             display_args["interact"] = display_args["interact"] and interact
 
             if multiple is not None:
@@ -1109,23 +1242,15 @@ class ADVCharacter(object):
 
             # If dynamic is set, evaluate the name expression.
             if self.dynamic:
-                who = renpy.python.py_eval(who)
-
-            def sub(s, scope=None, force=False, translate=True):
-                return renpy.substitutions.substitute(s, scope=scope, force=force, translate=translate)[0]
+                if callable(who):
+                    who = who()
+                else:
+                    who = renpy.python.py_eval(who)
 
             if who is not None:
-                if renpy.config.new_substitutions:
-                    who_pattern = sub(self.who_prefix + "[[who]" + self.who_suffix)
-                    who = who_pattern.replace("[who]", sub(who))
-                else:
-                    who = self.who_prefix + who + self.who_suffix
+                who = self.prefix_suffix("who", self.who_prefix, who, self.who_suffix)
 
-            if renpy.config.new_substitutions:
-                what_pattern = sub(self.what_prefix + "[[what]" + self.what_suffix)
-                what = what_pattern.replace("[what]", sub(what, translate=True))
-            else:
-                what = self.what_prefix + what + self.what_suffix
+            what = self.prefix_suffix("what", self.what_prefix, what, self.what_suffix)
 
             # Run the add_function, to add this character to the
             # things like NVL-mode.
@@ -1135,11 +1260,13 @@ class ADVCharacter(object):
             else:
                 self.do_add(who, what)
 
+            dtt = DialogueTextTags(what)
+
             # Now, display the damned thing.
-            self.do_display(who, what, cb_args=self.cb_args, **display_args)
+            self.do_display(who, what, cb_args=self.cb_args, dtt=dtt, **display_args)
 
             # Indicate that we're done.
-            if _call_done:
+            if _call_done and not dtt.has_done:
 
                 if multiple is not None:
                     self.do_done(who, what, multiple=multiple)
@@ -1147,7 +1274,7 @@ class ADVCharacter(object):
                     self.do_done(who, what)
 
                 # Finally, log this line of dialogue.
-                if who and isinstance(who, (str, unicode)):
+                if who and isinstance(who, basestring):
                     renpy.exports.log(who)
 
                 renpy.exports.log(what)
@@ -1156,15 +1283,23 @@ class ADVCharacter(object):
         finally:
 
             if (multiple is None) and interact:
-                renpy.store._side_image_attributes = old_side_image_attributes
+                renpy.store._side_image_attributes = old_side_image_attributes # type: ignore
 
-                if old_attr_state is not None:
-                    _, images = old_attr_state
+                if old_attr_state is not None: # type: ignore
+                    _, images = old_attr_state # type: ignore
                     before = images.get_attributes(None, self.image_tag)
 
-                if self.restore_say_attributes(False, old_attr_state, interact):
-                    after = images.get_attributes(None, self.image_tag)
-                    self.handle_say_transition('restore', before, after)
+                if self.restore_say_attributes(False, old_attr_state, interact): # type: ignore
+                    after = images.get_attributes(None, self.image_tag) # type: ignore
+                    self.handle_say_transition('restore', before, after) # type: ignore
+
+    def statement_name(self):
+        if self._statement_name is not None:
+            return self._statement_name
+        elif not (self.condition is None or renpy.python.py_eval(self.condition)):
+            return "say-condition-false"
+        else:
+            return "say"
 
     def predict(self, what):
 
@@ -1173,7 +1308,7 @@ class ADVCharacter(object):
         old_side_image_attributes = renpy.store._side_image_attributes
 
         if self.image_tag:
-            attrs = ( self.image_tag, ) + renpy.game.context().images.get_attributes("master", self.image_tag)
+            attrs = (self.image_tag,) + renpy.game.context().images.get_attributes("master", self.image_tag)
         else:
             attrs = None
 
@@ -1210,31 +1345,31 @@ class ADVCharacter(object):
         if history_length is None:
             return
 
-        if not renpy.store._history:  # @UndefinedVariable
+        if not renpy.store._history: # type: ignore
             return
 
-        history = renpy.store._history_list  # @UndefinedVariable
+        history = renpy.store._history_list # type: ignore
 
         h = HistoryEntry()
 
-        h.kind = kind
+        h.kind = kind # type: ignore
 
         h.who = who
         h.what = what
 
-        h.who_args = self.who_args
-        h.what_args = self.what_args
-        h.window_args = self.window_args
-        h.show_args = self.show_args
+        h.who_args = self.who_args # type: ignore
+        h.what_args = self.what_args # type: ignore
+        h.window_args = self.window_args # type: ignore
+        h.show_args = self.show_args # type: ignore
 
-        h.image_tag = self.image_tag
+        h.image_tag = self.image_tag # type: ignore
 
         h.multiple = multiple
 
         if renpy.game.context().rollback:
-            h.rollback_identifier = renpy.game.log.current.identifier
+            h.rollback_identifier = renpy.game.log.current.identifier # type: ignore
         else:
-            h.rollback_identifier = None
+            h.rollback_identifier = None # type: ignore
 
         for k, v in kwargs.items():
             setattr(h, k, v)
@@ -1258,16 +1393,18 @@ class ADVCharacter(object):
         if history_length is None:
             return
 
-        if not renpy.store._history:  # @UndefinedVariable
+        if not renpy.store._history: # type: ignore
             return
 
-        renpy.store._history_list.pop()  # @UndefinedVariable
+        # The history can be reset at any time, so check that we have some.
+        if renpy.store._history_list: # type: ignore
+            renpy.store._history_list.pop() # type: ignore
 
 
 def Character(name=NotSet, kind=None, **properties):
     """
     :doc: character
-    :args: (name, kind=adv, **args)
+    :args: (name=..., kind=adv, **args)
     :name: Character
 
     Creates and returns a Character object, which controls the look
@@ -1275,8 +1412,9 @@ def Character(name=NotSet, kind=None, **properties):
 
     `name`
         If a string, the name of the character for dialogue. When
-        ``name`` is None, display of the name is omitted, as for
-        narration.
+        `name` is None, display of the name is omitted, as for
+        narration. If no name is given, the name is taken from
+        `kind`, and otherwise defaults to None.
 
     `kind`
         The Character to base this Character off of. When used, the
@@ -1329,9 +1467,13 @@ def Character(name=NotSet, kind=None, **properties):
     These options help to control the display of the name.
 
     `dynamic`
-        If true, then `name` should be a string containing a Python
-        expression. That string will be evaluated before each line
-        of dialogue, and the result used as the name of the character.
+        If true, then `name` should either be a string containing a Python
+        expression, a function, or a callable object. If it's a string,
+        That string will be evaluated before each line of dialogue, and
+        the result used as the name of the character. Otherwise, the
+        function or callable object will be called with no arguments
+        before each line of dialogue, and the return value of the call will
+        be used as the name of the character.
 
     **Controlling Interactions.**
     These options control if the dialogue is displayed, if an
@@ -1361,7 +1503,7 @@ def Character(name=NotSet, kind=None, **properties):
     `callback`
         A function that is called when events occur while the
         character is speaking. See the section on
-        :ref:`character-callbacks` fore more information.
+        :ref:`character-callbacks` for more information.
 
     **Click-to-continue.**
     A click-to-continue indicator is displayed once all the text has
