@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -19,12 +19,15 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
+
+
 from renpy.minstore import *
 
 # But please note that this will not be available in the body
 # of user code, unless we re-import it.
-import renpy.display
-import renpy.text
+import renpy
 
 import renpy.display.im as im
 import renpy.display.anim as anim
@@ -45,6 +48,9 @@ _window_subtitle = ''
 # Should rollback be allowed?
 _rollback = True
 
+# Should beginning a new rollback be allowed?
+_begin_rollback = True
+
 # Should skipping be allowed?
 _skipping = True
 
@@ -65,6 +71,12 @@ _text_rect = None
 _menu = False
 main_menu = False
 
+# Is autosaving allowed?
+_autosave = True
+
+# Should live2d fading happen?
+_live2d_fade = True
+
 
 class _Config(object):
 
@@ -74,7 +86,7 @@ class _Config(object):
     def __setstate__(self, data):
         return
 
-    def register(self, name, default, cat=None, help=None):  # @ReservedAssignment
+    def register(self, name, default, cat=None, help=None): # @ReservedAssignment
         setattr(self, name, default)
         _config.help.append((cat, name, help))
 
@@ -93,7 +105,7 @@ class _Config(object):
             raise Exception('config.%s is not a known configuration variable.' % (name))
 
         if name == "script_version":
-            renpy.store._set_script_version(value)  # E1101 @UndefinedVariable
+            renpy.store._set_script_version(value) # type: ignore
 
         if name == "developer":
             if value == "auto":
@@ -110,18 +122,21 @@ class _Config(object):
             delattr(renpy.config, name)
 
 
+
 # The styles object.
 style = None
 
 config = _Config()
 library = config
 
-eval = renpy.python.py_eval  # @ReservedAssignment
+eval = renpy.python.py_eval # @ReservedAssignment
 
 # Displayables.
 Bar = renpy.display.behavior.Bar
 Button = renpy.display.behavior.Button
+ImageButton = renpy.display.behavior.ImageButton
 Input = renpy.display.behavior.Input
+TextButton = renpy.display.behavior.TextButton
 
 ImageReference = renpy.display.image.ImageReference
 DynamicImage = renpy.display.image.DynamicImage
@@ -136,6 +151,11 @@ FileCurrentScreenshot = renpy.display.imagelike.FileCurrentScreenshot
 LiveComposite = renpy.display.layout.LiveComposite
 LiveCrop = renpy.display.layout.LiveCrop
 LiveTile = renpy.display.layout.LiveTile
+
+Composite = renpy.display.layout.Composite
+Crop = renpy.display.layout.Crop
+Tile = renpy.display.layout.Tile
+
 Flatten = renpy.display.layout.Flatten
 
 Null = renpy.display.layout.Null
@@ -163,6 +183,11 @@ DragGroup = renpy.display.dragdrop.DragGroup
 Sprite = renpy.display.particle.Sprite
 SpriteManager = renpy.display.particle.SpriteManager
 
+Matrix = renpy.display.matrix.Matrix # @UndefinedVariable
+
+Live2D = renpy.gl2.live2d.Live2D
+
+Model = renpy.display.model.Model
 
 # Currying things.
 Alpha = renpy.curry.curry(renpy.display.layout.Alpha)
@@ -183,7 +208,6 @@ CropMove = renpy.curry.curry(renpy.display.transition.CropMove)
 PushMove = renpy.curry.curry(renpy.display.transition.PushMove)
 Pixellate = renpy.curry.curry(renpy.display.transition.Pixellate)
 
-
 OldMoveTransition = renpy.curry.curry(renpy.display.movetransition.OldMoveTransition)
 MoveTransition = renpy.curry.curry(renpy.display.movetransition.MoveTransition)
 MoveFactory = renpy.curry.curry(renpy.display.movetransition.MoveFactory)
@@ -196,6 +220,7 @@ MultipleTransition = renpy.curry.curry(renpy.display.transition.MultipleTransiti
 ComposeTransition = renpy.curry.curry(renpy.display.transition.ComposeTransition)
 Pause = renpy.curry.curry(renpy.display.transition.NoTransition)
 SubTransition = renpy.curry.curry(renpy.display.transition.SubTransition)
+
 # Misc.
 ADVSpeaker = ADVCharacter = renpy.character.ADVCharacter
 Speaker = Character = renpy.character.Character
@@ -205,15 +230,15 @@ MultiPersistent = renpy.persistent.MultiPersistent
 Action = renpy.ui.Action
 BarValue = renpy.ui.BarValue
 
+AudioData = renpy.audio.audio.AudioData
+
 # NOTE: When exporting something from here, decide if we need to add it to
 # renpy.pyanalysis.pure_functions.
 
-Style = renpy.style.Style  # @UndefinedVariable
-
-absolute = renpy.display.core.absolute
-
-NoRollback = renpy.python.NoRollback
+Style = renpy.style.Style # type: ignore
 
+SlottedNoRollback = renpy.rollback.SlottedNoRollback
+NoRollback = renpy.rollback.NoRollback
 
 class _layout_class(__builtins__["object"]):
     """
@@ -243,6 +268,7 @@ class _layout_class(__builtins__["object"]):
 
 
 Fixed = _layout_class(renpy.display.layout.MultiBox, """
+:name: Fixed
 :doc: disp_box
 :args: (*args, **properties)
 
@@ -296,16 +322,17 @@ def AlphaBlend(control, old, new, alpha=False):
 def At(d, *args):
     """
     :doc: disp_at
+    :name: At
 
     Given a displayable `d`, applies each of the transforms in `args`
     to it. The transforms are applied in left-to-right order, so that
     the outermost transform is the rightmost argument. ::
 
         transform birds_transform:
-             xpos -200
-             linear 10 xpos 800
-             pause 20
-             repeat
+            xpos -200
+            linear 10 xpos 800
+            pause 20
+            repeat
 
         image birds = At("birds.png", birds_transform)
         """
@@ -326,12 +353,9 @@ def At(d, *args):
 Color = renpy.color.Color
 color = renpy.color.Color
 
-# Conveniently get rid of all the packages we had imported before.
-import renpy.exports as renpy  # @Reimport
-
 # The default menu functions.
-menu = renpy.display_menu
-predict_menu = renpy.predict_menu
+menu = renpy.exports.display_menu
+predict_menu = renpy.exports.predict_menu
 
 # The default transition.
 default_transition = None
@@ -344,40 +368,40 @@ suppress_overlay = False
 
 # The default ADVCharacter.
 adv = ADVCharacter(None,
-                   who_prefix='',
-                   who_suffix='',
-                   what_prefix='',
-                   what_suffix='',
+                who_prefix='',
+                who_suffix='',
+                what_prefix='',
+                what_suffix='',
 
-                   show_function=renpy.show_display_say,
-                   predict_function=renpy.predict_show_display_say,
+                show_function=renpy.exports.show_display_say,
+                predict_function=renpy.exports.predict_show_display_say,
 
-                   condition=None,
-                   dynamic=False,
-                   image=None,
+                condition=None,
+                dynamic=False,
+                image=None,
 
-                   interact=True,
-                   slow=True,
-                   slow_abortable=True,
-                   afm=True,
-                   ctc=None,
-                   ctc_pause=None,
-                   ctc_timedpause=None,
-                   ctc_position="nestled",
-                   all_at_once=False,
-                   with_none=None,
-                   callback=None,
-                   type='say',
-                   advance=True,
+                interact=True,
+                slow=True,
+                slow_abortable=True,
+                afm=True,
+                ctc=None,
+                ctc_pause=None,
+                ctc_timedpause=None,
+                ctc_position="nestled",
+                all_at_once=False,
+                with_none=None,
+                callback=None,
+                type='say',
+                advance=True,
 
-                   who_style='say_label',
-                   what_style='say_dialogue',
-                   window_style='say_window',
-                   screen='say',
-                   mode='say',
-                   voice_tag=None,
+                who_style='say_label',
+                what_style='say_dialogue',
+                window_style='say_window',
+                screen='say',
+                mode='say',
+                voice_tag=None,
 
-                   kind=False)
+                kind=False)
 
 # predict_say and who are defined in 00library.rpy, but we add default
 # versions here in case there is a problem with initialization. (And
@@ -388,7 +412,7 @@ def predict_say(who, what):
     who = Character(who, kind=adv)
     try:
         who.predict(what)
-    except:
+    except Exception:
         pass
 
 
@@ -421,6 +445,7 @@ _in_replay = None
 
 # Used to store the side image attributes.
 _side_image_attributes = None
+_side_image_attributes_reset = False
 
 # True if we're in the main_menu, False otherwise. This controls autosave,
 # among other things.
@@ -430,19 +455,14 @@ main_menu = False
 # error handling screen.
 _ignore_action = None
 
+# The save slot that Ren'Py saves to on quit.
+_quit_slot = None
+
+# The directory in which Ren'Py saves screenshots.
+_screenshot_pattern = None
+
 # Make these available to user code.
 import sys
 import os
 
-
-def public_api():
-    ui
-    im
-    object
-    range
-    sorted
-    os
-    sys
-
-
-del public_api
+globals()["renpy"] = renpy.exports
