@@ -1,4 +1,4 @@
-# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,14 +21,14 @@
 
 # This file contains support for state-machine controlled animations.
 
-from __future__ import print_function
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
 
-import renpy.display
-import renpy.game
-import renpy.easy
 
 import random
 
+import renpy
+
 
 class State(object):
     """
@@ -121,7 +121,7 @@ class Edge(object):
         self.prob = prob
 
     def add(self, sma):
-        for _i in xrange(0, self.prob):
+        for _i in range(0, self.prob):
             sma.edges.setdefault(self.old, []).append(self)
 
 
@@ -202,7 +202,7 @@ class SMAnimation(renpy.display.core.Displayable):
         self.state = None
 
     def visit(self):
-        return [ i.image for i in self.states.itervalues() ]
+        return [ i.image for i in self.states.values() ]
 
     def pick_edge(self, state):
         """
@@ -302,76 +302,15 @@ class SMAnimation(renpy.display.core.Displayable):
 
         args = [ ]
 
-        for state in self.states.itervalues():
+        for state in self.states.values():
             args.append(state.motion_copy(child))
 
-        for edges in self.edges.itervalues():
+        for edges in self.edges.values():
             args.extend(edges)
 
         return SMAnimation(self.initial, delay=self.delay, *args, **self.properties)
 
 
-# class Animation(renpy.display.core.Displayable):
-#     """
-#     A Displayable that draws an animation, which is a series of images
-#     that are displayed with time delays between them.
-#     """
-
-#     def __init__(self, *args, **properties):
-#         """
-#         Odd (first, third, fifth, etc.) arguments to Animation are
-#         interpreted as image filenames, while even arguments are the
-#         time to delay between each image. If the number of arguments
-#         is odd, the animation will stop with the last image (well,
-#         actually delay for a year before looping). Otherwise, the
-#         animation will restart after the final delay time.
-
-#         @param anim_timebase: If True, the default, use the animation
-#         timebase. Otherwise, use the displayable timebase.
-#         """
-
-#         properties.setdefault('style', 'animation')
-#         self.anim_timebase = properties.pop('anim_timebase', True)
-
-#         super(Animation, self).__init__(**properties)
-
-#         self.images = [ ]
-#         self.delays = [ ]
-
-#         for i, arg in enumerate(args):
-
-#             if i % 2 == 0:
-#                 self.images.append(renpy.easy.displayable(arg))
-#             else:
-#                 self.delays.append(arg)
-
-#         if len(self.images) > len(self.delays):
-#             self.delays.append(365.25 * 86400.0) # One year, give or take.
-
-#     def render(self, width, height, st, at):
-
-#         if self.anim_timebase:
-#             t = at % sum(self.delays)
-#         else:
-#             t = st % sum(self.delays)
-
-#         for image, delay in zip(self.images, self.delays):
-#             if t < delay:
-#                 renpy.display.render.redraw(self, delay - t)
-
-#                 im = renpy.display.render.render(image, width, height, t, at)
-#                 width, height = im.get_size()
-#                 rv = renpy.display.render.Render(width, height)
-#                 rv.blit(im, (0, 0))
-
-#                 return rv
-
-#             else:
-#                 t = t - delay
-
-#     def visit(self):
-#         return self.images
-
 def Animation(*args, **kwargs):
     newargs = [ ]
 
@@ -430,7 +369,7 @@ class TransitionAnimation(renpy.display.core.Displayable):
                 transitions.append(arg)
 
         if len(images) > len(delays):
-            delays.append(365.25 * 86400.0)  # One year, give or take.
+            delays.append(365.25 * 86400.0) # One year, give or take.
         if len(images) > len(transitions):
             transitions.append(None)
 
@@ -474,7 +413,7 @@ class Blink(renpy.display.core.Displayable):
     """
     """
 
-    def __init__(self, image, on=0.5, off=0.5, rise=0.5, set=0.5,  # @ReservedAssignment
+    def __init__(self, image, on=0.5, off=0.5, rise=0.5, set=0.5, # @ReservedAssignment
                  high=1.0, low=0.0, offset=0.0, anim_timebase=False, **properties):
         """
         This takes as an argument an image or widget, and blinks that image
@@ -521,6 +460,8 @@ class Blink(renpy.display.core.Displayable):
 
     def render(self, height, width, st, at):
 
+        delay = 0
+
         if self.anim_timebase:
             t = at
         else:
@@ -560,6 +501,10 @@ class Blink(renpy.display.core.Displayable):
         rv.blit(rend, (0, 0))
         rv.alpha = alpha
 
+        rv.add_shader("renpy.alpha")
+        rv.add_uniform("u_renpy_alpha", alpha)
+        rv.add_uniform("u_renpy_over", 1.0)
+
         if not renpy.game.less_updates:
             renpy.display.render.redraw(self, delay)
 
