@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -22,15 +22,17 @@
 # TODO: Use overlap (rather than simple pointer location) to determine
 # drag and drop.
 
-from __future__ import print_function
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
 
-import renpy.display
+
+import pygame_sdl2 as pygame
+
+import renpy
 from renpy.display.render import render, Render, redraw
 from renpy.display.core import absolute
 from renpy.display.behavior import map_event, run, run_unhovered
 
-import pygame_sdl2 as pygame
-
 
 def default_drag_group():
     """
@@ -47,16 +49,19 @@ def default_drag_group():
 
     return rv
 
+
 def default_drag_joined(drag):
     return [ (drag, 0, 0) ]
 
+
 def default_drop_allowable(drop, drags):
     return True
 
-class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
+
+class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
     """
     :doc: drag_drop class
-    :args: (d=None, drag_name=None, draggable=True, droppable=True, drag_raise=True, dragged=None, dropped=None, drag_handle=(0.0, 0.0, 1.0, 1.0), drag_joined=..., clicked=None, hovered=None, unhovered=None, mouse_drop=False, **properties)
+    :args: (d=None, drag_name=None, draggable=True, droppable=True, drag_raise=True, dragging=None, dragged=None, dropped=None, drag_handle=(0.0, 0.0, 1.0, 1.0), drag_joined=..., clicked=None, hovered=None, unhovered=None, mouse_drop=False, **properties)
 
     A displayable that represents an object that can be dragged around
     its enclosing area. A Drag can also represent an area that
@@ -75,8 +80,8 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
     * ``idle`` - otherwise.
 
     The drag handle is a rectangle inside the child. The mouse must be over
-    a non-transparent pixel inside the drag handle for dragging or clicking
-    to occur.
+    a pixel inside the drag handle for dragging or clicking to occur. If the
+    :propref:`focus_mask` property is True, that pixel must not be transparent.
 
     A newly-created draggable is added to the default DragGroup. A draggable
     can only be in a single DragGroup - if it's added to a second group,
@@ -85,7 +90,8 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
     When a Drag is first rendered, if it's position cannot be determined
     from the DragGroup it is in, the position of its upper-left corner
     is computed using the standard layout algorithm. Once that position
-
+    has been computed, the layout properties are ignored in favor of the
+    position stored inside the Drag.
 
     `d`
         If present, the child of this Drag. Drags use the child style
@@ -114,6 +120,12 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
         a list of Drags that are being dragged. The return value of this
         callback is ignored.
 
+    `dragging`
+        A callback (or list of callbacks) that is called when the Drag is being
+        dragged. It is called with one argument, a list of Drags that are
+        being dragged. If the callback returns a value other than None, that
+        value is returned as the result of the interaction.
+
     `dragged`
         A callback (or list of callbacks) that is called when the Drag
         has been dragged. It is called with two arguments. The first is
@@ -180,12 +192,17 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
     been rendered, the following fields become available:
 
     `x`, `y`
-         The position of the Drag relative to its parent, in pixels.
+        The position of the Drag relative to its parent, in pixels.
+
+    `start_x`, `start_y`
+        The drag start position of the Drag relative to its parent, in pixels.
 
     `w`, `h`
-         The width and height of the Drag's child, in pixels.
+        The width and height of the Drag's child, in pixels.
     """
 
+    z = 0
+
     focusable = True
 
     drag_group = None
@@ -193,6 +210,7 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
     drag_offscreen = False
     activated = None
     alternate = None
+    dragging = None
 
     # The time a click started, or None if a click is not in progress.
     click_time = None
@@ -217,6 +235,7 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
                  activated=None,
                  alternate=None,
                  style="drag",
+                 dragging=None,
                  **properties):
 
         super(Drag, self).__init__(style=style, **properties)
@@ -225,6 +244,7 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
         self.draggable = draggable
         self.droppable = droppable
         self.drag_raise = drag_raise
+        self.dragging = dragging
         self.dragged = dragged
         self.dropped = dropped
         self.drop_allowable = drop_allowable
@@ -294,12 +314,15 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
         self.target_at_delay = 0
 
         # The displayable we were last dropping on.
-        self.last_drop = None
+        self.last_drop = None # type: renpy.display.core.Displayable|None
 
         # Did we move over the course of this drag?
         self.drag_moved = False
 
-        if replaces is not None:
+        # A z index that's changed when something is raised or lowered.
+        self.z = 0
+
+        if isinstance(replaces, Drag):
             self.x = replaces.x
             self.y = replaces.y
             self.at = replaces.at
@@ -316,6 +339,7 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
             self.last_drop = replaces.last_drop
             self.mouse_drop = replaces.mouse_drop
             self.click_time = replaces.click_time
+            self.z = replaces.z
 
         if d is not None:
             self.add(d)
@@ -330,10 +354,10 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
         linear move.
         """
 
-        if type(x) is float:
+        if (type(x) is float) and self.parent_width is not None:
             x = int(x * self.parent_width)
 
-        if type(y) is float:
+        if (type(y) is float) and self.parent_height is not None:
             y = int(y * self.parent_height)
 
         self.target_x = x
@@ -399,12 +423,31 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
         if self.drag_group is not None:
             self.drag_group.lower_children([ self ])
 
+    def update_style_prefix(self):
+        """
+        This updates the style prefix for all Drag's associated
+        with this drag movement.
+        """
+        # We may not be in the drag_joined group.
+        self.set_style_prefix("idle_", True)
+
+        # Set the style for joined_set
+        for i in [i[0] for i in self.drag_joined(self)]:
+            i.set_style_prefix("selected_hover_", True)
+
+        if self.last_drop is not None:
+            self.last_drop.set_style_prefix("selected_idle_", True)
+
     def visit(self):
         return [ self.child ]
 
     def focus(self, default=False):
         super(Drag, self).focus(default)
 
+        # Update state back after restart_interaction
+        if default and self.drag_moved:
+            self.update_style_prefix()
+
         rv = None
 
         if not default:
@@ -425,8 +468,8 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
         if child is None:
             child = self.child
 
-        self.parent_width = width
-        self.parent_height = height
+        self.parent_width = renpy.display.render.render_width
+        self.parent_height = renpy.display.render.render_height
 
         cr = render(child, width, height, st, at)
         cw, ch = cr.get_size()
@@ -489,8 +532,8 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
         else:
             # Snap in progress
             done = (at - self.at) / (self.target_at - self.at)
-            self.x = absolute(self.x + done * (self.target_x - self.x))
-            self.y = absolute(self.y + done * (self.target_y - self.y))
+            self.x = absolute(self.x + done * (self.target_x - self.x)) # type: ignore
+            self.y = absolute(self.y + done * (self.target_y - self.y)) # type: ignore
             redraw(self, 0)
 
         if self.draggable or self.clicked is not None:
@@ -516,7 +559,7 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
             elif mask is not None:
                 try:
                     mask = renpy.display.render.render(mask, fw, fh, st, at)
-                except:
+                except Exception:
                     if callable(mask):
                         mask = mask
                     else:
@@ -552,14 +595,16 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
             self.click_time = st
             renpy.game.interface.timeout(renpy.config.longpress_duration)
 
+        joined = [ ] # typing
+
         if grabbed:
             joined_offsets = self.drag_joined(self)
-            joined = [ i[0] for i in joined_offsets ]
+            joined = [ i[0] for i in joined_offsets ] # type: list[Drag]
 
         elif self.draggable and map_event(ev, "drag_activate"):
 
             joined_offsets = self.drag_joined(self)
-            joined = [ i[0] for i in joined_offsets ]
+            joined = [ i[0] for i in joined_offsets ] # type: list[Drag]
 
             if not joined:
                 raise renpy.display.core.IgnoreEvent()
@@ -585,6 +630,8 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
 
             grabbed = True
 
+            renpy.exports.play(self.style.activate_sound)
+
         elif (self.alternate is not None) and map_event(ev, "button_alternate"):
             rv = run(self.alternate)
             if rv is not None:
@@ -592,10 +639,12 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
 
             raise renpy.display.core.IgnoreEvent()
 
-        if ((self.alternate is not None) and
+        if (
+            (self.alternate is not None) and
             renpy.display.touch and
             (self.click_time is not None) and
-                ((st - self.click_time) > renpy.config.longpress_duration)):
+            ((st - self.click_time) > renpy.config.longpress_duration)
+        ):
 
             self.click_time = None
 
@@ -603,6 +652,8 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
             if rv is not None:
                 return rv
 
+            renpy.exports.vibrate(renpy.config.longpress_vibrate)
+
         # Handle clicking on droppables.
         if not grabbed:
             if self.clicked is not None and map_event(ev, "drag_deactivate"):
@@ -622,26 +673,19 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
 
             handled = True
 
-            if not self.drag_moved and (self.start_x != par_x or self.start_y != par_y):
+            if (not self.drag_moved) and (self.start_x != par_x or self.start_y != par_y):
                 self.drag_moved = True
                 self.click_time = None
 
-                # We may not be in the drag_joined group.
-                self.set_style_prefix("idle_", True)
-
-                # Set the style.
-                for i in joined:
-                    i.set_style_prefix("selected_hover_", True)
-
                 # Raise the joined items.
                 if self.drag_raise and self.drag_group is not None:
-                    self.drag_group.raise_children(joined)
+                    self.drag_group.raise_children(joined) # type: ignore
 
             if self.drag_moved:
-                for i, xo, yo in joined_offsets:
+                for i, xo, yo in joined_offsets: # type: ignore
 
-                    new_x = int(par_x - self.grab_x + xo)
-                    new_y = int(par_y - self.grab_y + yo)
+                    new_x = int(par_x - self.grab_x + xo) # type: ignore
+                    new_y = int(par_y - self.grab_y + yo) # type: ignore
 
                     if not self.drag_offscreen:
                         new_x = max(new_x, 0)
@@ -657,6 +701,12 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
                     i.target_x = new_x
                     i.target_y = new_y
                     i.target_at = self.at
+                    # Call the dragging callback.
+                    drag = joined[0]
+                    if drag.dragging is not None:
+                        rv = run(drag.dragging, joined)
+                        if rv is not None:
+                            return rv
                     redraw(i, 0)
 
         else:
@@ -664,9 +714,9 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
 
         if (self.drag_group is not None) and self.drag_moved:
             if self.mouse_drop:
-                drop = self.drag_group.get_drop_at(joined, par_x, par_y)
+                drop = self.drag_group.get_drop_at(joined, par_x, par_y) # type: ignore
             else:
-                drop = self.drag_group.get_best_drop(joined)
+                drop = self.drag_group.get_best_drop(joined) # type: ignore
         else:
             drop = None
 
@@ -675,10 +725,10 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
             if self.last_drop is not None:
                 self.last_drop.set_style_prefix("idle_", True)
 
-            if drop is not None:
-                drop.set_style_prefix("selected_idle_", True)
+            self.last_drop = drop # type: ignore
 
-            self.last_drop = drop
+        if self.drag_moved:
+            self.update_style_prefix()
 
         if map_event(ev, 'drag_deactivate'):
 
@@ -687,7 +737,7 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
             renpy.display.focus.set_grab(None)
 
             if drop is not None:
-                drop.set_style_prefix("idle_", True)
+                drop.set_style_prefix("idle_", True) # type: ignore
 
             for i in joined:
                 i.set_style_prefix("idle_", True)
@@ -708,8 +758,8 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
                         return rv
 
                 # Call the drop callback.
-                if drop is not None and drop.dropped is not None:
-                    rv = run(drop.dropped, drop, joined)
+                if drop is not None and drop.dropped is not None: # type: ignore
+                    rv = run(drop.dropped, drop, joined) # type: ignore
                     if rv is not None:
                         return rv
 
@@ -753,13 +803,13 @@ class DragGroup(renpy.display.layout.MultiBox):
 
     `min_overlap`
         An integer which means the minimum number of pixels at the
-        overlap so that drop will be allow.
+        overlap for the drop to be allowed.
     """
 
-    _list_type = renpy.python.RevertableList
+    z_serial = 0
+    sorted = False
 
-    def __unicode__(self):
-        return "DragGroup"
+    _list_type = renpy.revertable.RevertableList
 
     def __init__(self, *children, **properties):
         properties.setdefault("style", "fixed")
@@ -772,12 +822,16 @@ class DragGroup(renpy.display.layout.MultiBox):
 
         super(DragGroup, self).__init__(**properties)
 
-        if replaces is not None:
-            self.positions = renpy.python.RevertableDict(replaces.positions)
+        self.sorted = False
+
+        if isinstance(replaces, DragGroup):
+            self.positions = renpy.revertable.RevertableDict(replaces.positions)
             self.sensitive = replaces.sensitive
+            self.z_serial = replaces.z_serial
         else:
-            self.positions = renpy.python.RevertableDict()
+            self.positions = renpy.revertable.RevertableDict()
             self.sensitive = True
+            self.z_serial = 0
 
         for i in children:
             self.add(i)
@@ -792,9 +846,10 @@ class DragGroup(renpy.display.layout.MultiBox):
         if not isinstance(child, Drag):
             raise Exception("Only drags can be added to a drag group.")
 
-        child.drag_group = self
         super(DragGroup, self).add(child)
 
+        self.sorted = False
+
     def remove(self, child):
         """
         :doc: drag_drop method
@@ -808,6 +863,17 @@ class DragGroup(renpy.display.layout.MultiBox):
         child.x = None
         super(DragGroup, self).remove(child)
 
+    def render(self, width, height, st, at):
+
+        for i in self.children:
+            i.drag_group = self
+
+        if not self.sorted:
+            self.children.sort(key=lambda i : i.z)
+            self.sorted = True
+
+        return super(DragGroup, self).render(width, height, st, at)
+
     def event(self, ev, x, y, st):
 
         if not self.sensitive:
@@ -817,50 +883,36 @@ class DragGroup(renpy.display.layout.MultiBox):
 
     def raise_children(self, l):
         """
-        Raises the children in `l` to the top of this drag_group, using the
-        order given in l for those children.
+        Raises the children in the list `l` to the top of this drag group.
+        Each is raised in the order that it appears in `l`, which means that
+        the last element of `l` will be raised closest to the player.
         """
 
-        s = set(l)
+        self.sorted = False
 
-        offset_map = { }
+        for i in l:
+            self.z_serial += 1
+            i.z = self.z_serial
 
-        children = [ ]
-        offsets = [ ]
-
-        for i, c in enumerate(self.children):
-            if i < len(self.offsets):
-                o = self.offsets[i]
-            else:
-                o = (0, 0)
-
-            if c not in s:
-                children.append(c)
-                offsets.append(o)
-            else:
-                offset_map[c] = o
-
-        for c in l:
-            if c in offset_map:
-                children.append(c)
-                offsets.append(offset_map[c])
-
-        self.children = self._list_type(children)
-        self.offsets = self._list_type(offsets)
+        renpy.display.render.redraw(self, 0)
 
     def lower_children(self, l):
         """
+        Lowers the children in the list `l` to the bottom of this drag group.
+        Each is lowered in the order that it appears in `l`, which means that
+        the last element of `l` will be the lowest of the children.
+
         Lowers the children in `l` to the bottom of this drag group, with
         the one at the bottom being the lowest.
         """
 
-        self.children.reverse()
-        self.offsets.reverse()
+        self.sorted = False
 
-        self.raise_children(l)
+        for i in l:
+            self.z_serial += 1
+            i.z = -self.z_serial
 
-        self.children.reverse()
-        self.offsets.reverse()
+        renpy.display.render.redraw(self, 0)
 
     def get_best_drop(self, joined):
         """
@@ -932,14 +984,14 @@ class DragGroup(renpy.display.layout.MultiBox):
         this DragGroup.
         """
 
-        return renpy.python.RevertableList(self.children)
+        return renpy.revertable.RevertableList(self.children)
 
     def get_child_by_name(self, name):
         """
         :doc: drag_drop method
 
         Returns the first child of this DragGroup that has a drag_name
-        of name.
+        of `name`.
         """
 
         for i in self.children:
