@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,12 +21,15 @@
 
 # This file contains code to emulate various other devices on the PC.
 
-from __future__ import print_function
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
 
-import renpy.display
 
 import os
+
 import pygame_sdl2 as pygame
+import renpy
+
 
 # The function that's called to perform the emulation. This function has
 # the signature of null_emulator.
@@ -38,9 +41,6 @@ overlay = [ ]
 # True if we're in ios mode, where we don't allow keys.
 ios = False
 
-# True if the framebuffer doesn't support alpha.
-always_opaque = False
-
 
 def null_emulator(ev, x, y):
     """
@@ -125,7 +125,7 @@ def dynamic_keyboard(st, at):
 
     if keyboard is None:
         keyboard = renpy.store.Fixed(
-            renpy.store.Solid("#000", yalign=1.0, ymaximum=.625),
+            renpy.store.Solid("#0008", yalign=1.0, ymaximum=.625),
             renpy.store.Text("On-Screen Keyboard", xalign=.5, yalign=.75),
             )
         null = renpy.store.Null()
@@ -146,19 +146,18 @@ def init_emulator():
     global emulator
     global overlay
     global ios
-    global always_opaque
 
     name = os.environ.get("RENPY_EMULATOR", "")
 
     if name == "touch":
         emulator = touch_emulator
         overlay = [ renpy.store.DynamicDisplayable(dynamic_keyboard) ]
-        always_opaque = True
+
     elif name == "ios-touch":
         emulator = touch_emulator
         overlay = [ renpy.store.DynamicDisplayable(dynamic_keyboard) ]
         ios = True
-        always_opaque = True
+
     elif name == "tv":
         emulator = tv_emulator
         overlay = [ renpy.display.motion.Transform(
@@ -167,8 +166,22 @@ def init_emulator():
             yalign=0.5,
             size=(int(renpy.config.screen_height * 16.0 / 9.0), renpy.config.screen_height),
             ) ]
-        always_opaque = True
+
     else:
         emulator = null_emulator
         overlay = [ ]
-        always_opaque = False
+
+
+def early_init_emulator():
+
+    name = os.environ.get("RENPY_EMULATOR", "")
+
+    if name:
+
+        renpy.exports.windows = False # type: ignore
+        renpy.exports.linux = False # type: ignore
+        renpy.exports.macintosh = False # type: ignore
+        renpy.exports.web = False # type: ignore
+        renpy.exports.android = renpy.exports.variant("android") # type: ignore
+        renpy.exports.ios = renpy.exports.variant("ios") # type: ignore
+        renpy.exports.mobile = renpy.exports.android or renpy.exports.ios  # type: ignore
