@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,8 +21,63 @@
 
 # This file contains code to manage focus on the display.
 
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
+
+
+import operator
+
 import pygame_sdl2 as pygame
-import renpy.display
+import renpy
+
+# The focus storage api.
+
+focus_storage = { }
+
+def capture_focus(name="default"):
+    """
+    :doc: other
+
+    If a displayable is currently focused, captured the rectangular bounding
+    box of that displayable, and stores it with `name`. If not, removes any
+    focus stored with `name`.
+
+    Captured focuses are not saved when the game is saveed.
+
+    `name`
+        Should be a string. The name "tooltip" is special, as it's
+        automatically captured when a displayable with a tooltip gains focus.
+    """
+
+    rect = focus_coordinates()
+
+    if rect[0] is None:
+        rect = None
+
+    if rect is not None:
+        focus_storage[name] = rect
+    else:
+        focus_storage.pop(name, None)
+
+
+def clear_capture_focus(name="default"):
+    """
+    :doc: other
+
+    Clear the captured focus with `name`.
+    """
+
+    focus_storage.pop(name, None)
+
+
+def get_focus_rect(name="default"):
+    """
+    :undocumented:
+
+    Returns the captured focus. Used in the implementation of FocusRect.
+    """
+
+    return focus_storage.get(name, None)
 
 
 class Focus(object):
@@ -57,6 +112,25 @@ class Focus(object):
             self.h,
             self.screen)
 
+    def inside(self, rect):
+        """
+        Returns true if this focus is entirely contained inside the given
+        rectangle.
+        """
+
+        minx, miny, w, h = rect
+
+        maxx = minx + w
+        maxy = miny + h
+
+        if self.x is None:
+            return False
+
+        if (minx <= self.x < maxx) and (miny <= self.y < maxy) and (minx <= self.x + self.w < maxx) and (miny <= self.y + self.h < maxy):
+            return True
+
+        return False
+
 
 # The current focus argument.
 argument = None
@@ -64,11 +138,17 @@ argument = None
 # The screen of the currently focused widget.
 screen_of_focused = None
 
+# The names of the current focused screen.
+screen_of_focused_names = set()
+
+# The names of the last focused screen.
+screen_of_last_focused_names = set()
+
 # The widget currently grabbing the input, if any.
 grab = None
 
 # The default focus for the current screen.
-default_focus = None
+global_focus = None
 
 # The type of input that caused the focus to change last. One of
 # "keyboard" (for keyboard-like focus devices) or "mouse" (for mouse-like)
@@ -79,37 +159,52 @@ focus_type = "mouse"
 # the focus to change.
 pending_focus_type = "mouse"
 
-
 # The current tooltip and tooltip screen.
 tooltip = None
 
-# Sets the currently focused widget.
+# The last tooltip that was not None.
+last_tooltip = None
+
+# Overrides the currently focused displayable.
+override = None
 
 
 def set_focused(widget, arg, screen):
     global argument
-    argument = arg
-
     global screen_of_focused
+    global screen_of_focused_names
+    global tooltip
+    global last_tooltip
+    global screen_of_last_focused_names
+
+    argument = arg
     screen_of_focused = screen
 
+    if screen is not None:
+        screen_of_focused_names = { screen.screen_name[0], screen.tag }
+    else:
+        screen_of_focused_names = set()
+
     renpy.game.context().scene_lists.focused = widget
 
     renpy.display.tts.displayable(widget)
 
-    global tooltip
-
     # Figure out the tooltip.
 
     if widget is None:
         new_tooltip = None
     else:
-        new_tooltip = widget._tooltip
+        new_tooltip = widget._get_tooltip()
 
     if tooltip != new_tooltip:
         tooltip = new_tooltip
+        capture_focus("tooltip")
         renpy.exports.restart_interaction()
 
+        if tooltip is not None:
+            last_tooltip = tooltip
+            screen_of_last_focused_names = screen_of_focused_names
+
 
 def get_focused():
     """
@@ -131,22 +226,25 @@ def get_mouse():
         return focused.style.mouse
 
 
-def get_tooltip(screen=None):
+def get_tooltip(screen=None, last=False):
     """
     Gets the tooltip information.
     """
 
     if screen is None:
-        return tooltip
+        if last:
+            return last_tooltip
+        else:
+            return tooltip
 
-    if screen_of_focused is None:
-        return None
 
-    if screen_of_focused.screen_name[0] == screen:
-        return tooltip
+    if last:
+        if screen in screen_of_last_focused_names:
+            return last_tooltip
 
-    if screen_of_focused.tag == screen:
-        return tooltip
+    else:
+        if screen in screen_of_focused_names:
+            return tooltip
 
     return None
 
@@ -174,15 +272,25 @@ def take_focuses():
 
     renpy.display.render.take_focuses(focus_list)
 
-    global default_focus
-    default_focus = None
+    global global_focus
+    global_focus = None
+
+    global grab
+
+    grab_found = False
 
     for f in focus_list:
         if f.x is None:
-            default_focus = f
+            global_focus = f
 
-    if (default_focus is not None) and (get_focused() is None):
-        change_focus(default_focus, True)
+        if f.widget is grab:
+            grab_found = True
+
+    if not grab_found:
+        grab = None
+
+    if (global_focus is not None) and (get_focused() is None):
+        change_focus(global_focus, True)
 
 
 def focus_coordinates():
@@ -206,6 +314,17 @@ def focus_coordinates():
 # A map from id(displayable) to the displayable that replaces it.
 replaced_by = { }
 
+# The modal generation - the number of times mark_modal has been called.
+# Only displayables in the latest modal generation can gain focus.
+modal_generation = 0
+
+# The maximum default_focus number found the last time before_interact
+# was called.
+old_max_default = 0
+
+def mark_modal():
+    global modal_generation
+    modal_generation += 1
 
 def before_interact(roots):
     """
@@ -213,17 +332,23 @@ def before_interact(roots):
     displayables.
     """
 
-    global new_grab
+    global override
     global grab
+    global modal_generation
+
+    modal_generation = 0
 
     # a list of focusable, name, screen tuples.
     fwn = [ ]
 
     def callback(f, n):
-        fwn.append((f, n, renpy.display.screen._current_screen))
+        fwn.append((f, n, renpy.display.screen._current_screen, modal_generation))
 
     for root in roots:
-        root.find_focusable(callback, None)
+        try:
+            root.find_focusable(callback, None)
+        except renpy.display.layout.IgnoreLayers:
+            pass
 
     # Assign a full name to each focusable.
 
@@ -233,7 +358,7 @@ def before_interact(roots):
 
     for fwn_tuple in fwn:
 
-        f, n, screen = fwn_tuple
+        f, n, screen, gen = fwn_tuple
 
         serial = namecount.get(n, 0)
         namecount[n] = serial + 1
@@ -249,6 +374,33 @@ def before_interact(roots):
 
     fwn = fwn2
 
+    # Determine the default, as determined by the current screen.
+    defaults = [ ]
+
+    for f, n, screen, gen in fwn:
+        if gen != modal_generation:
+            continue
+
+        if f.default:
+            defaults.append((f.default, f, screen))
+
+    if defaults:
+        if len(defaults) > 1:
+            defaults.sort(key=operator.itemgetter(0))
+
+        max_default, max_default_focus, max_default_screen = defaults[-1]
+
+    else:
+        max_default = 0
+        max_default_focus = None
+        max_default_screen = None
+
+    # Should we do the max_default logic?
+    should_max_default = (renpy.display.interface.last_event is None) or (renpy.display.interface.last_event.type not in [ pygame.MOUSEBUTTONDOWN, pygame.MOUSEBUTTONUP, pygame.MOUSEMOTION ])
+
+    # Is this an explicit change, using the override operation?
+    explicit = False
+
     # We assume id(None) is not in replaced_by.
     replaced_by.pop(None, None)
 
@@ -257,57 +409,86 @@ def before_interact(roots):
 
     current = get_focused()
     current = replaced_by.get(id(current), current)
+    old_current = current
 
+    # Update the grab.
+    grab = replaced_by.get(id(grab), None)
+
+    if override is not None:
+        d = renpy.exports.get_displayable(base=True, *override) # type: ignore
+
+        if (d is not None) and (current is not d) and not grab:
+            current = d
+            explicit = True
+
+    override = None
+
+    # When increasing the default focus, and the user is not using the mouse,
+    # switch to the default.
+    if should_max_default and (max_default > old_max_default):
+        current = max_default_focus
+        set_focused(max_default_focus, None, max_default_screen)
+
+    # Try to find the current focus.
     if current is not None:
         current_name = current.full_focus_name
 
-        for f, n, screen in fwn:
+        for f, n, screen, gen in fwn:
+
+            if gen != modal_generation:
+                continue
+
             if f.full_focus_name == current_name:
                 current = f
-                set_focused(f, None, screen)
+                set_focused(f, argument, screen)
                 break
         else:
             current = None
 
-    # Otherwise, focus the default widget, or nothing.
-    if current is None:
+    if grab is not None:
+        current = grab
 
-        for f, n, screen in fwn:
-            if f.default:
-                current = f
-                set_focused(f, None, screen)
-                break
-        else:
-            set_focused(None, None, None)
+    # If nothing has focus, focus the default if the highest priority has changed,
+    # or if the default is None.
+    if (should_max_default and (max_default > 0) and (current is None) and
+        (renpy.display.interface.start_interact or (max_default != old_max_default))):
+
+        current = max_default_focus
+        set_focused(max_default_focus, None, max_default_screen)
+
+    if current is None:
+        set_focused(None, None, None)
 
     # Finally, mark the current widget as the focused widget, and
     # all other widgets as unfocused.
-    for f, n, screen in fwn:
+    for f, n, screen, modal in fwn:
         if f is not current:
             renpy.display.screen.push_current_screen(screen)
             try:
-                f.unfocus(default=True)
+                if (f is old_current) and renpy.config.always_unfocus:
+                    f.unfocus(default=False)
+                else:
+                    f.unfocus(default=not explicit)
+
             finally:
                 renpy.display.screen.pop_current_screen()
 
     if current:
         renpy.display.screen.push_current_screen(screen_of_focused)
         try:
-            current.focus(default=True)
+            current.focus(default=not explicit)
         finally:
             renpy.display.screen.pop_current_screen()
 
-    # Update the grab.
-    grab = replaced_by.get(id(grab), None)
-
     # Clear replaced_by.
     replaced_by.clear()
 
-# This changes the focus to be the widget contained inside the new
-# focus object.
-
 
 def change_focus(newfocus, default=False):
+    """
+    Change the focus to the displayable in ``newfocus``.
+    """
+
     rv = None
 
     if grab:
@@ -371,16 +552,22 @@ def mouse_handler(ev, x, y, default=False):
 
     global pending_focus_type
 
+    if grab:
+        return
+
     if ev is not None:
         if ev.type not in (pygame.MOUSEMOTION, pygame.MOUSEBUTTONUP, pygame.MOUSEBUTTONDOWN):
             return
         else:
             pending_focus_type = "mouse"
 
-    new_focus = renpy.display.render.focus_at_point(x, y)
+    try:
+        new_focus = renpy.display.render.focus_at_point(x, y)
+    except renpy.display.layout.IgnoreLayers:
+        new_focus = None
 
     if new_focus is None:
-        new_focus = default_focus
+        new_focus = global_focus
 
     return change_focus(new_focus, default=default)
 
@@ -392,10 +579,13 @@ def mouse_handler(ev, x, y, default=False):
 def focus_extreme(xmul, ymul, wmul, hmul):
 
     max_focus = None
-    max_score = -(65536**2)
+    max_score = -(65536 ** 2)
 
     for f in focus_list:
 
+        if not f.widget.style.keyboard_focus:
+            continue
+
         if f.x is None:
             continue
 
@@ -415,8 +605,8 @@ def focus_extreme(xmul, ymul, wmul, hmul):
 # This calculates the distance between two points, applying
 # the given fudge factors. The distance is left squared.
 def points_dist(x0, y0, x1, y1, xfudge, yfudge):
-    return (( x0 - x1 ) * xfudge ) ** 2 + \
-           (( y0 - y1 ) * yfudge ) ** 2
+    return ((x0 - x1) * xfudge) ** 2 + \
+           ((y0 - y1) * yfudge) ** 2
 
 
 # This computes the distance between two horizontal lines. (So the
@@ -489,7 +679,15 @@ def focus_nearest(from_x0, from_y0, from_x1, from_y1,
     current = get_focused()
 
     if not current:
-        change_focus(focus_list[0])
+
+        for f in focus_list:
+
+            if not f.widget.style.keyboard_focus:
+                continue
+
+            change_focus(f)
+            return
+
         return
 
     # Find the current focus.
