@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -23,8 +23,12 @@
 # Most of the guts of this file have been moved into im.py, with only some
 # of the stuff thar uses images remaining.
 
-import renpy.display
-import renpy.text
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
+
+
+
+import renpy
 from renpy.display.render import render, Render
 
 import collections
@@ -35,7 +39,7 @@ images = { }
 
 # A map from image tag to lists of possible attributes for images with that
 # tag.
-image_attributes = collections.defaultdict(list)
+image_attributes = collections.defaultdict(dict)
 
 # The set of image names Ren'Py knows about, as strings with spaces.
 image_names = [ ]
@@ -94,7 +98,7 @@ def get_tag_method(tag, method):
     it returns None.
     """
 
-    ti =  images.get((tag,), None)
+    ti = images.get((tag,), None)
 
     if ti is None:
         return None
@@ -111,12 +115,24 @@ def check_image_attributes(tag, attributes):
     Otherwise, returns None.
     """
 
-    ca = get_tag_method(tag, "_choose_attributes")
+    l = [ ]
 
-    if ca is not None:
-        return ca(tag, attributes, None)
+    for attrs, d in image_attributes[tag].items():
 
-    l = get_available_image_attributes(tag, attributes)
+        remainder = [ i for i in attributes if i not in attrs ]
+
+        ca = getattr(d, "_choose_attributes", None)
+
+        if ca is not None:
+
+            chosen = ca(tag, remainder, None)
+            if chosen is not None:
+                l.append(list(attrs) + list(chosen))
+
+        else:
+
+            if not remainder:
+                l.append(attrs)
 
     # Check to see if there's an image that is exactly the one we want.
     for i in l:
@@ -133,7 +149,7 @@ def get_ordered_image_attributes(tag, attributes=(), sort=None):
     """
     :doc: image_func
 
-    Returns a list of image tags, ordered in a way that makes sense to
+    Returns a list of image attributes, ordered in a way that makes sense to
     present to the user.
 
     `attributes`
@@ -142,37 +158,87 @@ def get_ordered_image_attributes(tag, attributes=(), sort=None):
         can be in a single image at the same time.)
 
     `sort`
-        If not None, the returned list of attributes is sorted. This is a function
-        that should be used as a tiebreaker.
-
+        If not None, the returned list of attributes is sorted. This is a
+        one-argument function that should be used as a tiebreaker - see
+        `this tutorial <https://docs.python.org/3/howto/sorting.html#key-functions>`_
+        for more information.
     """
 
-    la = get_tag_method(tag, "_list_attributes")
-
-    if la is not None:
-        return la(tag, attributes)
+    sequences = [ ]
 
     attrcount = collections.defaultdict(int)
     attrtotalpos = collections.defaultdict(float)
 
-    for attrlist in get_available_image_attributes(tag, attributes):
-        for i, attr in enumerate(attrlist):
-            attrcount[attr] += 1
-            attrtotalpos[attr] += i
+    for attrs, d in sorted(image_attributes[tag].items()):
+
+        la = getattr(d, "_list_attributes", None)
+        if la is not None:
+
+            sequence = list(attrs) + la(tag, [ i for i in attributes if i not in attrs ])
+
+            if not all(i in sequence for i in attributes):
+                continue
+
+            sequences.append(sequence)
+
+        else:
+
+            if not all(i in attrs for i in attributes):
+                continue
+
+            for i, attr in enumerate(attrs):
+                attrcount[attr] += 1
+                attrtotalpos[attr] += i
 
     if sort is None:
+        return list(set(attrcount.keys()) | set(j for i in sequences for j in i))
 
-        return list(attrcount.keys())
+    # If we have a sequence, do a topological sort on the before-after relation -
+    # with an ajustment to make sure it will complete even if it loops.
 
-    else:
+    rv = [ ]
 
-        l = [ ]
+    # A map from an attribute to all the attributes it is after.
+    after = collections.defaultdict(set)
 
-        for attr in attrcount:
-            l.append((attrtotalpos[attr] / attrcount[attr], sort(attr), attr))
+    for i in sequences:
+        while i:
 
-        l.sort()
-        return [ i[2] for i in l ]
+            j = i.pop(0)
+
+            # Ensure it exists.
+            after[j]
+
+            for k in i:
+                after[k].add(j)
+
+    while after:
+
+        mincount = min(len(i) for i in after.values())
+        ready = set(k for k, v in after.items() if len(v) == mincount)
+
+        for i in ready:
+            del after[i]
+
+        for k in after:
+            after[k] = after[k] - ready
+
+        ready = list(ready)
+        ready.sort(key=lambda a : (sort(a), a))
+
+        rv.extend(ready)
+
+    l = [ ]
+
+    for attr in attrcount:
+        if attr not in rv:
+            l.append((attrtotalpos[attr] // attrcount[attr], sort(attr), attr))
+
+    l.sort()
+    for i in l:
+        rv.append(i[2])
+
+    return rv
 
 
 def register_image(name, d):
@@ -188,7 +254,7 @@ def register_image(name, d):
     rest = name[1:]
 
     images[name] = d
-    image_attributes[tag].append(rest)
+    image_attributes[tag][rest] = d
 
     image_names.append(" ".join(name))
 
@@ -225,6 +291,20 @@ def image_exists(name, exact=False):
     return False
 
 
+def get_registered_image(name):
+    """
+    :doc: image_func
+
+    If an image with the same name has been :ref:`registered <defining-images>`,
+    returns it. Otherwise, returns None.
+    """
+
+    if not isinstance(name, tuple):
+        name = tuple(name.split())
+
+    return images.get(name)
+
+
 def wrap_render(child, w, h, st, at):
     rend = render(child, w, h, st, at)
     rv = Render(rend.width, rend.height)
@@ -261,9 +341,10 @@ class ImageReference(renpy.display.core.Displayable):
         super(ImageReference, self).__init__(**properties)
 
         self.name = name
+        self.target = None # type: renpy.display.core.Displayable|None
 
-    def __unicode__(self):
-        return u"<ImageReference {!r}>".format(self.name)
+    def _repr_info(self):
+        return repr(self.name)
 
     def __hash__(self):
         return hash(self.name)
@@ -304,6 +385,8 @@ class ImageReference(renpy.display.core.Displayable):
             if renpy.config.debug:
                 raise Exception(msg)
 
+        target = None # typing
+
         args = [ ]
 
         while name:
@@ -319,18 +402,24 @@ class ImageReference(renpy.display.core.Displayable):
             error("Image '%s' not found." % ' '.join(self.name))
             return False
 
+        if name and (self._args.name == name):
+            error("Image '{}' refers to itself.".format(' '.join(name)))
+            return False
+
+        args += self._args.args
+
         try:
 
             a = self._args.copy(name=name, args=args)
-
             self.target = target._duplicate(a)
 
         except Exception as e:
 
-            if renpy.config.debug:
+            if renpy.config.raise_image_exceptions and (renpy.config.debug or renpy.config.developer):
                 raise
 
             error(str(e))
+            return False
 
         # Copy the old transform over.
         new_transform = self.target._target()
@@ -350,6 +439,9 @@ class ImageReference(renpy.display.core.Displayable):
 
     def _duplicate(self, args):
 
+        if args and args.args:
+            args.extraneous()
+
         rv = self._copy(args)
         rv.target = None
 
@@ -358,7 +450,6 @@ class ImageReference(renpy.display.core.Displayable):
                 rv.name = rv.name._duplicate(args)
 
         rv.find_target()
-        rv._duplicatable = rv.target._duplicatable
 
         return rv
 
@@ -367,7 +458,8 @@ class ImageReference(renpy.display.core.Displayable):
         if self.target is None:
             self.find_target()
 
-        self._duplicatable = self.target._duplicatable
+        self.target._unique()
+        self._duplicatable = False
 
     def _in_current_store(self):
 
@@ -456,10 +548,10 @@ class DynamicImage(renpy.display.core.Displayable):
     nosave = [ 'raw_target' ]
 
     # The target that this image currently resolves to.
-    target = None
+    target = None # type: renpy.display.core.Displayable|None
 
     # The raw target that the image resolves to, before it has been parameterized.
-    raw_target = None
+    raw_target = None # type: renpy.display.core.Displayable|None
 
     # Have we been locked, so we never change?
     locked = False
@@ -467,30 +559,42 @@ class DynamicImage(renpy.display.core.Displayable):
     # The name used for hashing.
     hash_name = None
 
+    # A copy of the last scope given to this displayable, if the
+    # displayable uses a prefix.
+    scope = None
+
     def __init__(self, name, scope=None, **properties):
         super(DynamicImage, self).__init__(**properties)
 
         self.name = name
 
-        if scope is not None:
-            self.find_target(scope)
-            self._uses_scope = True
-        else:
-            self._uses_scope = False
+        self._uses_scope = False
 
-        if isinstance(name, basestring) and ("[prefix_" in name):
-            self._duplicatable = True
+        if isinstance(name, basestring):
+            if ("[prefix_" in name):
+                self._duplicatable = True
+
+            if "[" in name.replace("[prefix_]", ""):
+                self._uses_scope = True
 
         if isinstance(name, list):
             for i in name:
                 if ("[prefix_" in i):
                     self._duplicatable = True
 
+                if "[" in i.replace("[prefix_]", ""):
+                    self._uses_scope = True
+
+        if scope is None:
+            self._uses_scope = False
+        else:
+            self.find_target(scope)
+
     def _scope(self, scope, update):
         return self.find_target(scope, update)
 
-    def __unicode__(self):
-        return u"DynamicImage {!r}".format(self.name)
+    def _repr_info(self):
+        return repr(self.name)
 
     def __hash__(self):
 
@@ -512,6 +616,9 @@ class DynamicImage(renpy.display.core.Displayable):
         if self.name != o.name:
             return False
 
+        if self._uses_scope and (self.target != o.target):
+            return False
+
         return True
 
     def _target(self):
@@ -520,13 +627,31 @@ class DynamicImage(renpy.display.core.Displayable):
         else:
             return self
 
+    def set_style_prefix(self, prefix, root):
+
+        if (prefix != self.style.prefix) and self._duplicatable:
+            self.target = None
+            self.raw_target = None
+
+        super(DynamicImage, self).set_style_prefix(prefix, root)
+
     def find_target(self, scope=None, update=True):
 
         if self.locked and (self.target is not None):
             return
 
+        if scope is not None:
+            if self._uses_scope and (self._duplicatable or self._args.prefix):
+                self.scope = dict(scope)
+
+        elif self._uses_scope:
+            scope = self.scope
+
         if self._args.prefix is None:
-            prefix = ""
+            if self._duplicatable:
+                prefix = self.style.prefix
+            else:
+                prefix = ""
         else:
             prefix = self._args.prefix
 
@@ -534,7 +659,7 @@ class DynamicImage(renpy.display.core.Displayable):
             search = [ ]
             target = renpy.easy.dynamic_image(self.name, scope, prefix=prefix, search=search)
         except KeyError as ke:
-            raise Exception("In DynamicImage %r: Could not find substitution '%s'." % (self.name, unicode(ke.args[0])))
+            raise Exception("In DynamicImage %r: Could not find substitution '%s'." % (self.name, str(ke.args[0])))
         except Exception as e:
             raise Exception("In DynamicImage %r: %r" % (self.name, e))
 
@@ -556,12 +681,16 @@ class DynamicImage(renpy.display.core.Displayable):
         if not update:
             return True
 
-        self.raw_target = target
+        raw_target = target # type: renpy.display.core.Displayable
         old_target = self.target
 
-        if target._duplicatable:
-            target = target._duplicate(self._args)
+        if raw_target._duplicatable:
+            target = raw_target._duplicate(self._args)
 
+            if not self._duplicatable:
+                self.target._unique()
+
+        self.raw_target = raw_target
         self.target = target
 
         renpy.display.render.redraw(self, 0)
@@ -579,13 +708,25 @@ class DynamicImage(renpy.display.core.Displayable):
 
         return True
 
+    _duplicatable = True
+
     def _duplicate(self, args):
+
+        if args and args.args:
+            args.extraneous()
+
         rv = self._copy(args)
         rv.target = None
+        rv.raw_target = None
         # This does not set _duplicatable, since it should always remain the
         # same.
         return rv
 
+    def _unique(self):
+        if self.target is not None:
+            self.target._unique()
+            self._duplicatable = False
+
     def _in_current_store(self):
         rv = self._copy()
 
@@ -685,9 +826,9 @@ class ShownImageInfo(renpy.object.Object):
             self.attributes = { }
             self.shown = set()
 
-            for layer in self.images:
-                for tag in self.images[layer]:
-                    self.attributes[layer, tag] = self.images[layer][tag][1:]
+            for layer in self.images: # type: ignore
+                for tag in self.images[layer]: # type: ignore
+                    self.attributes[layer, tag] = self.images[layer][tag][1:] # type: ignore
                     self.shown.add((layer, tag))
 
     def get_attributes(self, layer, tag, default=()):
@@ -696,6 +837,8 @@ class ShownImageInfo(renpy.object.Object):
         layer.
         """
 
+        layer = renpy.exports.default_layer(layer, tag)
+
         return self.attributes.get((layer, tag), default)
 
     def showing(self, layer, name, exact=False):
@@ -707,6 +850,8 @@ class ShownImageInfo(renpy.object.Object):
         tag = name[0]
         rest = name[1:]
 
+        layer = renpy.exports.default_layer(layer, tag)
+
         if (layer, tag) not in self.shown:
             return None
 
@@ -731,6 +876,14 @@ class ShownImageInfo(renpy.object.Object):
 
         return { t for l, t in self.shown if l == layer }
 
+    def get_hidden_tags(self, layer):
+        """
+        Returns the set of tags on layer that have attributes,
+        but aren't being shown.
+        """
+
+        return { t for l, t in self.attributes if l == layer if (l, t) not in self.shown }
+
     def predict_scene(self, layer):
         """
         Predicts the scene statement being called on layer.
@@ -739,7 +892,7 @@ class ShownImageInfo(renpy.object.Object):
         if layer is None:
             layer = 'master'
 
-        for l, t in self.attributes.keys():
+        for l, t in list(self.attributes.keys()):
             if l == layer:
                 del self.attributes[l, t]
 
@@ -757,6 +910,8 @@ class ShownImageInfo(renpy.object.Object):
         tag = name[0]
         rest = name[1:]
 
+        layer = renpy.exports.default_layer(layer, tag)
+
         self.attributes[layer, tag] = rest
 
         if show:
@@ -765,60 +920,92 @@ class ShownImageInfo(renpy.object.Object):
     def predict_hide(self, layer, name):
         tag = name[0]
 
+        layer = renpy.exports.default_layer(layer, tag)
+
         if (layer, tag) in self.attributes:
             del self.attributes[layer, tag]
 
         self.shown.discard((layer, tag))
 
-    def apply_attributes(self, layer, tag, name, wanted=[], remove=[]):
+    def apply_attributes(self, layer, tag, name):
         """
         Given a layer, tag, and an image name (with attributes),
         returns the canonical name of an image, if one exists. Raises
-        an exception if it's ambiguious, and returns None if an image
+        an exception if it's ambiguous, and returns None if an image
         with that name couldn't be found.
         """
 
+        f = renpy.config.adjust_attributes.get(name[0], None) or renpy.config.adjust_attributes.get(None, None)
+        if f is not None:
+            name = f(name)
+
+        layer = renpy.exports.default_layer(layer, tag)
+
         # If the name matches one that exactly exists, return it.
-        if (name in images) and not (wanted or remove):
-            ca = get_tag_method(tag, "_choose_attributes")
+        if name in images:
+            ca = getattr(images[name], "_choose_attributes", None)
+
             if ca is None:
                 return name
 
         nametag = name[0]
 
-        # The set of attributes a matching image must have.
-        required = set(name[1:])
+        # Find any attributes applied previously.
+        defaults = self.attributes.get((layer, tag), None)
 
-        # The set of attributes a matching image may have.
-        optional = set(wanted) | set(self.attributes.get((layer, tag), [ ]))
+        # If no record, it's the first show, so try to fetch defaults.
+        if defaults is None:
+            f = renpy.config.default_attribute_callbacks.get(name[0], None) \
+                or renpy.config.default_attribute_callbacks.get(None, None)
+            if f is not None:
+                defaults = f(name)
+
+        # The list of attributes a matching image may have.
+        optional = list(defaults) if defaults else [ ]
+
+        # The list of attributes a matching image must have.
+        required = [ ]
 
-        # Deal with banned attributes..
         for i in name[1:]:
             if i[0] == "-":
-                optional.discard(i[1:])
-                required.discard(i)
 
-        for i in remove:
-            optional.discard(i)
+                i = i[1:]
+
+                if i in optional:
+                    optional.remove(i)
+
+                if i in required:
+                    required.remove(i)
+
+            else:
+                required.append(i)
 
         return self.choose_image(nametag, required, optional, name)
 
     def choose_image(self, tag, required, optional, exception_name):
 
-        ca = get_tag_method(tag, "_choose_attributes")
-        if ca is not None:
-            attrs = ca(tag, required, optional)
-
-            if attrs is not None:
-                return (tag,) + attrs
-
         # The longest length of an image that matches.
         max_len = -1
 
         # The list of matching images.
         matches = None
 
-        for attrs in image_attributes[tag]:
+        for attrs, d in image_attributes[tag].items():
+
+            if not all((i in required) or (i in optional) for i in attrs):
+                continue
+
+            ca = getattr(d, "_choose_attributes", None)
+
+            if ca:
+                ca_required = [ i for i in required if i not in attrs ]
+                ca_optional = [ i for i in optional if i not in attrs ]
+                newattrs = ca(tag, ca_required, ca_optional)
+
+                if newattrs is None:
+                    continue
+
+                attrs = attrs + newattrs
 
             num_required = 0
 
@@ -827,27 +1014,22 @@ class ShownImageInfo(renpy.object.Object):
                     num_required += 1
                     continue
 
-                elif i not in optional:
-                    break
+            # We don't have any not-found attributes. But we might not
+            # have all of the attributes.
 
-            else:
+            if num_required != len(required):
+                continue
 
-                # We don't have any not-found attributes. But we might not
-                # have all of the attributes.
+            len_attrs = len(set(attrs))
 
-                if num_required != len(required):
-                    continue
+            if len_attrs < max_len:
+                continue
 
-                len_attrs = len(set(attrs))
+            if len_attrs > max_len:
+                max_len = len_attrs
+                matches = [ ]
 
-                if len_attrs < max_len:
-                    continue
-
-                if len_attrs > max_len:
-                    max_len = len_attrs
-                    matches = [ ]
-
-                matches.append((tag, ) + attrs)
+            matches.append((tag,) + attrs)
 
         if matches is None:
             return None
@@ -861,8 +1043,7 @@ class ShownImageInfo(renpy.object.Object):
             return None
 
 
-renpy.display.core.ImagePredictInfo = ShownImageInfo
-
+renpy.display.core.ImagePredictInfo = ShownImageInfo # type: ignore
 
 # Functions that have moved from this module to other modules,
 # that live here for the purpose of backward-compatibility.
