@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,13 +21,17 @@
 
 # This file handles imagemap caching.
 
-import pygame_sdl2 as pygame
-import renpy.display
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
 
-from renpy.display.render import render
 
 import hashlib
 
+import pygame_sdl2 as pygame
+import renpy
+from renpy.display.render import render
+
+
 # A list of cache images we've already written.
 cached = set()
 
@@ -92,7 +96,7 @@ class ImageMapCache(renpy.object.Object):
 
         # A list that, for each hotspot, gives the rectangle in the cache
         # image corresponding to that hotspot.
-        self.cache_rect = None
+        self.cache_rect = None # type: list[None|tuple[int, int, int, int]]|None
 
         # The size of the cache.
         self.cache_width = None
@@ -120,8 +124,8 @@ class ImageMapCache(renpy.object.Object):
         if rv is not None:
             return rv
 
-        self.md5.update(repr(d.identity))
-        self.md5.update(repr(rect))
+        self.md5.update(repr(d.identity).encode("utf-8"))
+        self.md5.update(repr(rect).encode("utf-8"))
 
         index = len(self.imagerect)
         rv = ImageCacheCrop(self, index)
@@ -135,7 +139,7 @@ class ImageMapCache(renpy.object.Object):
     def layout(self):
         self.areas.sort()
         self.areas.reverse()
-        self.cache_rect = [ None ] * len(self.areas)
+        self.cache_rect = [ None ] * len(self.areas) # type: ignore
 
         # The width of the cache image.
         width = self.areas[0][0]
@@ -151,7 +155,7 @@ class ImageMapCache(renpy.object.Object):
                 line_height = 0
                 x = 0
 
-            self.cache_rect[i] = (x+1, y+1, w-2, h-2)
+            self.cache_rect[i] = (x+1, y+1, w-2, h-2) # type: ignore
 
             x += w
             if line_height < h:
@@ -175,7 +179,7 @@ class ImageMapCache(renpy.object.Object):
         cache = pygame.Surface((self.cache_width, self.cache_height), pygame.SRCALPHA, 32)
 
         for i, (d, rect) in enumerate(self.imagerect):
-            x, y, _w, _h = self.cache_rect[i]
+            x, y, _w, _h = self.cache_rect[i] # type: ignore
 
             surf = renpy.display.im.cache.get(d).subsurface(rect)
             cache.blit(surf, (x, y))
@@ -212,7 +216,7 @@ class ImageMapCache(renpy.object.Object):
         if renpy.config.developer:
             try:
                 self.write_cache(filename)
-            except:
+            except Exception:
                 pass
 
         if renpy.loader.loadable(filename):
