@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -22,13 +22,16 @@
 # This module wraps the pygame surface class (and associated functions). It
 # ensures that returned surfaces have a 2px border around them.
 
-from __future__ import print_function
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
+
+
 
 import sys
-import pygame_sdl2 as pygame
 import threading
-import renpy.display
-import renpy.audio
+
+import pygame_sdl2 as pygame
+import renpy
 
 
 # Sample surfaces, with and without alpha.
@@ -54,7 +57,7 @@ def set_rgba_masks():
 
     # Sort the components by absolute value.
     masks = list(sample_alpha.get_masks())
-    masks.sort(key=lambda a : abs(a))
+    masks.sort(key=abs)
 
     # Choose the masks.
     if sys.byteorder == 'big':
@@ -75,11 +78,6 @@ class Surface(pygame.Surface):
     its mode, as necessary.
     """
 
-    opaque = False
-
-    def is_opaque(self):
-        return self.opaque
-
     def convert_alpha(self, surface=None):
         return copy_surface_unscaled(self, True)
 
@@ -87,21 +85,21 @@ class Surface(pygame.Surface):
         return copy_surface(self, False)
 
     def copy(self):
-        return copy_surface(self, self)
+        return copy_surface(self, self) # type:ignore
 
     def subsurface(self, rect):
         rv = pygame.Surface.subsurface(self, rect)
         return rv
 
 
-def surface((width, height), alpha):
+def surface(rect, alpha): # (tuple, bool|Surface) -> Surface
     """
     Constructs a new surface. The allocated surface is actually a subsurface
     of a surface that has a 2 pixel border in all directions.
 
     `alpha` - True if the new surface should have an alpha channel.
     """
-
+    (width, height) = rect
     if isinstance(alpha, pygame.Surface):
         alpha = alpha.get_masks()[3]
 
@@ -115,13 +113,14 @@ def surface((width, height), alpha):
     if sample is None:
         sample = pygame.Surface((4, 4), pygame.SRCALPHA, 32)
 
-    surf = Surface((width + 4, height + 4), 0, sample)
+    surf = Surface((width + 4, height + 4), 0, sample) # type:ignore
     return surf.subsurface((2, 2, width, height))  # E1101
 
+
 surface_unscaled = surface
 
 
-def copy_surface(surf, alpha=True):
+def copy_surface(surf, alpha=True): # (Surface, bool|Surface) -> Surface
     """
     Creates a copy of the surface.
     """
@@ -130,6 +129,7 @@ def copy_surface(surf, alpha=True):
     renpy.display.accelerator.nogil_copy(surf, rv)  # @UndefinedVariable
     return rv
 
+
 copy_surface_unscaled = copy_surface
 
 
@@ -143,8 +143,6 @@ image_load_lock = threading.RLock()
 
 
 def load_image(f, filename):
-    global count
-
     _basename, _dot, ext = filename.rpartition('.')
 
     try:
@@ -164,6 +162,7 @@ def load_image(f, filename):
     rv = copy_surface_unscaled(surf)
     return rv
 
+
 load_image_unscaled = load_image
 
 
@@ -173,6 +172,7 @@ def flip(surf, horizontal, vertical):
     surf = pygame.transform.flip(surf, horizontal, vertical)
     return copy_surface_unscaled(surf)
 
+
 flip_unscaled = flip
 
 
@@ -181,6 +181,7 @@ def rotozoom(surf, angle, zoom):
     surf = pygame.transform.rotozoom(surf, angle, zoom)
     return copy_surface_unscaled(surf)
 
+
 rotozoom_unscaled = rotozoom
 
 
@@ -188,6 +189,7 @@ def transform_scale(surf, size):
     surf = pygame.transform.scale(surf, size)
     return copy_surface_unscaled(surf, surf)
 
+
 transform_scale_unscaled = transform_scale
 
 
@@ -195,4 +197,5 @@ def transform_rotate(surf, angle):
     surf = pygame.transform.rotate(surf, angle)
     return copy_surface(surf)
 
+
 transform_rotate_unscaled = transform_rotate
