@@ -1349,8 +1349,8 @@ def current_screen(): # type: () -> ScreenDisplayable|None
     :doc: screens
     :name: renpy.current_screen
 
-    Returns the ScreendDisplayable corresponding to the screen currently being
-    updated, rendered, or processing an
+    Returns the ScreenDisplayable corresponding to the screen currently being
+    updated, rendered, or processed.
 
     See :func:`get_screen` for documented fields on ScreenDisplayable.
     """
