@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,17 +21,39 @@
 
 # Functions that make the user's life easier.
 
-from __future__ import print_function
-import renpy.display
-import renpy.styledata
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
+
+from typing import Any
+
 import contextlib
 import time
 
+import renpy
+
 Color = renpy.color.Color
 color = renpy.color.Color
 
 
-def displayable_or_none(d, scope=None, dynamic=True):
+def lookup_displayable_prefix(d):
+    """
+    Given `d`, a string given a displayable, returns the displayable it
+    corresponds to or None if it does not correspond to one.
+    """
+
+    prefix, colon, arg = d.partition(":")
+
+    if not colon:
+        return None
+
+    fn = renpy.config.displayable_prefix.get(prefix, None)
+    if fn is None:
+        return None
+
+    return displayable(fn(arg))
+
+
+def displayable_or_none(d, scope=None, dynamic=True): # type: (Any, dict|None, bool) -> renpy.display.core.Displayable|None
 
     if isinstance(d, renpy.display.core.Displayable):
         return d
@@ -44,6 +66,11 @@ def displayable_or_none(d, scope=None, dynamic=True):
             raise Exception("An empty string cannot be used as a displayable.")
         elif ("[" in d) and renpy.config.dynamic_images and dynamic:
             return renpy.display.image.DynamicImage(d, scope=scope)
+
+        rv = lookup_displayable_prefix(d)
+
+        if rv is not None:
+            return rv
         elif d[0] == '#':
             return renpy.store.Solid(d)
         elif "." in d:
@@ -52,10 +79,10 @@ def displayable_or_none(d, scope=None, dynamic=True):
             return renpy.store.ImageReference(tuple(d.split()))
 
     if isinstance(d, Color):
-        return renpy.store.Solid(d)
+        return renpy.store.Solid(d) # type: ignore
 
     if isinstance(d, list):
-        return renpy.display.image.DynamicImage(d, scope=scope)
+        return renpy.display.image.DynamicImage(d, scope=scope) # type: ignore
 
     # We assume the user knows what he's doing in this case.
     if hasattr(d, '_duplicate'):
@@ -67,7 +94,7 @@ def displayable_or_none(d, scope=None, dynamic=True):
     raise Exception("Not a displayable: %r" % (d,))
 
 
-def displayable(d, scope=None):
+def displayable(d, scope=None): # type(d, dict|None=None) -> renpy.display.core.Displayable|None
     """
     :doc: udd_utility
     :name: renpy.displayable
@@ -85,6 +112,11 @@ def displayable(d, scope=None):
             raise Exception("An empty string cannot be used as a displayable.")
         elif ("[" in d) and renpy.config.dynamic_images:
             return renpy.display.image.DynamicImage(d, scope=scope)
+
+        rv = lookup_displayable_prefix(d)
+
+        if rv is not None:
+            return rv
         elif d[0] == '#':
             return renpy.store.Solid(d)
         elif "." in d:
@@ -108,7 +140,7 @@ def displayable(d, scope=None):
     raise Exception("Not a displayable: %r" % (d,))
 
 
-def dynamic_image(d, scope=None, prefix=None, search=None):
+def dynamic_image(d, scope=None, prefix=None, search=None): # type: (Any, dict|None, str|None, list|None) -> renpy.display.core.Displayable|None
     """
     Substitutes a scope into `d`, then returns a displayable.
 
@@ -121,10 +153,13 @@ def dynamic_image(d, scope=None, prefix=None, search=None):
 
     def find(name):
 
+        if renpy.exports.image_exists(name):
+            return True
+
         if renpy.loader.loadable(name):
             return True
 
-        if renpy.exports.image_exists(name):
+        if lookup_displayable_prefix(name):
             return True
 
         if (len(d) == 1) and (renpy.config.missing_image_callback is not None):
@@ -164,14 +199,12 @@ def dynamic_image(d, scope=None, prefix=None, search=None):
             if search is not None:
                 search.append(rv)
 
-    else:
+    rv = d[-1]
 
-        rv = d[-1]
+    if find(rv):
+        return displayable_or_none(rv, dynamic=False)
 
-        if find(rv):
-            return displayable_or_none(rv, dynamic=False)
-
-        return None
+    return None
 
 
 def predict(d):
@@ -203,7 +236,7 @@ def split_properties(properties, *prefixes):
     For example, this splits properties beginning with text from
     those that do not::
 
-        text_properties, button_properties = renpy.split_properties("text_", "")
+        text_properties, button_properties = renpy.split_properties(properties, "text_", "")
     """
 
     rv = [ ]
@@ -216,7 +249,7 @@ def split_properties(properties, *prefixes):
 
     prefix_d = list(zip(prefixes, rv))
 
-    for k, v in properties.iteritems():
+    for k, v in properties.items():
         for prefix, d in prefix_d:
             if k.startswith(prefix):
                 d[k[len(prefix):]] = v
