@@ -1,4 +1,4 @@
-# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,10 +21,14 @@
 
 # This file contains code for formatting tracebacks.
 
-from __future__ import print_function
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
+
+
+
 import traceback
 import sys
-import cStringIO
+import io
 import platform
 import linecache
 import time
@@ -35,31 +39,31 @@ import renpy
 FSENCODING = sys.getfilesystemencoding() or "utf-8"
 
 
-def write_utf8_traceback_list(out, l):
+def write_traceback_list(out, l):
     """
-    Given the traceback list l, writes it to out as utf-8.
+    Given the traceback list, writes it to out as unicode.
     """
 
     ul = [ ]
 
     for filename, line, what, text in l:
 
-        # Filename is either unicode or an fsecoded string.
-        if not isinstance(filename, unicode):
-            filename = unicode(filename, FSENCODING, "replace")
+        # Filename is either unicode or fsecoded bytes.
+        if isinstance(filename, bytes):
+            filename = filename.decode(FSENCODING)
 
         # Line is a number.
 
         # Assume what is in a unicode encoding, since it is either python,
         # or comes from inside Ren'Py.
 
-        if isinstance(text, str):
-            text = text.decode("utf-8", "replace")
+        if isinstance(text, bytes):
+            text = text.decode("utf-8")
 
         ul.append((filename, line, what, text))
 
     for t in traceback.format_list(ul):
-        out.write(t.encode("utf-8", "replace"))
+        out.write(t)
 
 
 def traceback_list(tb):
@@ -90,7 +94,7 @@ def traceback_list(tb):
                 if report is not None:
                     l.extend(report)
                     continue
-            except:
+            except Exception:
                 pass
 
         l.append((filename, line_number, name, None))
@@ -101,7 +105,7 @@ def traceback_list(tb):
         if line is None:
             try:
                 line = linecache.getline(filename, line_number)
-            except:
+            except Exception:
                 line = ''
 
         rv.append((filename, line_number, name, line))
@@ -132,22 +136,22 @@ def open_error_file(fn, mode):
     """
 
     try:
-        new_fn = os.path.join(renpy.config.logdir, fn)
-        f = file(new_fn, mode)
+        new_fn = os.path.join(renpy.config.logdir, fn) # type: ignore
+        f = open(new_fn, mode)
         return f, new_fn
-    except:
+    except Exception:
         pass
 
     try:
-        f = file(fn, mode)
+        f = open(fn, mode)
         return f, fn
-    except:
+    except Exception:
         pass
 
     import tempfile
 
     new_fn = os.path.join(tempfile.gettempdir(), "renpy-" + fn)
-    return file(new_fn, mode), new_fn
+    return open(new_fn, mode), new_fn
 
 
 def report_exception(e, editor=True):
@@ -156,8 +160,9 @@ def report_exception(e, editor=True):
     traceback.txt. If `editor` is True, opens the traceback
     up in a text editor.
 
-    Returns a two-unicode tuple, with the first item being
-    a simple message, and the second being a full traceback.
+    Returns a three-item tuple, with the first item being
+    a simplified traceback, the second being a full traceback,
+    and the third being the traceback filename.
     """
 
     # Note: Doki Doki Literature club calls this as ("Words...", False).
@@ -165,46 +170,24 @@ def report_exception(e, editor=True):
 
     import codecs
 
-    type, _value, tb = sys.exc_info()  # @ReservedAssignment
-
-    def safe_utf8(e):
-        try:
-            m = unicode(e)
-        except:
-            try:
-                if len(e.args) == 0:
-                    m = ""
-                elif len(e.args) == 1:
-                    m = e.args[0]
-                else:
-                    m = " ".join(e.args)
-            except:
-                try:
-                    m = repr(e)
-                except:
-                    m = "<Could not encode exception.>"
-
-        if isinstance(m, unicode):
-            return m.encode("utf-8", "replace")
-        else:
-            return m
+    type, _value, tb = sys.exc_info() # @ReservedAssignment
 
     # Return values - which can be displayed to the user.
-    simple = cStringIO.StringIO()
-    full = cStringIO.StringIO()
+    simple = io.StringIO()
+    full = io.StringIO()
 
     full_tl = traceback_list(tb)
     simple_tl = filter_traceback_list(full_tl)
 
-    print(renpy.game.exception_info, file=simple)
-    write_utf8_traceback_list(simple, simple_tl)
+    print(str(renpy.game.exception_info), file=simple)
+    write_traceback_list(simple, simple_tl)
     print(type.__name__ + ":", end=' ', file=simple)
-    print(safe_utf8(e), file=simple)
+    print(str(e), file=simple)
 
     print("Full traceback:", file=full)
-    write_utf8_traceback_list(full, full_tl)
+    write_traceback_list(full, full_tl)
     print(type.__name__ + ":", end=' ', file=full)
-    print(safe_utf8(e), file=full)
+    print(str(e), file=full)
 
     # Write to stdout/stderr.
     try:
@@ -212,16 +195,17 @@ def report_exception(e, editor=True):
         sys.stdout.write(full.getvalue())
         sys.stdout.write("\n")
         sys.stdout.write(simple.getvalue())
-    except:
+    except Exception:
         pass
 
-    print(file=full)
+    print('', file=full)
+
     try:
-        print(platform.platform(), file=full)
+        print(str(platform.platform()), str(platform.machine()), file=full)
         print(renpy.version, file=full)
-        print(safe_utf8(renpy.config.name + " " + renpy.config.version), file=full)
-        print(time.ctime(), file=full)
-    except:
+        print(renpy.config.name + " " + renpy.config.version, file=full)
+        print(str(time.ctime()), file=full)
+    except Exception:
         pass
 
     simple = simple.getvalue()
@@ -232,27 +216,32 @@ def report_exception(e, editor=True):
 
         f, traceback_fn = open_error_file("traceback.txt", "w")
 
-        f.write(codecs.BOM_UTF8)
+        with f:
+            f.write("\ufeff") # BOM
 
-        print("I'm sorry, but an uncaught exception occurred.", file=f)
-        print(file=f)
+            print("I'm sorry, but an uncaught exception occurred.", file=f)
+            print('', file=f)
 
-        f.write(simple)
+            f.write(simple)
 
-        print(file=f)
-        print("-- Full Traceback ------------------------------------------------------------", file=f)
-        print(file=f)
+            print('', file=f)
+            print("-- Full Traceback ------------------------------------------------------------", file=f)
+            print('', file=f)
 
-        f.write(full)
-        f.close()
+            f.write(full)
 
         try:
-            if editor and renpy.game.args.command == "run":  # @UndefinedVariable
-                renpy.exports.launch_editor([ traceback_fn ], 1, transient=1)
-        except:
+            renpy.util.expose_file(traceback_fn)
+        except Exception:
             pass
 
-    except:
-        pass
+        try:
+            if editor and ((renpy.game.args.command == "run") or (renpy.game.args.errors_in_editor)): # type: ignore
+                renpy.exports.launch_editor([ traceback_fn ], 1, transient=True)
+        except Exception:
+            pass
 
-    return simple.decode("utf-8", "replace"), full.decode("utf-8", "replace"), traceback_fn
+    except Exception:
+        traceback_fn = os.path.join(renpy.config.basedir, "traceback.txt") # type: ignore
+
+    return simple, full, traceback_fn
