@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -22,14 +22,17 @@
 # This file contains code responsible for managing the execution of a
 # renpy object, as well as the context object.
 
-from __future__ import print_function
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
+
+from future.utils import reraise
+
 import sys
 import time
 
-import renpy.display
-import renpy.test
+import renpy
 
-pyast = __import__("ast", { })
+import ast as pyast
 
 # The number of statements that have been run since the last infinite loop
 # check.
@@ -106,6 +109,9 @@ class LineLogEntry(object):
 
         return (self.filename == other.filename) and (self.line == other.line) and (self.node is other.node)
 
+    def __ne__(self, other):
+        return not (self == other)
+
 
 class Context(renpy.object.Object):
     """
@@ -130,7 +136,7 @@ class Context(renpy.object.Object):
     does participates in rollback.
     """
 
-    __version__ = 15
+    __version__ = 16
 
     nosave = [ 'next_node' ]
 
@@ -138,9 +144,31 @@ class Context(renpy.object.Object):
 
     force_checkpoint = False
 
+    come_from_name = None
+    come_from_label = None
+
+    temporary_attributes = None
+
+    deferred_translate_identifier = None
+
+    predict_return_stack = None # type: list|None
+
+    def __repr__(self):
+
+        if not self.current:
+            return "<Context>"
+
+        node = renpy.game.script.lookup(self.current)
+
+        return "<Context: {}:{} {!r}>".format(
+            node.filename,
+            node.linenumber,
+            node.diff_info(),
+            )
+
     def after_upgrade(self, version):
         if version < 1:
-            self.scene_lists.image_predict_info = self.predict_info.images
+            self.scene_lists.image_predict_info = self.predict_info.images # type: ignore
 
         if version < 2:
             self.abnormal = False
@@ -153,11 +181,11 @@ class Context(renpy.object.Object):
             self.interacting = False
 
         if version < 5:
-            self.modes = renpy.python.RevertableList([ "start" ])
+            self.modes = renpy.revertable.RevertableList([ "start" ])
             self.use_modes = True
 
         if version < 6:
-            self.images = self.predict_info.images
+            self.images = self.predict_info.images # type: ignore
 
         if version < 7:
             self.init_phase = False
@@ -176,9 +204,6 @@ class Context(renpy.object.Object):
         if version < 11:
             self.say_attributes = None
 
-        if version < 12:
-            self.translate_block_language = None
-
         if version < 13:
             self.line_log = [ ]
 
@@ -188,6 +213,9 @@ class Context(renpy.object.Object):
         if version < 15:
             self.abnormal_stack = [ False ] * len(self.return_stack)
 
+        if version < 16:
+            self.alternate_translate_identifier = None
+
     def __init__(self, rollback, context=None, clear=False):
         """
         `clear`
@@ -210,7 +238,7 @@ class Context(renpy.object.Object):
 
         self.rollback = rollback
         self.runtime = 0
-        self.info = renpy.python.RevertableObject()
+        self.info = renpy.revertable.RevertableObject()
         self.seen = False
 
         # True if there has just been an abnormal transfer of control,
@@ -226,7 +254,7 @@ class Context(renpy.object.Object):
 
         # A map from the name of a music channel to the MusicContext
         # object corresponding to that channel.
-        self.music = renpy.python.RevertableDict()
+        self.music = { }
 
         # True if we're in the middle of a call to ui.interact. This
         # will cause Ren'Py to generate an error if we call ui.interact
@@ -237,7 +265,7 @@ class Context(renpy.object.Object):
         self.init_phase = False
 
         # When deferring a rollback, the arguments to pass to renpy.exports.rollback.
-        self.defer_rollback = None
+        self.defer_rollback = None # type: tuple[int, bool]|None
 
         # The exception handler that is called when an exception occurs while executing
         # code. If None, a default handler is used. This is reset when run is called.
@@ -245,6 +273,7 @@ class Context(renpy.object.Object):
 
         # The attributes that are used by the current say statement.
         self.say_attributes = None
+        self.temporary_attributes = None
 
         # A list of lines that were run since the last time this log was
         # cleared.
@@ -254,7 +283,7 @@ class Context(renpy.object.Object):
         # executed?
         self.force_checkpoint = False
 
-        # A mapt from a channel to the Movie playing on that channel.
+        # A map from a channel to the Movie playing on that channel.
         self.movie = { }
 
         if context:
@@ -263,9 +292,7 @@ class Context(renpy.object.Object):
 
             vars(self.info).update(vars(context.info))
 
-            for k, v in context.music.items():
-                self.music[k] = v.copy()
-
+            self.music = dict(context.music)
             self.movie = dict(context.movie)
 
             self.images = renpy.display.image.ShownImageInfo(context.images)
@@ -276,6 +303,9 @@ class Context(renpy.object.Object):
 
         self.scene_lists = renpy.display.core.SceneLists(oldsl, self.images)
 
+        for i in renpy.config.context_copy_remove_screens:
+            self.scene_lists.remove("screens", i, None)
+
         self.make_dynamic([ "_return", "_args", "_kwargs", "mouse_visible", "suppress_overlay", "_side_image_attributes" ])
         self.dynamic_stack.append({ })
 
@@ -284,17 +314,33 @@ class Context(renpy.object.Object):
                 self.scene_lists.clear(layer=i)
 
         # A list of modes that the context has been in.
-        self.modes = renpy.python.RevertableList([ "start" ])
+        self.modes = renpy.revertable.RevertableList([ "start" ])
         self.use_modes = True
 
         # The language we started with.
-        self.translate_language = None
+        self.translate_language = renpy.game.preferences.language # type: ignore
 
         # The identifier of the current translate block.
         self.translate_identifier = None
 
-        # The language of the current translate block.
-        self.translate_block_language = None
+        # The alternate identifier of the current translate block.
+        self.alternate_translate_identifier = None
+
+        # The translate identifier of the last say statement with
+        # interact = False.
+        self.deferred_translate_identifier = None
+
+    def replace_node(self, old, new):
+
+        def replace_one(name):
+            n = renpy.game.script.lookup(name)
+            if n is old:
+                return new.name
+
+            return name
+
+        self.current = replace_one(self.current)
+        self.return_stack = [ replace_one(i) for i in self.return_stack ]
 
     def make_dynamic(self, names, context=False):
         """
@@ -302,8 +348,6 @@ class Context(renpy.object.Object):
         their current value (if not already dynamic in the current call).
         """
 
-        store = renpy.store.__dict__
-
         if context:
             index = 0
         else:
@@ -314,8 +358,20 @@ class Context(renpy.object.Object):
             if i in self.dynamic_stack[index]:
                 continue
 
-            if i in store:
-                self.dynamic_stack[index][i] = store[i]
+            name = i
+            store = renpy.store.__dict__
+
+            while "." in name:
+                storename, _, name = name.partition(".")
+                storemodule = store.get(storename, None)
+
+                if not isinstance(storemodule, renpy.python.StoreModule):
+                    raise Exception("{} is not a valid namespace.".format(i.rpartition(".")[0]))
+
+                store = storemodule.__dict__
+
+            if name in store:
+                self.dynamic_stack[index][i] = store[name]
             else:
                 self.dynamic_stack[index][i] = Delete()
 
@@ -328,13 +384,17 @@ class Context(renpy.object.Object):
         if not self.dynamic_stack:
             return
 
-        store = renpy.store.__dict__
-
         dynamic = self.dynamic_stack.pop()
 
-        for k, v in dynamic.iteritems():
+        for k, v in dynamic.items():
+            store = renpy.store.__dict__
+
+            while "." in k:
+                namespace, _, k = k.partition(".")
+                store = store[namespace].__dict__
+
             if isinstance(v, Delete):
-                del store[k]
+                store.pop(k, None)
             else:
                 store[k] = v
 
@@ -351,7 +411,7 @@ class Context(renpy.object.Object):
 
         for dynamic in reversed(self.dynamic_stack):
 
-            for k, v in dynamic.iteritems():
+            for k, v in dynamic.items():
                 name = "store." + k
 
                 if isinstance(v, Delete) and (name in roots):
@@ -396,14 +456,14 @@ class Context(renpy.object.Object):
                 node = renpy.game.script.lookup(i)
                 if not node.filename.replace("\\", "/").startswith("common/"):
                     rv.append((node.filename, node.linenumber, "script call", None))
-            except:
+            except Exception:
                 pass
 
         try:
             node = renpy.game.script.lookup(self.current)
             if not node.filename.replace("\\", "/").startswith("common/"):
                 rv.append((node.filename, node.linenumber, "script", None))
-        except:
+        except Exception:
             pass
 
         return rv
@@ -416,10 +476,18 @@ class Context(renpy.object.Object):
         """
 
         ps = pyast.Pass(lineno=node.linenumber, col_offset=0)
-        module = pyast.Module(lineno=node.linenumber, col_offset=0, body=[ ps ])
+        module = pyast.Module(lineno=node.linenumber, col_offset=0, body=[ ps ], type_ignores=[])
         code = compile(module, node.filename, 'exec')
         exec(code)
 
+    def come_from(self, name, label):
+        """
+        When control reaches name, call label. Only for internal use.
+        """
+
+        self.come_from_name = name
+        self.come_from_label = label
+
     def run(self, node=None):
         """
         Executes as many nodes as possible in the current context. If the
@@ -437,8 +505,17 @@ class Context(renpy.object.Object):
         developer = renpy.config.developer
         tracing = sys.gettrace() is not None
 
+        # Is this the first time through the loop?
+        first = True
+
         while node:
 
+            if node.name == self.come_from_name:
+                self.come_from_name = None
+                node = self.call(self.come_from_label, return_site=node.name)
+                self.make_dynamic([ "_return", "_begin_rollback" ])
+                renpy.store._begin_rollback = False
+
             this_node = node
             type_node_name = type(node).__name__
 
@@ -455,7 +532,10 @@ class Context(renpy.object.Object):
                 if ll_entry not in self.line_log:
                     self.line_log.append(ll_entry)
 
-            if self.force_checkpoint or (node.rollback == "force"):
+            if not renpy.store._begin_rollback:
+                update_rollback = False
+                force_rollback = False
+            elif first or self.force_checkpoint or (node.rollback == "force"):
                 update_rollback = True
                 force_rollback = True
             elif not renpy.config.all_nodes_rollback and (node.rollback == "never"):
@@ -465,13 +545,20 @@ class Context(renpy.object.Object):
                 update_rollback = True
                 force_rollback = False
 
+            # Force a new rollback to start to match things in the forward log.
+            if renpy.game.log.forward and renpy.game.log.forward[0][0] == node.name:
+                update_rollback = True
+                force_rollback = True
+
+            first = False
+
             if update_rollback:
 
                 if self.rollback and renpy.game.log:
                     renpy.game.log.begin(force=force_rollback)
 
-                if self.force_checkpoint:
-                    renpy.game.log.checkpoint(hard=False)
+                if self.rollback and self.force_checkpoint:
+                    renpy.game.log.force_checkpoint = True
                     self.force_checkpoint = False
 
             self.seen = False
@@ -512,14 +599,21 @@ class Context(renpy.object.Object):
                     short, full, traceback_fn = renpy.error.report_exception(e, editor=False)
 
                     try:
+                        handled = False
+
                         if self.exception_handler is not None:
                             self.exception_handler(short, full, traceback_fn)
-                        elif renpy.display.error.report_exception(short, full, traceback_fn):
-                            raise
+                            handled = True
+                        elif renpy.config.exception_handler is not None:
+                            handled = renpy.config.exception_handler(short, full, traceback_fn)
+
+                        if not handled:
+                            if renpy.display.error.report_exception(short, full, traceback_fn):
+                                raise
                     except renpy.game.CONTROL_EXCEPTIONS as ce:
                         raise ce
                     except Exception as ce:
-                        raise exc_info[0], exc_info[1], exc_info[2]
+                        reraise(exc_info[0], exc_info[1], exc_info[2])
 
                 node = self.next_node
 
@@ -530,7 +624,7 @@ class Context(renpy.object.Object):
             except renpy.game.CallException as e:
 
                 if e.from_current:
-                    return_site = node.name
+                    return_site = getattr(node, "statement_start", node).name
                 else:
                     if self.next_node is None:
                         raise Exception("renpy.call can't be used when the next node is undefined.")
@@ -542,7 +636,7 @@ class Context(renpy.object.Object):
                 renpy.store._kwargs = e.kwargs
 
             if self.seen:
-                renpy.game.persistent._seen_ever[self.current] = True  # @UndefinedVariable
+                renpy.game.persistent._seen_ever[self.current] = True # type: ignore
                 renpy.game.seen_session[self.current] = True
 
             renpy.plog(2, "    end {} ({}:{})", type_node_name, this_node.filename, this_node.linenumber)
@@ -614,15 +708,29 @@ class Context(renpy.object.Object):
 
             if node is None:
 
-                if renpy.config.developer:
-                    raise Exception("Could not find return label {!r}.".format(self.return_stack[-1]))
+                if not pop:
+                    return None
 
-                self.return_stack.pop()
-                self.call_location_stack.pop()
-                self.pop_dynamic()
-                self.abnormal = self.abnormal_stack.pop()
+                # If we can't find anything, try to recover.
 
-                continue
+                if renpy.config.return_not_found_label:
+
+                    while len(self.return_stack) > 1:
+                        self.pop_call()
+
+                    node = renpy.game.script.lookup(renpy.config.return_not_found_label)
+
+                else:
+
+                    if renpy.config.developer:
+                        raise Exception("Could not find return label {!r}.".format(self.return_stack[-1]))
+
+                    self.return_stack.pop()
+                    self.call_location_stack.pop()
+                    self.pop_dynamic()
+                    self.abnormal = self.abnormal_stack.pop()
+
+                    continue
 
             if pop:
                 self.return_stack.pop()
@@ -668,7 +776,7 @@ class Context(renpy.object.Object):
         Returns the node corresponding to `label`
         """
 
-        self.predict_return_stack = list(self.predict_return_stack)
+        self.predict_return_stack = list(self.predict_return_stack) # type: ignore
         self.predict_return_stack.append(return_site)
 
         return renpy.game.script.lookup(label)
@@ -713,7 +821,7 @@ class Context(renpy.object.Object):
         for label in renpy.config.predict_statements_callback(self.current):
 
             if not renpy.game.script.has_label(label):
-                return
+                continue
 
             node = renpy.game.script.lookup(label)
 
@@ -744,14 +852,14 @@ class Context(renpy.object.Object):
                         nodes.append((n, self.images, self.predict_return_stack))
                         seen.add(n)
 
-            except:
+            except Exception:
 
-                if renpy.config.debug_image_cache:
+                if renpy.config.debug_prediction:
                     import traceback
 
-                    print()
-                    traceback.print_exc()
                     print("While predicting images.")
+                    traceback.print_exc()
+                    print()
 
             self.images = old_images
             self.predict_return_stack = None
@@ -774,7 +882,7 @@ class Context(renpy.object.Object):
             return False
 
         if ever:
-            seen = renpy.game.persistent._seen_ever  # @UndefinedVariable
+            seen = renpy.game.persistent._seen_ever # type: ignore
         else:
             seen = renpy.game.seen_session
 
@@ -792,7 +900,7 @@ class Context(renpy.object.Object):
 
         self.defer_rollback = None
 
-        renpy.exports.rollback(force, checkpoints)
+        renpy.exports.rollback(bool(force), checkpoints)
 
     def get_return_stack(self):
         return list(self.return_stack)
@@ -818,20 +926,15 @@ def run_context(top):
     the RestartContext and RestartTopContext exceptions.
     """
 
-    label = None
+    if renpy.config.context_callback is not None:
+        renpy.config.context_callback()
 
     while True:
 
+        context = renpy.game.context()
+
         try:
 
-            context = renpy.game.context()
-
-            if label and renpy.game.script.has_label(label):
-                context.call(label)
-                context.make_dynamic([ "_return" ])
-
-            label = None
-
             context.run()
 
             rv = renpy.store._return
@@ -841,29 +944,15 @@ def run_context(top):
             return rv
 
         except renpy.game.RestartContext as e:
-
-            # Apply defaults.
-            renpy.exports.execute_default_statement(False)
-
-            if e.label:
-                label = e.label
-
             continue
 
         except renpy.game.RestartTopContext as e:
             if top:
-
-                # Apply defaults.
-                renpy.exports.execute_default_statement(False)
-
-                if e.label:
-                    label = e.label
-
                 continue
 
             else:
                 raise
 
-        except:
+        except Exception:
             context.pop_all_dynamic()
             raise
