@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -19,38 +19,60 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-from __future__ import print_function
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
+
+
+xrange = range
+
+unicode = str # @ReservedAssignment
 
 python_list = _list = list
 python_dict = _dict = dict
-python_object = _object = object
+python_object = _object = object # type: ignore
 python_set = _set = set
 
 _type = type
 
-from renpy.python import RevertableList as __renpy__list__
-list = __renpy__list__  # @ReservedAssignment
+from renpy.revertable import RevertableList as __renpy__list__
+list = __renpy__list__ # @ReservedAssignment
 
-from renpy.python import RevertableDict as __renpy__dict__
-dict = __renpy__dict__  # @ReservedAssignment
+from renpy.revertable import RevertableDict as __renpy__dict__
+dict = __renpy__dict__ # @ReservedAssignment
 
-from renpy.python import RevertableSet as __renpy__set__
-set = __renpy__set__  # @ReservedAssignment
+from renpy.revertable import RevertableSet as __renpy__set__
+set = __renpy__set__ # @ReservedAssignment
 Set = __renpy__set__
 
-from renpy.python import RevertableObject as object  # @UnusedImport
+from renpy.revertable import RevertableObject as object # @UnusedImport
 
-from renpy.python import revertable_range as range  # @UnusedImport
-from renpy.python import revertable_sorted as sorted  # @UnusedImport
+from renpy.revertable import revertable_range as range # @UnusedImport
+from renpy.revertable import revertable_sorted as sorted # @UnusedImport
 
-import renpy.ui as ui  # @UnusedImport
-import renpy.exports as renpy  # @Reimport @UnusedImport
-from renpy.translation import translate_string as __  # @UnusedImport
+import renpy.ui as ui # @UnusedImport
+from renpy.translation import translate_string as __ # @UnusedImport
 
 from renpy.python import store_eval as eval
 
 from renpy.display.core import absolute
 
+import renpy
+globals()["renpy"] = renpy.exports
+
+_print = print
+
+
+def print(*args, **kwargs):
+    """
+    :undocumented:
+
+    This is a variant of the print function that forces a checkpoint
+    at the start of the next statement, so that it can't be rolled past.
+    """
+
+    renpy.game.context().force_checkpoint = True # type: ignore
+    _print(*args, **kwargs)
+
 
 def _(s):
     """
@@ -122,3 +144,56 @@ def _p(s):
 
     rv += " ".join(para)
     return rv
+
+def input(*args, **kwargs):
+    """
+    :undocumented:
+    """
+
+    raise Exception("The Python input and raw_input functions do not work with Ren'Py. Please use the renpy.input function instead.")
+
+raw_input = input
+
+
+__all__ = [
+    'PY2',
+    'Set',
+    '_',
+    '__',
+    '__renpy__dict__',
+    '__renpy__list__',
+    '__renpy__set__',
+    '_dict',
+    '_list',
+    '_object',
+    '_p',
+    '_print',
+    '_set',
+    '_type',
+    'absolute',
+    'basestring',
+    'bchr',
+    'bord',
+    'dict',
+    'eval',
+    'input',
+    'list',
+    'object',
+    'open',
+    'print',
+    'python_dict',
+    'python_list',
+    'python_object',
+    'python_set',
+    'range',
+    'raw_input',
+    'set',
+    'sorted',
+    'str',
+    'tobytes',
+    'ui',
+    'unicode',
+    ]
+
+if PY2:
+    __all__ = [ bytes(i) for i in __all__ ] # type: ignore
