@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -19,9 +19,11 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-from __future__ import print_function
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
 
-import renpy.display
+
+import renpy
 import contextlib
 
 # Grab the python versions of the parser and ast modules.
@@ -218,7 +220,7 @@ class Parser(object):
                 if c is None:
                     l.error('Expected screen language statement.')
 
-                rv.extend(c)
+                rv.extend(c) # type: ignore
                 count += 1
 
         return rv
@@ -229,7 +231,7 @@ class Parser(object):
         and expr instances, and adjusts the line number.
         """
 
-        if isinstance(expr, unicode):
+        if isinstance(expr, str):
             expr = renpy.python.escape_unicode(expr)
 
         try:
@@ -237,10 +239,10 @@ class Parser(object):
         except SyntaxError as e:
             raise renpy.parser.ParseError(
                 filename,
-                lineno + e[1][1] - 1,
+                lineno + e.args[1][1] - 1,
                 "Syntax error while parsing python expression.",
-                e[1][3],
-                e[1][2])
+                e.args[1][3],
+                e.args[1][2])
 
         increment_lineno(rv, lineno-1)
 
@@ -252,7 +254,7 @@ class Parser(object):
         adjusts the line number. Returns a list of statements.
         """
 
-        if isinstance(code, unicode):
+        if isinstance(code, str):
             code = renpy.python.escape_unicode(code)
 
         try:
@@ -261,10 +263,10 @@ class Parser(object):
 
             raise renpy.parser.ParseError(
                 filename,
-                lineno + e[1][1] - 1,
+                lineno + e.args[1][1] - 1,
                 "Syntax error while parsing python code.",
-                e[1][3],
-                e[1][2])
+                e.args[1][3],
+                e.args[1][2])
 
         increment_lineno(rv, lineno-1)
 
@@ -445,7 +447,7 @@ class FunctionStatementParser(Parser):
                         rv.pop()
 
                         rv.extend(self.parse_exec("%s = (%s, %d)" % (child_name, name, child_index)))
-                        rv.extend(c)
+                        rv.extend(c) # type: ignore
 
                         needs_close = False
 
@@ -639,7 +641,7 @@ ui_properties = [
 
 
 def add(thing):
-    parser.add(thing)
+    parser.add(thing) # type: ignore
 
 
 ##############################################################################
@@ -716,6 +718,7 @@ Keyword("default")
 Keyword("length")
 Keyword("allow")
 Keyword("exclude")
+Keyword("copypaste")
 Keyword("prefix")
 Keyword("suffix")
 Keyword("changed")
@@ -880,6 +883,7 @@ Keyword("drag_name")
 Keyword("draggable")
 Keyword("droppable")
 Keyword("drag_raise")
+Keyword("dragging")
 Keyword("dragged")
 Keyword("dropped")
 Keyword("drop_allowable")
@@ -911,9 +915,6 @@ add(position_properties)
 
 class PassParser(Parser):
 
-    def __init__(self, name):
-        super(PassParser, self).__init__(name)
-
     def parse(self, l, name):
         return self.parse_exec("pass", l.number)
 
@@ -923,9 +924,6 @@ PassParser("pass")
 
 class DefaultParser(Parser):
 
-    def __init__(self, name):
-        super(DefaultParser, self).__init__(name)
-
     def parse(self, l, name):
 
         name = l.require(l.word)
@@ -956,23 +954,34 @@ class UseParser(Parser):
 
         args = renpy.parser.parse_arguments(l)
 
+        extrapos = None
+        extrakw = None
+
         if args:
 
+            index = 0
+
             for k, v in args.arguments:
-                if k is None:
+
+                if index in args.starred_indexes:
+                    extrapos = v
+                elif index in args.doublestarred_indexes:
+                    extrakw = v
+                elif k is None:
                     code += ", (%s)" % v
                 else:
                     code += ", %s=(%s)" % (k, v)
 
+                index += 1
+
         code += ", _name=%s, _scope=_scope" % name
 
         if args:
 
-            if args.extrapos:
-                code += ", *(%s)" % args.extrapos
-
-            if args.extrakw:
-                code += ", **(%s)" % args.extrakw
+            if extrapos:
+                code += ", *(%s)" % extrapos
+            if extrakw:
+                code += ", **(%s)" % extrakw
 
         code += ")"
 
@@ -1264,7 +1273,7 @@ class ScreenLangScreen(renpy.object.Object):
             values = renpy.ast.apply_arguments(self.parameters, args, kwargs)
             scope.update(values)
 
-        renpy.python.py_exec_bytecode(self.code.bytecode, locals=scope)
+        renpy.python.py_exec_bytecode(self.code.bytecode, locals=scope) # type: ignore
 
 
 class ScreenParser(Parser):
@@ -1303,7 +1312,7 @@ class ScreenParser(Parser):
         lineno = l.number
 
         screen.name = l.require(l.word)
-        screen.parameters = renpy.parser.parse_parameters(l)
+        screen.parameters = renpy.parser.parse_parameters(l) # type: ignore
 
         while parse_keyword(l):
             continue
@@ -1335,7 +1344,7 @@ class ScreenParser(Parser):
                 if c is None:
                     l.error('Expected a screen language statement.')
 
-                rv.extend(c)
+                rv.extend(c) # type: ignore
                 count += 1
 
         node = ast.Module(body=rv, lineno=lineno, col_offset=0)
@@ -1355,7 +1364,7 @@ class ScreenParser(Parser):
 #        print screen.name, "-----------------------------------------"
 #        unparse.Unparser(node)
 
-        screen.code = renpy.ast.PyCode(node, location, 'exec')
+        screen.code = renpy.ast.PyCode(node, location, 'exec') # type: ignore
 
         return screen
 
