@@ -1,4 +1,4 @@
-# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -22,7 +22,10 @@
 # This file contains support for string translation and string formatting
 # operations.
 
-from __future__ import print_function
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
+
+
 
 import renpy
 import string
@@ -58,7 +61,7 @@ class Formatter(string.Formatter):
         # The parts we've seen.
         literal = ''
         value = ''
-        format = ''  # @ReservedAssignment
+        format = '' # @ReservedAssignment
         conversion = None
 
         state = LITERAL
@@ -104,7 +107,7 @@ class Formatter(string.Formatter):
                         state = LITERAL
                         literal = ''
                         value = ''
-                        format = ''  # @ReservedAssignment
+                        format = '' # @ReservedAssignment
                         conversion = None
                         continue
 
@@ -128,7 +131,7 @@ class Formatter(string.Formatter):
                     state = LITERAL
                     literal = ''
                     value = ''
-                    format = ''  # @ReservedAssignment
+                    format = '' # @ReservedAssignment
                     conversion = None
                     continue
 
@@ -147,7 +150,7 @@ class Formatter(string.Formatter):
                     state = LITERAL
                     literal = ''
                     value = ''
-                    format = ''  # @ReservedAssignment
+                    format = '' # @ReservedAssignment
                     conversion = None
                     continue
 
@@ -161,26 +164,47 @@ class Formatter(string.Formatter):
         if literal:
             yield (literal, None, None, None)
 
+    def get_field(self, field_name, args, kwargs):
+        obj, arg_used = super(Formatter, self).get_field(field_name, args, kwargs)
+
+        return (obj, kwargs), arg_used
+
     def convert_field(self, value, conversion):
+        value, kwargs = value
+
+        if conversion is None:
+            return value
+
+        if not conversion:
+            raise ValueError("Conversion specifier can't be empty.")
+
+        if set(conversion) - set("rstqulci!"):
+            raise ValueError("Unknown symbols in conversion specifier, this must use only the \"rstqulci\".")
+
+        if "r" in conversion:
+            value = repr(value)
+            conversion = conversion.replace("r", "")
+        elif "s" in conversion:
+            value = str(value)
+            conversion = conversion.replace("s", "")
 
         if not conversion:
             return value
 
-        if "r" in conversion:
-            value = repr(value)
-        elif "s" in conversion:
-            value = unicode(value)
+        # All conversion symbols below assume we have a string.
+        if not isinstance(value, basestring):
+            value = str(value)
 
         if "t" in conversion:
-            if not isinstance(value, basestring):
-                value = unicode(value)
-
             value = renpy.translation.translate_string(value)
 
-        if "q" in conversion:
-            if not isinstance(value, basestring):
-                value = unicode(value)
+        if "i" in conversion:
+            try:
+                value = self.vformat(value, (), kwargs)
+            except RuntimeError: # PY3 RecursionError
+                raise ValueError("Substitution {!r} refers to itself in a loop.".format(value))
 
+        if "q" in conversion:
             value = value.replace("{", "{{")
 
         if "u" in conversion:
@@ -231,7 +255,7 @@ def substitute(s, scope=None, force=False, translate=True):
     """
 
     if not isinstance(s, basestring):
-        s = unicode(s)
+        s = str(s)
 
     if translate:
         s = renpy.translation.translate_string(s)
@@ -246,14 +270,15 @@ def substitute(s, scope=None, force=False, translate=True):
     old_s = s
 
     if scope is not None:
-        kwargs = MultipleDict(scope, renpy.store.__dict__)  # @UndefinedVariable
+        kwargs = MultipleDict(scope, renpy.store.__dict__) # @UndefinedVariable
     else:
-        kwargs = renpy.store.__dict__  # @UndefinedVariable
+        kwargs = renpy.store.__dict__ # @UndefinedVariable
 
     try:
-        s = formatter.vformat(s, (), kwargs)
-    except:
-        if renpy.display.predict.predicting:  # @UndefinedVariable
+        s = formatter.vformat(s, (), kwargs) # type: ignore
+    except Exception:
+        if renpy.display.predict.predicting: # @UndefinedVariable
             return " ", True
+        raise
 
     return s, (s != old_s)
