@@ -1,4 +1,4 @@
-# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -19,10 +19,15 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-from __future__ import print_function
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
+
+from typing import Optional, Callable
 
 import math
-import renpy.display
+
+import pygame_sdl2
+import renpy
 
 from renpy.text.textsupport import TAG, TEXT, PARAGRAPH, DISPLAYABLE
 
@@ -31,7 +36,7 @@ import renpy.text.texwrap as texwrap
 import renpy.text.font as font
 import renpy.text.extras as extras
 
-from _renpybidi import log2vis, WRTL, RTL, ON  # @UnresolvedImport
+from _renpybidi import log2vis, WRTL, RTL, ON # @UnresolvedImport
 
 BASELINE = -65536
 
@@ -134,7 +139,6 @@ def outline_blits(blits, outline):
 
     return rv
 
-
 class DrawInfo(object):
     """
     This object is supplied as a parameter to the draw method of the various
@@ -156,6 +160,10 @@ class DrawInfo(object):
 
     # No implementation, this is set up in the layout object.
 
+    surface = None # type: Optional[pygame_sdl2.surface.Surface]
+    override_color = None # type: Optional[tuple[int, int, int, int]]
+    outline = 0 # type: float
+    displayable_blits = None # type: Optional[list[tuple[renpy.display.core.Displayable, int, int]]]
 
 class TextSegment(object):
     """
@@ -188,12 +196,14 @@ class TextSegment(object):
             self.ruby_bottom = source.ruby_bottom
             self.hinting = source.hinting
             self.outline_color = source.outline_color
+            self.ignore = source.ignore
 
         else:
             self.hyperlink = 0
             self.cps = 0
             self.ruby_top = False
             self.ruby_bottom = False
+            self.ignore = False
 
     def __repr__(self):
         return "<TextSegment font={font}, size={size}, bold={bold}, italic={italic}, underline={underline}, color={color}, black_color={black_color}, hyperlink={hyperlink}, vertical={vertical}>".format(**self.__dict__)
@@ -239,6 +249,9 @@ class TextSegment(object):
         Return the list of glyphs corresponding to unicode string s.
         """
 
+        if self.ignore:
+            return [ ]
+
         fo = font.get_font(self.font, self.size, self.bold, self.italic, 0, self.antialias, self.vertical, self.hinting, layout.oversample)
         rv = fo.glyphs(s)
 
@@ -385,7 +398,7 @@ class SpaceSegment(object):
 
 class DisplayableSegment(object):
     """
-    A segment that's used to render horizontal or vertical whitespace.
+    A segment that's used to render displayables.
     """
 
     def __init__(self, ts, d, renders):
@@ -399,6 +412,9 @@ class DisplayableSegment(object):
 
         self.width, self.height = rend.get_size()
 
+        if isinstance(d, renpy.display.behavior.CaretBlink):
+            self.width = 0
+
         self.hyperlink = ts.hyperlink
         self.cps = ts.cps
         self.ruby_top = ts.ruby_top
@@ -406,6 +422,9 @@ class DisplayableSegment(object):
 
     def glyphs(self, s, layout):
 
+        if isinstance(self.d, renpy.display.layout.Null) and (self.d.width == 0) and (self.d.height == 0):
+            return [ ]
+
         glyph = textsupport.Glyph()
 
         w = layout.scale_int(self.width)
@@ -432,13 +451,19 @@ class DisplayableSegment(object):
         return rv
 
     def draw(self, glyphs, di, xo, yo, layout):
+        if not glyphs:
+            return
+
         glyph = glyphs[0]
 
         if di.displayable_blits is not None:
-
             di.displayable_blits.append((self.d, glyph.x, glyph.y, glyph.width, glyph.ascent, glyph.line_spacing, glyph.time))
 
     def assign_times(self, gt, glyphs):
+
+        if not glyphs:
+            return gt
+
         if self.cps != 0:
             gt += 1.0 / self.cps
 
@@ -504,7 +529,7 @@ class Layout(object):
         width = min(32767, width)
         height = min(32767, height)
 
-        if drawable_res and (not size_only) and renpy.config.drawable_resolution_text:
+        if drawable_res and (not size_only) and renpy.config.use_drawable_resolution and renpy.config.drawable_resolution_text:
             # How much do we want to oversample the text by, compared to the
             # virtual resolution.
             self.oversample = renpy.display.draw.draw_per_virt
@@ -513,7 +538,9 @@ class Layout(object):
             self.reverse = renpy.display.draw.draw_to_virt
             self.forward = renpy.display.draw.virt_to_draw
 
-            self.outline_step = text.style.outline_scaling != "linear"
+            self.outline_step = text.style.outline_scaling == "step"
+
+            self.pixel_perfect = True
 
         else:
 
@@ -522,6 +549,8 @@ class Layout(object):
             self.forward = renpy.display.render.IDENTITY
             self.outline_step = True
 
+            self.pixel_perfect = False
+
         style = text.style
 
         self.line_overlap_split = self.scale_int(style.line_overlap_split)
@@ -543,12 +572,13 @@ class Layout(object):
         # data from one Layout to another.
         self.paragraph_glyphs = [ ]
 
-        width = self.scale_int(width)
-        height = self.scale_int(height)
-
+        # The virtual width and height offered to this Layout.
         self.width = width
         self.height = height
 
+        width = self.scale_int(width)
+        height = self.scale_int(height)
+
         # Figure out outlines and other info.
         outlines, xborder, yborder, xoffset, yoffset = self.figure_outlines(style)
         self.outlines = outlines
@@ -641,7 +671,7 @@ class Layout(object):
             self.paragraph_glyphs.append(list(par_glyphs))
 
             if splits_from:
-                textsupport.copy_splits(splits_from.paragraph_glyphs[p_num], par_glyphs)  # @UndefinedVariable
+                textsupport.copy_splits(splits_from.paragraph_glyphs[p_num], par_glyphs) # @UndefinedVariable
 
             else:
 
@@ -661,6 +691,8 @@ class Layout(object):
                     textsupport.annotate_unicode(par_glyphs, False, 2)
                 elif language == "japanese-strict":
                     textsupport.annotate_unicode(par_glyphs, False, 3)
+                elif language == "anywhere":
+                    textsupport.annotate_anywhere(par_glyphs)
                 else:
                     raise Exception("Unknown language: {0}".format(language))
 
@@ -748,12 +780,12 @@ class Layout(object):
             elif adjust_spacing == "vertical":
                 target_x_delta = 0.0
 
-            textsupport.tweak_glyph_spacing(all_glyphs, lines, target_x_delta, target_y_delta, maxx, y)  # @UndefinedVariable
+            textsupport.tweak_glyph_spacing(all_glyphs, lines, target_x_delta, target_y_delta, maxx, y) # @UndefinedVariable
 
             maxx = target_x
             y = target_y
 
-            textsupport.offset_glyphs(all_glyphs, 0, int(round(splits_from.baseline * self.oversample)) - find_baseline())
+            textsupport.offset_glyphs(all_glyphs, 0, round(splits_from.baseline * self.oversample) - find_baseline())
 
         # Figure out the size of the texture. (This is a little over-sized,
         # but it simplifies the code to not have to care about borders on a
@@ -798,27 +830,57 @@ class Layout(object):
             if key in self.textures:
                 continue
 
-            # Create the texture.
-            surf = renpy.display.pgrender.surface((sw + o, sh + o), True)
-
-            di.surface = surf
-            di.override_color = color
-            di.outline = o
-
             if color == None:
                 self.displayable_blits = [ ]
                 di.displayable_blits = self.displayable_blits
             else:
                 di.displayable_blits = None
 
+            # Create the texture.
+
+            tw = int(sw + o)
+            th = int(sh + o)
+
+            # If not a multiple of 32, round up.
+            tw = (tw | 0x1f) + 1 if (tw & 0x1f) else tw
+            th = (th | 0x1f) + 1 if (th & 0x1f) else th
+
+            surf = renpy.display.pgrender.surface((tw, th), True)
+
+            if renpy.game.preferences.high_contrast:
+                if color:
+                    surf.fill(color)
+                else:
+                    prefix = style.prefix
+                    prefix_color = style.color
+                    style.set_prefix("idle_")
+                    idle_color = style.color
+                    style.set_prefix(prefix)
+
+                    color = (255, 255, 255, 255)
+
+                    if idle_color != prefix_color:
+
+                        if "hover" in prefix:
+                            color = (255, 255, 224, 255)
+                        elif "selected" in style.prefix:
+                            color = (224, 255, 255, 255)
+
+            di.surface = surf
+            di.override_color = color
+            di.outline = o
+
             for ts, glyphs in par_seg_glyphs:
                 if ts is self.end_segment:
                     break
 
                 ts.draw(glyphs, di, self.add_left, self.add_top, self)
 
+            if renpy.config.debug_text_alignment:
+                self.make_alignment_grid(surf)
+
             renpy.display.draw.mutated_surface(surf)
-            tex = renpy.display.draw.load_texture(surf)
+            tex = renpy.display.draw.load_texture(surf, properties={ "mipmap" : renpy.config.mipmap_text if (style.mipmap is None) else style.mipmap })
 
             self.textures[key] = tex
 
@@ -847,6 +909,22 @@ class Layout(object):
                 renpy.display.to_log.write("     Available: (%d, %d) Laid-out: (%d, %d)", width, height, sw, sh)
                 renpy.display.to_log.write("     Text: %r", text.text)
 
+    def make_alignment_grid(self, surf):
+        w, h = surf.get_size()
+
+        for x in range(w):
+            for y in range(h):
+
+                if surf.get_at((x, y))[3] > 0:
+                    continue
+
+                if x == 0 or y == 0 or x == w - 1 or y == h - 1:
+                    surf.set_at((x, y), (255, 0, 0, 255))
+                elif (x ^ y) & 1:
+                    surf.set_at((x, y), (255, 255, 255, 255))
+                else:
+                    surf.set_at((x, y), (0, 0, 0, 255))
+
     def scale(self, n):
         if n is None:
             return n
@@ -860,7 +938,7 @@ class Layout(object):
         if isinstance(n, renpy.display.core.absolute):
             return int(n)
 
-        return int(round(n * self.oversample))
+        return round(n * self.oversample)
 
     def scale_outline(self, n):
         if n is None:
@@ -874,7 +952,7 @@ class Layout(object):
             if self.oversample < 1:
                 return n
 
-            return n * int(self.oversample)
+            return int(n * int(self.oversample))
 
         else:
             if n == 0:
@@ -933,9 +1011,9 @@ class Layout(object):
                 if isinstance(i[0], (TextSegment, SpaceSegment, DisplayableSegment)):
                     return
 
-            line.extend(tss[-1].subsegment(u"\u200B"))
+            line.extend(tss[-1].subsegment(u"\u200B")) # type: ignore
 
-        for type, text in tokens:  # @ReservedAssignment
+        for type, text in tokens: # @ReservedAssignment
 
             try:
 
@@ -951,6 +1029,11 @@ class Layout(object):
                     continue
 
                 elif type == TEXT:
+
+                    if (text_displayable.mask is not None):
+                        if text != u"\u200b":
+                            text = text_displayable.mask * len(text)
+
                     line.extend(tss[-1].subsegment(text))
                     continue
 
@@ -966,6 +1049,12 @@ class Layout(object):
                     tss.pop()
 
                     if not tss:
+
+                        tag = tag[1:]
+
+                        if not renpy.text.extras.text_tags.get(tag, True):
+                            raise Exception("{/%s} isn't valid, since the %s text tag doesn't take a close tag." % (tag, tag))
+
                         raise Exception("%r closes a text tag that isn't open." % text)
 
                 elif tag == "_start":
@@ -1016,6 +1105,9 @@ class Layout(object):
                 elif tag == "fast":
                     pass
 
+                elif tag == "done":
+                    pass
+
                 elif tag == "nw":
                     pass
 
@@ -1172,13 +1264,20 @@ class Layout(object):
                     ts = push()
                     ts.vertical = False
 
+                elif tag == "alt":
+                    ts = push()
+                    ts.ignore = True
+
+                elif tag == "noalt":
+                    ts = push()
+
                 elif tag[0] == "#":
                     pass
 
                 else:
                     raise Exception("Unknown text tag %r" % text)
 
-            except:
+            except Exception:
                 renpy.game.exception_info = "While processing text tag {{{!s}}} in {!r}.:".format(text, text_displayable.get_all_text())
                 raise
 
@@ -1202,7 +1301,7 @@ class Layout(object):
         l = [ ]
 
         for ts, s in p:
-            s, direction = log2vis(s, direction)
+            s, direction = log2vis(unicode(s), direction)
             l.append((ts, s))
 
         rtl = (direction == RTL or direction == WRTL)
@@ -1221,7 +1320,7 @@ class Layout(object):
         style_outlines = style.outlines
         dslist = style.drop_shadow
 
-        if not style_outlines and not dslist:
+        if (not style_outlines) and (not dslist) and not renpy.game.preferences.high_contrast:
             return [ (0, None, 0, 0) ], 0, 0, 0, 0
 
         outlines = [ ]
@@ -1263,6 +1362,9 @@ class Layout(object):
 
         outlines.append((0, None, 0, 0))
 
+        if renpy.game.preferences.high_contrast:
+            outlines = [ (2, (0, 0, 0, 255), 0, 0), (0, None, 0, 0) ]
+
         return outlines, right - left, bottom - top, -left, -top
 
     def blits_typewriter(self, st):
@@ -1326,13 +1428,13 @@ class Layout(object):
             if g.x + g.advance > max_x:
                 max_x = g.x + g.advance
 
-            if g.x  < min_x:
+            if g.x < min_x:
                 min_x = g.x
 
         ly = min(l.y + l.height + self.line_overlap_split, max_height)
 
         if min_x < max_x:
-            rv.append(Blit(min_x, max_y, max_x - min_x, ly - max_y, left=left, right=right, top=top, bottom=(l is self.lines[-1]) ))
+            rv.append(Blit(min_x, max_y, max_x - min_x, ly - max_y, left=left, right=right, top=top, bottom=(l is self.lines[-1])))
 
         return rv
 
@@ -1341,17 +1443,10 @@ class Layout(object):
         Return the time of the first glyph that should be shown after st.
         """
 
-        for l in self.lines:
-            if not l.glyphs:
-                continue
-
-            if l.max_time > st:
-                break
-
-        else:
+        if st >= self.max_time:
             return None
-
-        return 0
+        else:
+            return 0
 
 
 # The maximum number of entries in the layout cache.
@@ -1401,8 +1496,8 @@ def text_tick():
     slow_text = [ ]
 
 
-VERT_REVERSE = renpy.display.render.Matrix2D(0, -1, 1, 0)
-VERT_FORWARD = renpy.display.render.Matrix2D(0, 1, -1, 0)
+VERT_REVERSE = renpy.display.matrix.Matrix2D(0, -1, 1, 0)
+VERT_FORWARD = renpy.display.matrix.Matrix2D(0, 1, -1, 0)
 
 
 class Text(renpy.display.core.Displayable):
@@ -1430,6 +1525,14 @@ class Text(renpy.display.core.Displayable):
     `substitute`
         If true, text interpolation occurs. If false, it will not occur. If
         None, they are controlled by :var:`config.new_substitutions`.
+
+    `slow_done`
+        If not None, and if slow text mode is enabled (see the `slow` parameter), this is a
+        function or callable which is called with no arguments when the text finishes displaying.
+
+    `**properties`
+        Like other Displayables, Text takes style properties, including (among many others) the
+        :propref:`mipmap` property.
     """
 
     __version__ = 4
@@ -1439,6 +1542,8 @@ class Text(renpy.display.core.Displayable):
     locked = False
 
     language = None
+    mask = None
+    last_ctc = None
 
     def after_upgrade(self, version):
 
@@ -1456,7 +1561,7 @@ class Text(renpy.display.core.Displayable):
             self.end = None
             self.dirty = True
 
-    def __init__(self, text, slow=None, scope=None, substitute=None, slow_done=None, replaces=None, **properties):
+    def __init__(self, text, slow=None, scope=None, substitute=None, slow_done=None, replaces=None, mask=None, **properties):
 
         super(Text, self).__init__(**properties)
 
@@ -1480,7 +1585,10 @@ class Text(renpy.display.core.Displayable):
         self.dirty = True
 
         # The text, after substitutions.
-        self.text = None
+        self.text = None # type: list|None
+
+        # A mask, for passwords and such.
+        self.mask = mask
 
         # Sets the text we're showing, and performs substitutions.
         self.set_text(text, scope, substitute)
@@ -1492,17 +1600,17 @@ class Text(renpy.display.core.Displayable):
         self.slow = slow
 
         # The callback to be called when slow-text mode ends.
-        self.slow_done = None
+        self.slow_done = slow_done # type:Callable|None
 
         # The ctc indicator associated with this text.
         self.ctc = None
 
         # The index of the start and end strings in the first segment of text.
         # (None to show the whole text.)
-        self.start = None
-        self.end = None
+        self.start = None # type: int|None
+        self.end = None # type: int|None
 
-        if replaces is not None:
+        if isinstance(replaces, Text):
             self.slow = replaces.slow
             self.slow_done = replaces.slow_done
             self.ctc = replaces.ctc
@@ -1544,19 +1652,18 @@ class Text(renpy.display.core.Displayable):
 
         return rv
 
-    def __unicode__(self):
+    def _repr_info(self):
         s = ""
 
         for i in self.text:
             if isinstance(i, basestring):
-                s += i
+                s += i # type: ignore
 
             if len(s) > 25:
                 s = s[:24] + u"\u2026"
                 break
 
-        s = s.replace("\\", "\\\\").replace("\"", "\\\"").replace("\n", "\\n")
-        return u"Text \"{}\"".format(s)
+        return repr(s)
 
     def get_all_text(self):
         """
@@ -1566,7 +1673,7 @@ class Text(renpy.display.core.Displayable):
 
         for i in self.text:
             if isinstance(i, basestring):
-                s += i
+                s += i # type: ignore
 
         return s
 
@@ -1605,10 +1712,8 @@ class Text(renpy.display.core.Displayable):
                     i, did_sub = renpy.substitutions.substitute(i, scope, substitute)
                     uses_scope = uses_scope or did_sub
 
-                if isinstance(i, str):
-                    i = unicode(i, "utf-8", "replace")
-                else:
-                    i = unicode(i)
+                if isinstance(i, bytes):
+                    i = str(i, "utf-8", "replace")
 
             new_text.append(i)
 
@@ -1640,6 +1745,10 @@ class Text(renpy.display.core.Displayable):
         self.ctc = ctc
         self.dirty = True
 
+    def set_last_ctc(self, last_ctc):
+        self.last_ctc = last_ctc
+        self.dirty = True
+
     def update(self):
         """
         This needs to be called after text has been updated, but before
@@ -1688,7 +1797,16 @@ class Text(renpy.display.core.Displayable):
         else:
             # Add the CTC.
             if self.ctc is not None:
-                text.append(self.ctc)
+                if isinstance(self.ctc, list):
+                    text.extend(self.ctc)
+                else:
+                    text.append(self.ctc)
+
+        if self.last_ctc is not None:
+            if isinstance(self.last_ctc, list):
+                text.extend(self.last_ctc)
+            else:
+                text.append(self.last_ctc)
 
         # Tokenize the text.
         tokens = self.tokenize(text)
@@ -1716,7 +1834,7 @@ class Text(renpy.display.core.Displayable):
         if self.dirty or self.displayables is None:
             self.update()
 
-        return list(self.displayables)
+        return list(self.displayables) # type: ignore
 
     def _tts(self):
 
@@ -1730,9 +1848,10 @@ class Text(renpy.display.core.Displayable):
             rv.append(i)
 
         rv = "".join(rv)
+        rv, _, _ = rv.partition("{done}")
         _, _, rv = rv.rpartition("{fast}")
 
-        rv = renpy.translation.dialogue.notags_filter(rv)
+        rv = renpy.text.extras.filter_alt_text(rv)
 
         alt = self.style.alt
 
@@ -1741,7 +1860,7 @@ class Text(renpy.display.core.Displayable):
 
         return rv
 
-    _tts_all = _tts
+    _tts_all = _tts # type: ignore
 
     def kill_layout(self):
         """
@@ -1889,7 +2008,6 @@ class Text(renpy.display.core.Displayable):
             for i in slow_text:
                 if i.slow:
                     i.call_slow_done(st)
-                    i.slow = False
 
             raise renpy.display.core.IgnoreEvent()
 
@@ -1898,11 +2016,6 @@ class Text(renpy.display.core.Displayable):
         if layout is None:
             return
 
-        if layout.redraw_typewriter(st) is None:
-            if self.slow:
-                self.call_slow_done(st)
-                self.slow = False
-
         for d, xo, yo in self.displayable_offsets:
             rv = d.event(ev, x - xo, y - yo, st)
             if rv is not None:
@@ -2135,9 +2248,9 @@ class Text(renpy.display.core.Displayable):
         # Figure out if we need to redraw or call slow_done.
         if self.slow:
             if redraw is not None:
-                renpy.display.render.redraw(self, redraw)
+                renpy.display.render.redraw(self, max(redraw, 0))
             else:
-                renpy.display.interface.timeout(0)
+                self.call_slow_done(st)
 
         rv.forward = layout.forward
         rv.reverse = layout.reverse
@@ -2149,6 +2262,9 @@ class Text(renpy.display.core.Displayable):
             vrv.blit(rv, (rv.height, 0))
             rv = vrv
 
+        if layout.pixel_perfect:
+            rv.properties = { "pixel_perfect" : True }
+
         return rv
 
     def tokenize(self, text):
@@ -2160,11 +2276,11 @@ class Text(renpy.display.core.Displayable):
 
         for i in text:
 
-            if isinstance(i, unicode):
+            if isinstance(i, str):
                 tokens.extend(textsupport.tokenize(i))
 
-            elif isinstance(i, str):
-                tokens.extend(textsupport.tokenize(unicode(i)))
+            elif isinstance(i, basestring):
+                tokens.extend(textsupport.tokenize(str(i)))
 
             elif isinstance(i, renpy.display.core.Displayable):
                 tokens.append((DISPLAYABLE, i))
@@ -2174,7 +2290,8 @@ class Text(renpy.display.core.Displayable):
 
         return tokens
 
-    def apply_custom_tags(self, tokens):
+    @staticmethod
+    def apply_custom_tags(tokens):
         """
         Apply new-style custom text tags.
         """
@@ -2187,7 +2304,7 @@ class Text(renpy.display.core.Displayable):
             kind, text = t
 
             if kind == TEXT and renpy.config.replace_text:
-                rv.append((TEXT, unicode(renpy.config.replace_text(text))))
+                rv.append((TEXT, str(renpy.config.replace_text(text))))
 
             elif kind != TAG:
                 rv.append(t)
@@ -2250,8 +2367,8 @@ class Text(renpy.display.core.Displayable):
                 new_tokens = [ ]
 
                 for kind2, text2 in new_contents:
-                    if isinstance(text2, str):
-                        text2 = unicode(text2)
+                    if isinstance(text2, bytes):
+                        text2 = str(text2)
 
                     new_tokens.append((kind2, text2))
 
