@@ -1,4 +1,4 @@
-# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -19,7 +19,10 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-from __future__ import absolute_import, division, print_function
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
+
+
 
 import renpy
 import json
@@ -36,13 +39,13 @@ def extract_strings_core(language, destination, merge=False, force=False):
 
     if merge:
         with open(destination, "r") as f:
-            result.update(json.load(f, "utf-8"))
+            result.update(json.load(f, encoding="utf-8"))
 
-    for k, v in st.translations.iteritems():
+    for k, v in st.translations.items():
         if v and v != k:
             result[k] = v
 
-    with open(destination, "wb") as f:
+    with open(destination, "w") as f:
         json.dump(result, f, ensure_ascii=True)
 
 
@@ -68,4 +71,5 @@ def extract_strings():
 
     return False
 
+
 renpy.arguments.register_command("extract_strings", extract_strings)
