@@ -1,4 +1,4 @@
-from __future__ import print_function
+from __future__ import print_function, unicode_literals
 
 import inspect
 import re
@@ -6,10 +6,14 @@ import collections
 import keyword
 import renpy.sl2
 import shutil
-import StringIO
+import io
 import os
+import textwrap
 
-import __builtin__
+try:
+    import builtins
+except ImportError:
+    import __builtin__ as builtins
 
 # Additional keywords in the Ren'Py script language.
 SCRIPT_KEYWORDS = """\
@@ -117,7 +121,7 @@ def sl2_regexps():
     style_part2 = "(?:" + "|".join(sorted(renpy.sl2.slparser.STYLE_PREFIXES)) + ")"
 
     items = list(groups.items())
-    items.sort(key=lambda a : ( tuple(sorted(a[1])), a[0][1] ) )
+    items.sort(key=lambda a : (tuple(sorted(a[1])), a[0][1]))
 
     for k, prefixes in items:
         names, style = k
@@ -165,8 +169,8 @@ def expanded_sl2_properties():
     return rv
 
 
-def write_keywords():
-    f = file("source/keywords.py", "w")
+def write_keywords(srcdir='source'):
+    outf = os.path.join(srcdir, 'keywords.py')
 
     kwlist = set(keyword.kwlist)
     kwlist |= script_keywords()
@@ -180,25 +184,24 @@ def write_keywords():
 
     kwlist.sort()
 
-    f.write("keywords = %r\n" % kwlist)
-    f.write("keyword_regex = %r\n" % ("|".join(re.escape(i) for i in kwlist)))
+    with open(outf, "w") as f:
 
-    properties = [ i for i in expanded_sl2_properties() if i not in kwlist ]
+        f.write("keywords = %r\n" % kwlist)
+        f.write("keyword_regex = %r\n" % ("|".join(re.escape(i) for i in kwlist)))
 
-    f.write("properties = %r\n" % properties)
+        properties = [ i for i in expanded_sl2_properties() if i not in kwlist ]
 
-    f.write("property_regexes = %r\n" % sl2_regexps())
+        f.write("properties = %r\n" % properties)
 
-    f.close()
+        f.write("property_regexes = %r\n" % sl2_regexps())
 
-    shutil.copy("source/keywords.py", "../tutorial/game/keywords.py")
+    shutil.copy(outf, os.path.join(srcdir, "../../tutorial/game/keywords.py"))
 
 
 # A map from filename to a list of lines that are supposed to go into
 # that file.
 line_buffer = collections.defaultdict(list)
 
-
 # A map from id(o) to the names it's documented under.
 documented = collections.defaultdict(list)
 
@@ -222,6 +225,13 @@ def scan(name, o, prefix=""):
     if not doc:
         return
 
+    if doc[0] == ' ':
+        print("Bad docstring for ", name, repr(doc))
+
+    if re.match(r'[\w\.]+\(', doc):
+        doc = doc.partition("\n\n")[2]
+        doc = textwrap.dedent(doc)
+
     # Break up the doc string, scan it for specials.
     lines = [ ]
 
@@ -268,7 +278,7 @@ def scan(name, o, prefix=""):
 
             try:
                 args = inspect.getargspec(init)
-            except:
+            except Exception:
                 args = None
 
         elif inspect.isfunction(o):
@@ -301,8 +311,12 @@ def scan(name, o, prefix=""):
     lb.append(prefix + "")
 
     if inspect.isclass(o):
-        for i in dir(o):
-            scan(i, getattr(o, i), prefix + "    ")
+        if (name not in [ "Matrix", "OffsetMatrix", "RotateMatrix", "ScaleMatrix" ]):
+            for i in dir(o):
+                scan(i, getattr(o, i), prefix + "    ")
+
+    if name == "identity":
+        raise Exception("identity")
 
     documented_list.append(o)
     documented[id(o)].append(name)
@@ -317,38 +331,37 @@ def scan_section(name, o):
         scan(name + n, getattr(o, n))
 
 
-def write_line_buffer():
+def write_line_buffer(incdir='source/inc'):
 
-    for k, v in line_buffer.iteritems():
+    for k, v in line_buffer.items():
 
-        # f = file("source/inc/" + k, "w")
+        f = io.StringIO()
 
-        f = StringIO.StringIO()
-
-        print(".. Automatically generated file - do not modify.", file=f)
-        print(file=f)
+        print(u".. Automatically generated file - do not modify.", file=f)
+        print(u"", file=f)
 
         for l in v:
             print(l, file=f)
 
         s = f.getvalue()
 
-        if os.path.exists("source/inc/" + k):
-            with open("source/inc/" + k) as f:
+        fname = os.path.join(incdir, k)
+        if os.path.exists(fname):
+            with open(fname) as f:
                 if f.read() == s:
                     print("Retaining", k)
                     continue
 
         print("Generating", k)
 
-        with open("source/inc/" + k, "w") as f:
+        with open(fname, "w") as f:
             f.write(s)
 
 
 name_kind = collections.defaultdict(str)
 
 
-def scan_docs():
+def scan_docs(srcdir='source', incdir='source/inc'):
     """
     Scans the documentation for functions, classes, and variables.
     """
@@ -364,12 +377,12 @@ def scan_docs():
 
             name_kind[m.group(2)] = m.group(1)
 
-    for i in os.listdir("source"):
+    for i in os.listdir(srcdir):
         if i.endswith(".rst"):
-            scan_file(os.path.join("source", i))
+            scan_file(os.path.join(srcdir, i))
 
-    for i in os.listdir("source/inc"):
-        scan_file(os.path.join("source", "inc", i))
+    for i in os.listdir(incdir):
+        scan_file(os.path.join(incdir, i))
 
 
 def format_name(name):
@@ -386,8 +399,6 @@ def format_name(name):
 
 def write_reserved(module, dest, ignore_builtins):
 
-    print("Writing", dest)
-
     with open(dest, "w") as f:
 
         for i in sorted(dir(module)):
@@ -398,13 +409,13 @@ def write_reserved(module, dest, ignore_builtins):
             if i.startswith("_"):
                 continue
 
-            if ignore_builtins and hasattr(__builtin__, i):
+            if ignore_builtins and hasattr(builtins, i):
                 continue
 
             f.write("* " + format_name(i) + "\n")
 
 
-def write_pure_const():
+def write_pure_const(incdir='source/inc'):
 
     def write_set(f, s):
         l = list(s)
@@ -413,19 +424,19 @@ def write_pure_const():
         for i in l:
             f.write("* " + format_name(i) + "\n")
 
-    pure = renpy.pyanalysis.pure_functions  # @UndefinedVariable
-    constants = renpy.pyanalysis.constants - pure  # @UndefinedVariable
+    pure = renpy.pyanalysis.pure_functions # @UndefinedVariable
+    constants = renpy.pyanalysis.constants - pure # @UndefinedVariable
 
-    with open("source/inc/pure_vars", "w") as f:
+    with open(os.path.join(incdir, "pure_vars"), "w") as f:
         write_set(f, pure)
 
-    with open("source/inc/const_vars", "w") as f:
+    with open(os.path.join(incdir, "const_vars"), "w") as f:
         write_set(f, constants)
 
 
-def write_easings(ns):
+def write_easings(ns, incdir='source/inc'):
 
-    with open("source/inc/easings", "w") as f:
+    with open(os.path.join(incdir, "easings"), "w") as f:
         f.write(".. csv-table::\n")
         f.write('    :header: "Ren\'Py Name", "easings.net Name"\n')
         f.write('\n')
@@ -440,22 +451,22 @@ def write_easings(ns):
             f.write('    "{}", "{}"\n'.format(name, stdname))
 
 
-def tq_script(name):
+def tq_script(name, srcdir='source'):
 
-    with open("../the_question/game/" + name, "r") as f:
+    with open(os.path.join(srcdir, "../../the_question/game", name), "r") as f:
         lines = f.readlines()
         lines = [ ("    " + i).rstrip() for i in lines ]
         return "\n".join(lines)
 
 
-def write_tq():
-    script = tq_script("script.rpy")
-    options = tq_script("options.rpy")
+def write_tq(srcdir='source'):
+    script = tq_script("script.rpy", srcdir=srcdir)
+    options = tq_script("options.rpy", srcdir=srcdir)
 
-    with open("source/thequestion.txt", "r") as f:
+    with open(os.path.join(srcdir, "thequestion.txt"), "r") as f:
         template = f.read()
 
-    with open("source/thequestion.rst", "w") as f:
+    with open(os.path.join(srcdir, "thequestion.rst"), "w") as f:
         f.write(template.format(script=script, options=options))
 
 
